// ============================================================================
// Copyright (c) 2015 by Terasic Inc.
// ============================================================================
//
// Permission:
//
// Terasic grants permission to use and modify this code for use
// in synthesis for all Terasic Development Boards and Altera Development
// Kits made by Terasic. Other use of this code, including the selling
// ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
// This VHDL/Verilog or C/C++ source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Terasic provides no warranty regarding the use
// or functionality of this code.
//
// ============================================================================
//
// Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
// HsinChu County, Taiwan
// 302
//
// web: http://www.terasic.com/
// email: support@terasic.com
//
// ============================================================================
// Major Functions:
//  This code is using for configuring the output frequency of 
//  SI570 I2C Programable XO/VCXO
// ============================================================================
// Design Description:
// 
//
//
// ===========================================================================
// Revision History :
// ============================================================================
// Ver  :| Author :| Mod. Date :| Changes Made:
// V1.0 :|        :| 2021/4/26 :| Initial Version
// ============================================================================
// XA(48MHz) --> DSPLLx free run(no clean) --> OUT2/6(7) --> FPGA XCVR bank
`define XCVR_644M53125     4'h0  
`define XCVR_625M          4'h1  
`define XCVR_322M265625    4'h2  
`define XCVR_312M5         4'h3  
`define XCVR_307M2         4'h4  
`define XCVR_250M          4'h5  
`define XCVR_184M32        4'h6 
`define XCVR_156M25        4'h7  // (default)
`define XCVR_153M6         4'h8  
`define XCVR_125M          4'h9  
`define XCVR_100M          4'hA  

// XA/INx --> PLL clean --> OUT0/4 --> FPGA IO bank
`define IO_50M             4'h0  // XA 48MHz --> DSPLL free-run 50MHZ --> OUT (default)
`define IO_IN0             4'h1  // IN0 (from FPGA, assume 30.72MHz)--> DSPLLx clean --> OUT
`define IO_IN1             4'h2  // IN1 (from FPGA, assume 31.25MHz)--> DSPLLx clean --> OUT  
`define IO_IN2             4'h3  // IN2 (30.72MHz)--> DSPLLx clean --> OUT  
`define IO_IN3             4'h4  // IN3 (from UFL, assume 10MHz)--> DSPLLx clean --> OUT 
 

module  DE10AGILEX_SI5397A_CONFIG ( 

input                   iCLK,        //50mhz
input                   iRST_n,
input                   iStart,        // rise edge trigger

input     [3:0]         iXCVR_CLK_0  ,// OUT2 frequency specified by constant XCVR_XXX. FPGA pin QSFPDDA_REFCLK_p 
input     [3:0]         iXCVR_CLK_1  ,// OUT6 frequency specified by constant XCVR_XXX. FPGA pin QSFPDDB_REFCLK_p 
input                   iXCVR_CLK_RSV,// OUT7 output control. 1: enable out, 0: disable output. FPGA pin QSFPDDRSV_REFCLK_p
input     [3:0]         iIO_CLK_0    ,// OUT0 frequency specified by constant IO_XXX. FPGA pin CLK_from_Si55397A[0]
input     [3:0]         iIO_CLK_1    ,// OUT4 frequency specified by constant IO_XXX. FPGA pin CLK_from_Si55397A[1]         
// si5340A conduit
inout			            I2C_CLK,
inout				         I2C_DATA,
output                  REG_CONFIG_DONE

);


//=============================================================================
// PARAMETER declarations
//=============================================================================



//=============================================================================
// REG/WIRE declarations
//=============================================================================

wire [6:0]  slave_addr;
wire [7:0]  byte_addr;
wire [7:0]  byte_data;
wire        wr_cmd;
wire [7:0]  oREAD_Data;
wire [3:0]  iFREQ_MODE;
wire        i2c_control_start;
wire 			i2c_reg_control_start;
wire 			i2c_bus_controller_state;
wire			iINITIAL_ENABLE;
wire 			system_start;
wire 			i2c_system_clk;
wire			i2c_controller_config_done;
wire			oController_Ready;
wire			initial_start;
wire 			i2c_read_data_rdy;
//=============================================================================
// Structural coding
//=============================================================================


si5397a_i2c_reg_controller si5397a_i2c_reg_controller(

.iCLK  (iCLK),
.iRST_n (iRST_n),
.iENABLE(system_start),

.XCVR0_REFCLK   (iXCVR_CLK_0  ), // XCVR_REF_XXX Si5397A  OUTPUT2
.XCVR1_REFCLK   (iXCVR_CLK_1  ), // XCVR_REF_XXX Si5397A  OUTPUT6
.XCVR_REFCLK_RSV(iXCVR_CLK_RSV), // XCVR_REF_XXX Si5397A  OUTPUT7
.IO0_CLK        (iIO_CLK_0    ), // OUT0
.IO1_CLK        (iIO_CLK_1    ), // OUT4


.iI2C_CONTROLLER_STATE      (i2c_bus_controller_state),
.iI2C_CONTROLLER_CONFIG_DONE(i2c_controller_config_done),
.oSLAVE_ADDR(slave_addr),
.oBYTE_ADDR(byte_addr),
.oBYTE_DATA(byte_data),
.oWR_CMD(wr_cmd),
.oStart    (i2c_reg_control_start),
.iI2C_READ_DATA(oREAD_Data),
.iI2C_READ_DATA_RDY  (i2c_read_data_rdy),
.oONE_CLK_CONFIG_DONE(REG_CONFIG_DONE),
.oController_Ready   (oController_Ready)
);


initial_config initial_config(

.iCLK(iCLK), // system   clock 50mhz 
.iRST_n(iRST_n), // system reset 
.oINITIAL_START(initial_start),
.iINITIAL_ENABLE(1'b1)
);


wire istart_rsing;

//assign system_start = iStart|initial_start;
//assign system_start = iStart;
assign system_start = istart_rsing | initial_start;


edge_detector edge_detector(

.iCLK         (iCLK),
.iRST_n       (iRST_n),
.iIn          (iStart),
.oFallING_EDGE(),
.oRISING_EDGE(istart_rsing)
);


clock_divider clock_divider(
.iCLK(iCLK),
.iRST_n(iRST_n),
.oCLK_OUT(i2c_system_clk)
);


i2c_bus_controller i2c_bus_controller	(

	.iCLK              (i2c_system_clk),
	.iRST_n            (iRST_n),
	.iStart            (i2c_reg_control_start),
	.iSlave_addr       (slave_addr),
	.iWord_addr        (byte_addr),
	.iSequential_read  (1'b0),
	.iRead_length      (8'd1),
	
	.i2c_clk           ( I2C_CLK),
	.i2c_data          ( I2C_DATA),
	.i2c_read_data     ( oREAD_Data),
	.i2c_read_data_rdy ( i2c_read_data_rdy),
	.wr_data           ( byte_data),
	.wr_cmd            ( wr_cmd),
   .oSYSTEM_STATE     ( i2c_bus_controller_state),
	.oCONFIG_DONE      ( i2c_controller_config_done)
				);


endmodule 
