// ============================================================================
// Copyright (c) 2021 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Mon Apr 26 16:39:33 2021
// ============================================================================

//`define ENABLE_DDR4A
//`define ENABLE_DDR4B
//`define ENABLE_DDR4C
//`define ENABLE_DDR4D
//`define ENABLE_PCIE
//`define ENABLE_QSFPDDA
//`define ENABLE_QSFPDDB
//`define ENABLE_QSFPRSV_REFCLK
//`define ENABLE_DDR4_DUAL_RANK

module DE10_Agilex_golden_top
   #(
`ifdef ENABLE_DDR4_DUAL_RANK
      parameter RANK = 1
`else
      parameter RANK = 0
`endif
   )
   (

      ///////// CLOCK /////////
      input              CLK_100_B2A,
      input              CLK_50_B3A,
      input              CLK_50_B3C,
      input              UFL_CLKIN,
      input              CLK_30M72, //reserved for future use
      input    [ 1: 0]   CLK_from_SI5397A_p,
      output   [ 1: 0]   CLK_to_SI5397A_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED_BRACKET, //LED is Low-Active
      output   [ 3: 0]   LED, //LED is Low-Active

`ifdef ENABLE_QSFPRSV_REFCLK
      ///////// QSFPRSV_REFCLK /////////
      input              QSFPDDRSV_REFCLK_p,
`endif /*ENABLE_QSFPRSV_REFCLK*/

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output   [RANK: 0] DDR4A_CK,
      output   [RANK: 0] DDR4A_CK_n,
      output   [RANK: 0] DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output   [RANK: 0] DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output   [RANK: 0] DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output   [RANK: 0] DDR4B_CK,
      output   [RANK: 0] DDR4B_CK_n,
      output   [RANK: 0] DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [RANK: 0] DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [RANK: 0] DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_EVENT_n,
      inout              DDR4B_SCL,
      inout              DDR4B_SDA,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_DDR4C
      ///////// DDR4C /////////
      input              DDR4C_REFCLK_p,
      output   [16: 0]   DDR4C_A,
      output   [ 1: 0]   DDR4C_BA,
      output   [ 1: 0]   DDR4C_BG,
      output   [RANK: 0] DDR4C_CK,
      output   [RANK: 0] DDR4C_CK_n,
      output   [RANK: 0] DDR4C_CKE,
      inout    [ 8: 0]   DDR4C_DQS,
      inout    [ 8: 0]   DDR4C_DQS_n,
      inout    [71: 0]   DDR4C_DQ,
      inout    [ 8: 0]   DDR4C_DBI_n,
      output   [RANK: 0] DDR4C_CS_n,
      output             DDR4C_RESET_n,
      output   [RANK: 0] DDR4C_ODT,
      output             DDR4C_PAR,
      input              DDR4C_ALERT_n,
      output             DDR4C_ACT_n,
      input              DDR4C_EVENT_n,
      inout              DDR4C_SCL,
      inout              DDR4C_SDA,
      input              DDR4C_RZQ,
`endif /*ENABLE_DDR4C*/

`ifdef ENABLE_DDR4D
      ///////// DDR4D /////////
      input              DDR4D_REFCLK_p,
      output   [16: 0]   DDR4D_A,
      output   [ 1: 0]   DDR4D_BA,
      output   [ 1: 0]   DDR4D_BG,
      output   [RANK: 0] DDR4D_CK,
      output   [RANK: 0] DDR4D_CK_n,
      output   [RANK: 0] DDR4D_CKE,
      inout    [ 8: 0]   DDR4D_DQS,
      inout    [ 8: 0]   DDR4D_DQS_n,
      inout    [71: 0]   DDR4D_DQ,
      inout    [ 8: 0]   DDR4D_DBI_n,
      output   [RANK: 0] DDR4D_CS_n,
      output             DDR4D_RESET_n,
      output   [RANK: 0] DDR4D_ODT,
      output             DDR4D_PAR,
      input              DDR4D_ALERT_n,
      output             DDR4D_ACT_n,
      input              DDR4D_EVENT_n,
      inout              DDR4D_SCL,
      inout              DDR4D_SDA,
      input              DDR4D_RZQ,
`endif /*ENABLE_DDR4D*/

      ///////// SI5397A /////////
      inout              SI5397A_I2C_SCL,
      inout              SI5397A_I2C_SDA,
      input              SI5397A_LOL_A_n,
      input              SI5397A_LOL_B_n,
      input              SI5397A_LOL_C_n,
      input              SI5397A_LOL_D_n,
      input              SI5397A_LOS_XAXB_n,
      output             SI5397A_OE_n,
      output             SI5397A_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_p,

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      output   [15: 0]   PCIE_TX_p,
      output   [15: 0]   PCIE_TX_n,
      input    [15: 0]   PCIE_RX_p,
      input    [15: 0]   PCIE_RX_n,
      input    [ 1: 0]   PCIE_REFCLK_p,
      input              PCIE_PERST_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFPDDA
      ///////// QSFPDDA /////////
      output   [ 7: 0]   QSFPDDA_TX_p,
      output   [ 7: 0]   QSFPDDA_TX_n,
      input    [ 7: 0]   QSFPDDA_RX_p,
      input    [ 7: 0]   QSFPDDA_RX_n,
      input              QSFPDDA_REFCLK_p,
      output             QSFPDDA_INITMODE,
      input              QSFPDDA_INTERRUPT_n,
      input              QSFPDDA_MOD_PRS_n,
      output             QSFPDDA_MOD_SEL_n,
      output             QSFPDDA_RST_n,
      inout              QSFPDDA_SCL,
      inout              QSFPDDA_SDA,
`else
      input              QSFPDDA_REFCLK_p,
`endif /*ENABLE_QSFPDDA*/

`ifdef ENABLE_QSFPDDB
      ///////// QSFPDDB /////////
      output   [ 7: 0]   QSFPDDB_TX_p,
      output   [ 7: 0]   QSFPDDB_TX_n,
      input    [ 7: 0]   QSFPDDB_RX_p,
      input    [ 7: 0]   QSFPDDB_RX_n,
      input              QSFPDDB_REFCLK_p,
      output             QSFPDDB_INITMODE,
      input              QSFPDDB_INTERRUPT_n,
      input              QSFPDDB_MOD_PRS_n,
      output             QSFPDDB_MOD_SEL_n,
      output             QSFPDDB_RST_n,
      inout              QSFPDDB_SCL,
      inout              QSFPDDB_SDA,
`endif /*ENABLE_QSFPDDB*/

      ///////// AG /////////
      output             AG_UART_TX,
      input              AG_UART_RX,
      input              AG_UART_CTS,
      output             AG_UART_RTS,

      ///////// EXP /////////
      input              EXP_EN,

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n

);


//=======================================================
//  REG/WIRE declarations
//=======================================================

wire				system_reset_n;
wire				pb_reset_n;
wire				led_hb;


//=======================================================
//  Structural coding
//=======================================================

assign SI5397A_OE_n = 1'b0;
assign SI5397A_RST_n = 1'b1;

gen_reset_n	system_gen_reset_n (
	.tx_clk(CLK_50_B3A),
	.reset_n_in(CPU_RESET_n),
	.reset_n_out(pb_reset_n)
);

assign system_reset_n = pb_reset_n;

heart_beat heart_beat(
	.clk(CLK_100_B2A),
	.led(led_hb)
);


assign LED[3:0] = system_reset_n ? {4{led_hb}} : 4'h00;

DE10AG_QSYS u0 (
	.clk_clk											(CLK_100_B2A),			//   input,	width = 1,            clk.clk
	.reset_n_reset_n								(system_reset_n),		//   input,	width = 1,          reset.reset

	.pio_led_export								(LED_BRACKET),			//  output,	width = 4,    pio_led.export
	.pio_button_export							(BUTTON),				//   input,	width = 2, pio_button.export
	.pio_sw_export									(SW)						//   input,	width = 2,     pio_sw.export
	
);

endmodule