// --------------------------------------------------------------------
// Copyright (c) 2012 by Terasic Technologies Inc.
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// --------------------------------------------------------------------
//
//                     Terasic Technologies Inc
//                     E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------

#include "terasic_includes.h"
#include "clock.h"
#include "si5397A_config.h"
#include "I2C_core.h"
#include <stdbool.h>

typedef bool (*LP_VERIFY_FUNC)(void);

bool TEST_Si5397A(void);
bool TEST_Board_Info(void);

typedef struct {
    LP_VERIFY_FUNC func;
    char szName[128];
} FUNC_INFO;

FUNC_INFO szFuncList[] = { { TEST_Si5397A, "Si5397A" },
                           { TEST_Board_Info, "Display Board Info" } };

void GUI_ShowMenu(void)
{
    int nNum, i;

    nNum = sizeof(szFuncList) / sizeof(szFuncList[0]);
    printf("======= Agilex Demo Program =======\n");
    for (i = 0; i < nNum; i++) {
        printf("[%d] %s\n", i, szFuncList[i].szName);
    }
    printf("Input your chioce:");
}

int GUI_QueryUser(void)
{
    int nChoice = 0;
    scanf("%d", &nChoice);
    printf("%d\n", nChoice);
    return nChoice;
}

//===============================================================
int main(void)
{
    int nChoice;
    int nNum;
    bool bPass;

    SI5397A_Config_Init(SI5397A_I2C_BASE);

    nNum = sizeof(szFuncList) / sizeof(szFuncList[0]);
    while (1) {
        GUI_ShowMenu();
        nChoice = GUI_QueryUser();
        if (nChoice >= 0 && nChoice < nNum) {
            bPass = szFuncList[nChoice].func();
            printf("%s Test:%s\n", szFuncList[nChoice].szName,
                   bPass ? "PASS" : "NG");
        }
    }

}

bool TEST_Si5397A(void)
{
    typedef struct {
        int FreqID;
        double FREQUENCY;
    } Si5397A_CONFIG;

    Si5397A_CONFIG config_list_xcvr[] = {
            { SI5397_XCVR_644M53125,  644.53125  },
            { SI5397_XCVR_625M,       625.0      },
            { SI5397_XCVR_322M265625, 322.265625 },
            { SI5397_XCVR_312M5,      312.5      },
            { SI5397_XCVR_307M2,      307.2      },
            { SI5397_XCVR_250M,       250.0      },
            { SI5397_XCVR_184M32,     184.32     },
            { SI5397_XCVR_156M25,     156.25     },
            { SI5397_XCVR_153M6 ,     153.6      },
            { SI5397_XCVR_125M,       125.0      },
            { SI5397_XCVR_100M,       100.0      } };
    Si5397A_CONFIG config_list_io[] = {
            { SI5397_IO_50M,          50.0       },
            { SI5397_IO_IN0,          30.72      },
            { SI5397_IO_IN1,          31.25      },
            { SI5397_IO_IN2,          30.72      },
            { SI5397_IO_IN3,          10.0       } };
    bool bPass, bThisPass;
    alt_u32 clk1, clk2, ScaledClk2;
    const int nTargetCnt = 10000;
    const int nTol = 3; //tolerance
    int nNum, i, nSel;
    const int config_num_xcvr = sizeof(config_list_xcvr) / sizeof(config_list_xcvr[0]);
    const int config_num_io = sizeof(config_list_io) / sizeof(config_list_io[0]);
    alt_u32 szBase[] = { QSFPDDA_REFCLK_BASE,
                         QSFPDDB_REFCLK_BASE,
                         QSFPDDRSV_REFCLK_BASE,
                         CLK_FROM_SI5397A_0_BASE,
                         CLK_FROM_SI5397A_1_BASE };
    const int channel_num_xcvr = 2;
    const int channel_num_xcvr_rsv = 1;
    const int channel_num_io = 2;
    alt_u8 szIndex[][32] = { "QSFPDDA_REFCLK", "QSFPDDB_REFCLK", "QSFPDDRSV_REFCLK", "CLK_from_SI5397A0", "CLK_from_SI5397A1" };
    int channel_frequency[] = { SI5397_XCVR_156M25, SI5397_XCVR_156M25, SI5397_XCVR_156M25, SI5397_IO_50M, SI5397_IO_50M};
    double FREQUENCY[] = { 156.25, 156.25, 156.25, 50, 50 };
    nNum = sizeof(szBase) / sizeof(szBase[0]);
    // show menu
    printf("===== Si5397A Programming =====\r\n");
    IOWR(SI5397A_OE_N_BASE, 0, 0x00);
    IOWR(SI5397A_RST_N_BASE, 0, 0x00);
    usleep(1000);
    IOWR(SI5397A_RST_N_BASE, 0, 0x01);
    for (i = 0; i < config_num_xcvr; i++) {
        printf("[%d] %.6f MHz\n", i, config_list_xcvr[i].FREQUENCY);
    }
    printf("[Other] exit\n");
    for (i = 0; i < channel_num_xcvr; i++) {
        printf("please select %s:", szIndex[i]);
        nSel = GUI_QueryUser();
        if (nSel < config_num_xcvr) {
            channel_frequency[i] = config_list_xcvr[nSel].FreqID;
            FREQUENCY[i] = config_list_xcvr[nSel].FREQUENCY;
        }
    }

    printf("[0] QSFPDDRSV_REFCLK Enable\n");
    printf("[1] QSFPDDRSV_REFCLK Disable\n");
    printf("[Other] exit\n");
    nSel = GUI_QueryUser();
    if (nSel < 2) {
        if (nSel == 0) {
            channel_frequency[2] = channel_frequency[0];
            FREQUENCY[2] = FREQUENCY[0];
        } else {
            channel_frequency[2] = SI5397_XCVR_NONE;
            FREQUENCY[2] = 0;
        }
    }

    printf("[0] XA 48MHz --> DSPLL free-run 50MHZ --> OUT\n");
    printf("[1] IN0 (from FPGA, assume 30.72MHz)--> DSPLLx clean --> OUT\n");
    printf("[2] IN1 (from FPGA, assume 31.25MHz)--> DSPLLx clean --> OUT\n");
    printf("[3] IN2 (30.72MHz)--> DSPLLx clean --> OUT\n");
    printf("[4] IN3 (from UFL, assume 10MHz)--> DSPLLx clean --> OUT\n");
    printf("[Other] exit\n");
    for (i = channel_num_xcvr + channel_num_xcvr_rsv;
            i < channel_num_xcvr + channel_num_xcvr_rsv + channel_num_io; i++) {
        printf("please select %s:", szIndex[i]);
        nSel = GUI_QueryUser();
        if (nSel < config_num_io) {
            channel_frequency[i] = config_list_io[nSel].FreqID;
            FREQUENCY[i] = config_list_io[nSel].FREQUENCY;
        }
    }

    bPass = SI5397A_Config(SI5397A_I2C_BASE, channel_frequency[3],
            channel_frequency[1], channel_frequency[4], channel_frequency[0], channel_frequency[2]);


    alt_u32 szClk1[32];
    alt_u32 szClk2[32];

    if (bPass) {
        for (i = 0; i < nNum; i++) {
            if (channel_frequency[i] == SI5397_XCVR_NONE) {
                continue;
            }
            usleep(200 * 1000);
            szClk1[i] = -1;
            szClk2[i] = -1;

            bThisPass = TRUE;
            if (CLOCK_Test(szBase[i], nTargetCnt + 2, &clk1, &clk2)) {
                szClk1[i] = clk1;
                szClk2[i] = clk2;
                ScaledClk2 = (int) ((float) clk2 * 50.0 / FREQUENCY[i]);
                if (abs(nTargetCnt - clk1) < nTol && abs(clk1 - ScaledClk2) < nTol) {
                    printf("%s PASS (clk1=%d, clk2=%d)\r\n", szIndex[i], (int) szClk1[i], (int) szClk2[i]);
                } else {
                    printf("%s NG (clk1=%d, clk2=%d)\r\n", szIndex[i], (int) szClk1[i], (int) szClk2[i]);
                    bThisPass = FALSE;
                }
            } else {
                bThisPass = FALSE;
                printf("Failed to read count of %s\r\n", szIndex[i]);
            }
            if (!bThisPass)
                bPass = FALSE;
        }
    }

    if (bPass) {
        bPass = SI5397A_Status(SI5397A_I2C_BASE);
    }

    return bPass;

}

bool TEST_Board_Info(void)
{
    uint16_t power_in_voltage, power_in_current;
    uint16_t core_voltage1, core_voltage2;
    float core_voltage1_float, core_voltage2_float;
    uint16_t core_current1, core_current2;
    float core_current1_float, core_current2_float;
    uint16_t fan_speed1, fan_speed2;
    uint16_t temperature_fpga, temperature_board1, temperature_board2;
    uint16_t temperature_sdm, temperature_e_tile, temperature_p_tile;
    uint16_t pin_status;

    float core_voltage, core_current;

    power_in_voltage = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_POWER_IN_VOLTAGE_BASE) & 0xFFFF;
    power_in_current = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_POWER_IN_CURRENT_BASE) & 0xFFFF;
    core_voltage1 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_CORE_VOLTAGE1_BASE) & 0xFFFF;
    core_current1 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_CORE_CURRENT1_BASE) & 0xFFFF;
    core_voltage2 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_CORE_VOLTAGE2_BASE) & 0xFFFF;
    core_current2 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_CORE_CURRENT2_BASE) & 0xFFFF;
    fan_speed1 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_FAN_SPEED1_BASE) & 0xFFFF;
    fan_speed2 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_FAN_SPEED2_BASE) & 0xFFFF;
    temperature_fpga = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_FPGA_BASE) & 0xFFFF;
    temperature_board1 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_BOARD1_BASE) & 0xFFFF;
    temperature_board2 = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_BOARD2_BASE) & 0xFFFF;
    temperature_sdm = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_SDM_BASE) & 0xFFFF;
    temperature_e_tile = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_E_TILE_BASE) & 0xFFFF;
    temperature_p_tile = IORD_ALTERA_AVALON_PIO_DATA(BOARD_INFO_TEMPERATURE_P_TILE_BASE) & 0xFFFF;

    core_voltage1_float = core_voltage1 / 1000.0;
    core_voltage2_float = core_voltage2 / 1000.0;
    core_voltage = (core_voltage1_float + core_voltage2_float) / 2.0;
    core_current1_float = core_current1 / 1000.0;
    core_current2_float = core_current2 / 1000.0;
    core_current = core_current1_float + core_current2_float;

    printf("==== Temperature ====\n");
    printf("\tFPGA: %d*C\n", temperature_fpga);
    printf("\tBoard 1: %d*C\n", temperature_board1);
    printf("\tBoard 2: %d*C\n", temperature_board2);
    printf("\tSDM: %d*C\n", temperature_sdm);
    printf("\tE-Tile: %d*C\n", temperature_e_tile);
    printf("\tP-Tile: %d*C\n", temperature_p_tile);
    printf("\n");
    printf("==== Fan ====\n");
    printf("\tFan 1 RPM: %d\n", fan_speed1);
    printf("\tFan 2 RPM: %d\n", fan_speed2);
    printf("\n");
    printf("==== Power (12V) Monitor ====\n");
    printf("\tVoltage       = %.3f V \n", power_in_voltage / 1000.0);
    printf("\tCurrent       = %.3f A \n", power_in_current / 1000.0);
    printf("\tPower         = %.3f W \n", power_in_voltage / 1000.0 * power_in_current / 1000.0);
    printf("==== Core Power Monitor ====\n");
    printf("\tVoltage       = %.3f V (%.3f + %.3f) / 2 \n", core_voltage, core_voltage1_float, core_voltage2_float);
    printf("\tCurrent       = %.3f A (%.3f + %.3f) \n", core_current, core_current1_float, core_current2_float);
    printf("\tPower         = %.3f W \n", core_voltage * core_current);

    return true;
}
