// ============================================================================
// Copyright (c) 2020 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// ============================================================================
//
//  Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
//
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Wed Aug  5 11:13:42 2020
// ============================================================================

`define ENABLE_DDR4A
`define ENABLE_DDR4B
`define ENABLE_DDR4C
`define ENABLE_DDR4D
//`define ENABLE_PCIE
//`define ENABLE_QSFPDDA
//`define ENABLE_QSFPDDB
//`define ENABLE_QSFPRSV_REFCLK
//`define ENABLE_DDR4_DUAL_RANK

module DE10_Agilex_golden_top
   #(
`ifdef ENABLE_DDR4_DUAL_RANK
      parameter RANK = 1
`else
      parameter RANK = 0
`endif
   )
   (

      ///////// CLOCK /////////
      input              CLK_100_B2A,
      input              CLK_50_B3A,
      input              CLK_50_B3C,
      input              UFL_CLKIN,
      input              CLK_30M72, //reserved for future use
      input    [ 1: 0]   CLK_from_SI5397A_p,
      output   [ 1: 0]   CLK_to_SI5397A_p,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED_BRACKET, //LED is Low-Active
      output   [ 3: 0]   LED, //LED is Low-Active

`ifdef ENABLE_QSFPRSV_REFCLK
      ///////// QSFPRSV_REFCLK /////////
      input              QSFPDDRSV_REFCLK_p,
`endif /*ENABLE_QSFPRSV_REFCLK*/

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output   [RANK: 0] DDR4A_CK,
      output   [RANK: 0] DDR4A_CK_n,
      output   [RANK: 0] DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output   [RANK: 0] DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output   [RANK: 0] DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output   [RANK: 0] DDR4B_CK,
      output   [RANK: 0] DDR4B_CK_n,
      output   [RANK: 0] DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [RANK: 0] DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [RANK: 0] DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_EVENT_n,
      inout              DDR4B_SCL,
      inout              DDR4B_SDA,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

`ifdef ENABLE_DDR4C
      ///////// DDR4C /////////
      input              DDR4C_REFCLK_p,
      output   [16: 0]   DDR4C_A,
      output   [ 1: 0]   DDR4C_BA,
      output   [ 1: 0]   DDR4C_BG,
      output   [RANK: 0] DDR4C_CK,
      output   [RANK: 0] DDR4C_CK_n,
      output   [RANK: 0] DDR4C_CKE,
      inout    [ 8: 0]   DDR4C_DQS,
      inout    [ 8: 0]   DDR4C_DQS_n,
      inout    [71: 0]   DDR4C_DQ,
      inout    [ 8: 0]   DDR4C_DBI_n,
      output   [RANK: 0] DDR4C_CS_n,
      output             DDR4C_RESET_n,
      output   [RANK: 0] DDR4C_ODT,
      output             DDR4C_PAR,
      input              DDR4C_ALERT_n,
      output             DDR4C_ACT_n,
      input              DDR4C_EVENT_n,
      inout              DDR4C_SCL,
      inout              DDR4C_SDA,
      input              DDR4C_RZQ,
`endif /*ENABLE_DDR4C*/

`ifdef ENABLE_DDR4D
      ///////// DDR4D /////////
      input              DDR4D_REFCLK_p,
      output   [16: 0]   DDR4D_A,
      output   [ 1: 0]   DDR4D_BA,
      output   [ 1: 0]   DDR4D_BG,
      output   [RANK: 0] DDR4D_CK,
      output   [RANK: 0] DDR4D_CK_n,
      output   [RANK: 0] DDR4D_CKE,
      inout    [ 8: 0]   DDR4D_DQS,
      inout    [ 8: 0]   DDR4D_DQS_n,
      inout    [71: 0]   DDR4D_DQ,
      inout    [ 8: 0]   DDR4D_DBI_n,
      output   [RANK: 0] DDR4D_CS_n,
      output             DDR4D_RESET_n,
      output   [RANK: 0] DDR4D_ODT,
      output             DDR4D_PAR,
      input              DDR4D_ALERT_n,
      output             DDR4D_ACT_n,
      input              DDR4D_EVENT_n,
      inout              DDR4D_SCL,
      inout              DDR4D_SDA,
      input              DDR4D_RZQ,
`endif /*ENABLE_DDR4D*/

      ///////// SI5397A /////////
      inout              SI5397A_I2C_SCL,
      inout              SI5397A_I2C_SDA,
      input              SI5397A_LOL_A_n,
      input              SI5397A_LOL_B_n,
      input              SI5397A_LOL_C_n,
      input              SI5397A_LOL_D_n,
      input              SI5397A_LOS_XAXB_n,
      output             SI5397A_OE_n,
      output             SI5397A_RST_n,

      ///////// GPIO /////////
      input    [ 1: 0]   GPIO_CLK,
      inout    [ 3: 0]   GPIO_p,

`ifdef ENABLE_PCIE
      ///////// PCIE /////////
      output   [15: 0]   PCIE_TX_p,
      output   [15: 0]   PCIE_TX_n,
      input    [15: 0]   PCIE_RX_p,
      input    [15: 0]   PCIE_RX_n,
      input    [ 1: 0]   PCIE_REFCLK_p,
      input              PCIE_PERST_n,
`endif /*ENABLE_PCIE*/

`ifdef ENABLE_QSFPDDA
      ///////// QSFPDDA /////////
      output   [ 7: 0]   QSFPDDA_TX_p,
      output   [ 7: 0]   QSFPDDA_TX_n,
      input    [ 7: 0]   QSFPDDA_RX_p,
      input    [ 7: 0]   QSFPDDA_RX_n,
      input              QSFPDDA_REFCLK_p,
      output             QSFPDDA_INITMODE,
      input              QSFPDDA_INTERRUPT_n,
      input              QSFPDDA_MOD_PRS_n,
      output             QSFPDDA_MOD_SEL_n,
      output             QSFPDDA_RST_n,
      inout              QSFPDDA_SCL,
      inout              QSFPDDA_SDA,
`else
      input              QSFPDDA_REFCLK_p,
`endif /*ENABLE_QSFPDDA*/

`ifdef ENABLE_QSFPDDB
      ///////// QSFPDDB /////////
      output   [ 7: 0]   QSFPDDB_TX_p,
      output   [ 7: 0]   QSFPDDB_TX_n,
      input    [ 7: 0]   QSFPDDB_RX_p,
      input    [ 7: 0]   QSFPDDB_RX_n,
      input              QSFPDDB_REFCLK_p,
      output             QSFPDDB_INITMODE,
      input              QSFPDDB_INTERRUPT_n,
      input              QSFPDDB_MOD_PRS_n,
      output             QSFPDDB_MOD_SEL_n,
      output             QSFPDDB_RST_n,
      inout              QSFPDDB_SCL,
      inout              QSFPDDB_SDA,
`endif /*ENABLE_QSFPDDB*/

      ///////// AG /////////
      output             AG_UART_TX,
      input              AG_UART_RX,
      input              AG_UART_CTS,
      output             AG_UART_RTS,

      ///////// EXP /////////
      input              EXP_EN,

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n

);




//=======================================================
//  REG/WIRE declarations
//=======================================================

wire ddr4_local_reset_req;
//ddr4a
wire ddr4_a_local_reset_done;
wire ddr4_a_status_local_cal_fail;
wire ddr4_a_status_local_cal_success;
//ddr4b
wire ddr4_b_local_reset_done;
wire ddr4_b_status_local_cal_fail;
wire ddr4_b_status_local_cal_success;
//ddr4c
wire ddr4_c_local_reset_done;
wire ddr4_c_status_local_cal_fail;
wire ddr4_c_status_local_cal_success;
//ddr4d
wire ddr4_d_local_reset_done;
wire ddr4_d_status_local_cal_fail;
wire ddr4_d_status_local_cal_success;


wire [11:0] ddr4_status;


//=======================================================
//  Structural coding
//=======================================================



assign ddr4_status = {ddr4_a_status_local_cal_fail, ddr4_a_status_local_cal_success, ddr4_a_local_reset_done,
                      ddr4_b_status_local_cal_fail, ddr4_b_status_local_cal_success, ddr4_b_local_reset_done,
							 ddr4_c_status_local_cal_fail, ddr4_c_status_local_cal_success, ddr4_c_local_reset_done,
							 ddr4_d_status_local_cal_fail, ddr4_d_status_local_cal_success, ddr4_d_local_reset_done
							 };

assign reset_n = !ninit_done;
wire ninit_done;
reset_release reset_release (
	.ninit_done(ninit_done)
	);


//=======================================================
//  Structural coding
//=======================================================

assign SI5340A_OE_n = 1'b0;
assign SI5340A_RST_n = 1'b1;


    Qsys u0 (
        .button_external_connection_export                 (BUTTON),                 //   input,   width = 2,               button_external_connection.export
        .clk_clk                                           (CLK_50_B3C),                                           //   input,   width = 1,                                      clk.clk
        .ddr4_local_reset_req_external_connection_export   (ddr4_local_reset_req),   //  output,   width = 1, ddr4_local_reset_req_external_connection.export
        .ddr4_status_external_connection_export            (ddr4_status),            //   input,  width = 12,          ddr4_status_external_connection.export
        
   		  
		  .emif_agilex_a_local_reset_req_local_reset_req     (ddr4_local_reset_req),     //   input,   width = 1,            emif_agilex_a_local_reset_req.local_reset_req
        .emif_agilex_a_local_reset_status_local_reset_done (ddr4_a_local_reset_done), //  output,   width = 1,         emif_agilex_a_local_reset_status.local_reset_done
        .emif_agilex_a_pll_ref_clk_clk                     (DDR4A_REFCLK_p),                     //   input,   width = 1,                emif_agilex_a_pll_ref_clk.clk
        .emif_agilex_a_oct_oct_rzqin                       (DDR4A_RZQ),                       //   input,   width = 1,                        emif_agilex_a_oct.oct_rzqin
        .emif_agilex_a_mem_mem_ck                          (DDR4A_CK),                          //  output,   width = 1,                        emif_agilex_a_mem.mem_ck
        .emif_agilex_a_mem_mem_ck_n                        (DDR4A_CK_n),                        //  output,   width = 1,                                         .mem_ck_n
        .emif_agilex_a_mem_mem_a                           (DDR4A_A),                           //  output,  width = 17,                                         .mem_a
        .emif_agilex_a_mem_mem_act_n                       (DDR4A_ACT_n),                       //  output,   width = 1,                                         .mem_act_n
        .emif_agilex_a_mem_mem_ba                          (DDR4A_BA),                          //  output,   width = 2,                                         .mem_ba
        .emif_agilex_a_mem_mem_bg                          (DDR4A_BG),                          //  output,   width = 2,                                         .mem_bg
        .emif_agilex_a_mem_mem_cke                         (DDR4A_CKE),                         //  output,   width = 1,                                         .mem_cke
        .emif_agilex_a_mem_mem_cs_n                        (DDR4A_CS_n),                        //  output,   width = 1,                                         .mem_cs_n
        .emif_agilex_a_mem_mem_odt                         (DDR4A_ODT),                         //  output,   width = 1,                                         .mem_odt
        .emif_agilex_a_mem_mem_reset_n                     (DDR4A_RESET_n),                     //  output,   width = 1,                                         .mem_reset_n
        .emif_agilex_a_mem_mem_par                         (DDR4A_PAR),                         //  output,   width = 1,                                         .mem_par
        .emif_agilex_a_mem_mem_alert_n                     (DDR4A_ALERT_n),                     //   input,   width = 1,                                         .mem_alert_n
        .emif_agilex_a_mem_mem_dqs                         (DDR4A_DQS),                         //   inout,   width = 9,                                         .mem_dqs
        .emif_agilex_a_mem_mem_dqs_n                       (DDR4A_DQS_n),                       //   inout,   width = 9,                                         .mem_dqs_n
        .emif_agilex_a_mem_mem_dq                          (DDR4A_DQ),                          //   inout,  width = 72,                                         .mem_dq
        .emif_agilex_a_mem_mem_dbi_n                       (DDR4A_DBI_n),                       //   inout,   width = 9,                                         .mem_dbi_n
        .emif_agilex_a_status_local_cal_success            (ddr4_a_status_local_cal_success),            //  output,   width = 1,                     emif_agilex_a_status.local_cal_success
        .emif_agilex_a_status_local_cal_fail               (ddr4_a_status_local_cal_fail),               //  output,   width = 1,                                         .local_cal_fail
         
		  
		  .emif_agilex_b_local_reset_req_local_reset_req     (ddr4_local_reset_req),     //   input,   width = 1,            emif_agilex_b_local_reset_req.local_reset_req
        .emif_agilex_b_local_reset_status_local_reset_done (ddr4_b_local_reset_done), //  output,   width = 1,         emif_agilex_b_local_reset_status.local_reset_done
        .emif_agilex_b_pll_ref_clk_clk                     (DDR4B_REFCLK_p),                     //   input,   width = 1,                emif_agilex_b_pll_ref_clk.clk
        .emif_agilex_b_oct_oct_rzqin                       (DDR4B_RZQ),                       //   input,   width = 1,                        emif_agilex_b_oct.oct_rzqin
        .emif_agilex_b_mem_mem_ck                          (DDR4B_CK),                          //  output,   width = 1,                        emif_agilex_b_mem.mem_ck
        .emif_agilex_b_mem_mem_ck_n                        (DDR4B_CK_n),                        //  output,   width = 1,                                         .mem_ck_n
        .emif_agilex_b_mem_mem_a                           (DDR4B_A),                           //  output,  width = 17,                                         .mem_a
        .emif_agilex_b_mem_mem_act_n                       (DDR4B_ACT_n),                       //  output,   width = 1,                                         .mem_act_n
        .emif_agilex_b_mem_mem_ba                          (DDR4B_BA),                          //  output,   width = 2,                                         .mem_ba
        .emif_agilex_b_mem_mem_bg                          (DDR4B_BG),                          //  output,   width = 2,                                         .mem_bg
        .emif_agilex_b_mem_mem_cke                         (DDR4B_CKE),                         //  output,   width = 1,                                         .mem_cke
        .emif_agilex_b_mem_mem_cs_n                        (DDR4B_CS_n),                        //  output,   width = 1,                                         .mem_cs_n
        .emif_agilex_b_mem_mem_odt                         (DDR4B_ODT),                         //  output,   width = 1,                                         .mem_odt
        .emif_agilex_b_mem_mem_reset_n                     (DDR4B_RESET_n),                     //  output,   width = 1,                                         .mem_reset_n
        .emif_agilex_b_mem_mem_par                         (DDR4B_PAR),                         //  output,   width = 1,                                         .mem_par
        .emif_agilex_b_mem_mem_alert_n                     (DDR4B_ALERT_n),                     //   input,   width = 1,                                         .mem_alert_n
        .emif_agilex_b_mem_mem_dqs                         (DDR4B_DQS),                         //   inout,   width = 9,                                         .mem_dqs
        .emif_agilex_b_mem_mem_dqs_n                       (DDR4B_DQS_n),                       //   inout,   width = 9,                                         .mem_dqs_n
        .emif_agilex_b_mem_mem_dq                          (DDR4B_DQ),                          //   inout,  width = 72,                                         .mem_dq
        .emif_agilex_b_mem_mem_dbi_n                       (DDR4B_DBI_n),                       //   inout,   width = 9,                                         .mem_dbi_n
        .emif_agilex_b_status_local_cal_success            (ddr4_b_status_local_cal_success),            //  output,   width = 1,                     emif_agilex_b_status.local_cal_success
        .emif_agilex_b_status_local_cal_fail               (ddr4_b_status_local_cal_fail),               //  output,   width = 1,                                         .local_cal_fail
        
	
		  .emif_agilex_c_local_reset_req_local_reset_req     (ddr4_local_reset_req),     //   input,   width = 1,            emif_agilex_c_local_reset_req.local_reset_req
        .emif_agilex_c_local_reset_status_local_reset_done (ddr4_c_local_reset_done), //  output,   width = 1,         emif_agilex_c_local_reset_status.local_reset_done
        .emif_agilex_c_pll_ref_clk_clk                     (DDR4C_REFCLK_p),                     //   input,   width = 1,                emif_agilex_c_pll_ref_clk.clk
        .emif_agilex_c_oct_oct_rzqin                       (DDR4C_RZQ),                       //   input,   width = 1,                        emif_agilex_c_oct.oct_rzqin
        .emif_agilex_c_mem_mem_ck                          (DDR4C_CK),                          //  output,   width = 1,                        emif_agilex_c_mem.mem_ck
        .emif_agilex_c_mem_mem_ck_n                        (DDR4C_CK_n),                        //  output,   width = 1,                                         .mem_ck_n
        .emif_agilex_c_mem_mem_a                           (DDR4C_A),                           //  output,  width = 17,                                         .mem_a
        .emif_agilex_c_mem_mem_act_n                       (DDR4C_ACT_n),                       //  output,   width = 1,                                         .mem_act_n
        .emif_agilex_c_mem_mem_ba                          (DDR4C_BA),                          //  output,   width = 2,                                         .mem_ba
        .emif_agilex_c_mem_mem_bg                          (DDR4C_BG),                          //  output,   width = 2,                                         .mem_bg
        .emif_agilex_c_mem_mem_cke                         (DDR4C_CKE),                         //  output,   width = 1,                                         .mem_cke
        .emif_agilex_c_mem_mem_cs_n                        (DDR4C_CS_n),                        //  output,   width = 1,                                         .mem_cs_n
        .emif_agilex_c_mem_mem_odt                         (DDR4C_ODT),                         //  output,   width = 1,                                         .mem_odt
        .emif_agilex_c_mem_mem_reset_n                     (DDR4C_RESET_n),                     //  output,   width = 1,                                         .mem_reset_n
        .emif_agilex_c_mem_mem_par                         (DDR4C_PAR),                         //  output,   width = 1,                                         .mem_par
        .emif_agilex_c_mem_mem_alert_n                     (DDR4C_ALERT_n),                     //   input,   width = 1,                                         .mem_alert_n
        .emif_agilex_c_mem_mem_dqs                         (DDR4C_DQS),                         //   inout,   width = 9,                                         .mem_dqs
        .emif_agilex_c_mem_mem_dqs_n                       (DDR4C_DQS_n),                       //   inout,   width = 9,                                         .mem_dqs_n
        .emif_agilex_c_mem_mem_dq                          (DDR4C_DQ),                          //   inout,  width = 72,                                         .mem_dq
        .emif_agilex_c_mem_mem_dbi_n                       (DDR4C_DBI_n),                       //   inout,   width = 9,                                         .mem_dbi_n
        .emif_agilex_c_status_local_cal_success            (ddr4_c_status_local_cal_success),            //  output,   width = 1,                     emif_agilex_c_status.local_cal_success
        .emif_agilex_c_status_local_cal_fail               (ddr4_c_status_local_cal_fail),               //  output,   width = 1,                                         .local_cal_fail
        

		  .emif_agilex_d_local_reset_req_local_reset_req     (ddr4_local_reset_req),     //   input,   width = 1,            emif_agilex_d_local_reset_req.local_reset_req
        .emif_agilex_d_local_reset_status_local_reset_done (ddr4_d_local_reset_done), //  output,   width = 1,         emif_agilex_d_local_reset_status.local_reset_done
        .emif_agilex_d_pll_ref_clk_clk                     (DDR4D_REFCLK_p),                     //   input,   width = 1,                emif_agilex_d_pll_ref_clk.clk
        .emif_agilex_d_oct_oct_rzqin                       (DDR4D_RZQ),                       //   input,   width = 1,                        emif_agilex_d_oct.oct_rzqin
        .emif_agilex_d_mem_mem_ck                          (DDR4D_CK),                          //  output,   width = 1,                        emif_agilex_d_mem.mem_ck
        .emif_agilex_d_mem_mem_ck_n                        (DDR4D_CK_n),                        //  output,   width = 1,                                         .mem_ck_n
        .emif_agilex_d_mem_mem_a                           (DDR4D_A),                           //  output,  width = 17,                                         .mem_a
        .emif_agilex_d_mem_mem_act_n                       (DDR4D_ACT_n),                       //  output,   width = 1,                                         .mem_act_n
        .emif_agilex_d_mem_mem_ba                          (DDR4D_BA),                          //  output,   width = 2,                                         .mem_ba
        .emif_agilex_d_mem_mem_bg                          (DDR4D_BG),                          //  output,   width = 2,                                         .mem_bg
        .emif_agilex_d_mem_mem_cke                         (DDR4D_CKE),                         //  output,   width = 1,                                         .mem_cke
        .emif_agilex_d_mem_mem_cs_n                        (DDR4D_CS_n),                        //  output,   width = 1,                                         .mem_cs_n
        .emif_agilex_d_mem_mem_odt                         (DDR4D_ODT),                         //  output,   width = 1,                                         .mem_odt
        .emif_agilex_d_mem_mem_reset_n                     (DDR4D_RESET_n),                     //  output,   width = 1,                                         .mem_reset_n
        .emif_agilex_d_mem_mem_par                         (DDR4D_PAR),                         //  output,   width = 1,                                         .mem_par
        .emif_agilex_d_mem_mem_alert_n                     (DDR4D_ALERT_n),                     //   input,   width = 1,                                         .mem_alert_n
        .emif_agilex_d_mem_mem_dqs                         (DDR4D_DQS),                         //   inout,   width = 9,                                         .mem_dqs
        .emif_agilex_d_mem_mem_dqs_n                       (DDR4D_DQS_n),                       //   inout,   width = 9,                                         .mem_dqs_n
        .emif_agilex_d_mem_mem_dq                          (DDR4D_DQ),                          //   inout,  width = 72,                                         .mem_dq
        .emif_agilex_d_mem_mem_dbi_n                       (DDR4D_DBI_n),                       //   inout,   width = 9,                                         .mem_dbi_n
        .emif_agilex_d_status_local_cal_success            (ddr4_d_status_local_cal_success),            //  output,   width = 1,                     emif_agilex_d_status.local_cal_success
        .emif_agilex_d_status_local_cal_fail               (ddr4_d_status_local_cal_fail),               //  output,   width = 1,                                         .local_cal_fail
        
        .reset_reset_n                                     (reset_n)                                      //   input,   width = 1,                                    reset.reset_n
    );
	 

	 
heart_beat  heart_beat_0 ( .CLK(DDR4A_REFCLK_p ),     .CLK_FREQ  (33_333_333) , . CK_1HZ (LED_BRACKET[0]) );
heart_beat  heart_beat_1 ( .CLK(DDR4B_REFCLK_p ),     .CLK_FREQ  (33_333_333) , . CK_1HZ (LED_BRACKET[1]) );
heart_beat  heart_beat_2 ( .CLK(DDR4C_REFCLK_p ),     .CLK_FREQ  (33_333_333) , . CK_1HZ (LED_BRACKET[2]) );
heart_beat  heart_beat_3 ( .CLK(DDR4D_REFCLK_p ),     .CLK_FREQ  (33_333_333) , . CK_1HZ (LED_BRACKET[3]) );

heart_beat  heart_beat_success ( .CLK(CLK_50_B3C ),     .CLK_FREQ  (50_000_000) , . CK_1HZ (heart_led) );
assign LED[0] =  (ddr4_a_status_local_cal_success) ?1'b0:heart_led;
assign LED[1] =  (ddr4_b_status_local_cal_success) ?1'b0:heart_led;
assign LED[2] =  (ddr4_c_status_local_cal_success) ?1'b0:heart_led;
assign LED[3] =  (ddr4_d_status_local_cal_success) ?1'b0:heart_led;


endmodule
