// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



`timescale 1ps/1ps

// DESCRIPTION
// Ethernet PCS66 Client

module alt_ehipc3_fm_sl_pcs66_client #(
    parameter OTN_MODE_EN       = 2'b0,
    parameter SIM_EMULATE       = 1'b0                        
)(
    // PCS 64/66 Encoding Interface 
    input           clk_tx,
    input           din_sclr_tx,
    input  [63:0]   din_d_tx, // bit 0 first
    input   [7:0]   din_c_tx,
    input           din_am_tx,
    input           din_valid_tx,
    output          din_ready_tx,
    input           dout_ready_tx,
    output          dout_valid_tx,
    output          dout_am_tx,
    output [65:0]   dout_tx,
    
    // PCS 66/64 Decoding Interface 
    input           clk_rx,
    input  [65:0]   din_rx, // bit 0 first
    input           din_valid_rx,
    input           din_am_rx,
    output [63:0]   dout_d_rx,
    output  [7:0]   dout_c_rx,
    output          dout_valid_rx,
    output          dout_am_rx
);


///////////////////////////// TX SIDE ///////////////////////////////////////

////////////////////
// encode
generate 
  if (OTN_MODE_EN == 1) begin: GEN_TX_OTN
    alt_ehipc3_fm_sl_delay6w1 d_am_tx (
      .clk  (clk_tx),
      .din  (din_am_tx),
      .dout (dout_am_tx)
    );

    alt_ehipc3_fm_sl_delay6w1 d_valid_tx (
      .clk  (clk_tx),
      .din  (din_valid_tx),
      .dout (dout_valid_tx)
    );

    alt_ehipc3_fm_sl_delay6w1 d_ready_tx (
      .clk  (clk_tx),
      .din  (dout_ready_tx),
      .dout (din_ready_tx)
    );
    
    ////////////////////
    // Encoder  
    wire [65:0] encoded_blk;
    alt_ehipc3_fm_sl_ethenc ee0_tx (
      .clk   (clk_tx),
      .sclr  (din_sclr_tx),
      .din_c (din_c_tx),
      .din_d (din_d_tx),
      .dout  (encoded_blk)
    );
    defparam ee0_tx .SIM_EMULATE = SIM_EMULATE; 
    
    ////////////////////
    // scramble
    wire [1:0] frm;
    wire [63:0] to_scram;
    assign {to_scram,frm} = encoded_blk;

    wire to_scram_valid;
    alt_ehipc3_fm_sl_delay4w1 d0_scram_valid_tx (
      .clk(clk_tx),
      .din(din_valid_tx),
      .dout(to_scram_valid)
    );

    wire [63:0] from_scram;
    alt_ehipc3_fm_sl_scram64 sc0_tx (
      .clk(clk_tx),
      .din_valid(to_scram_valid),
      .din(to_scram),
      .dout(from_scram),
      .dout_valid()
    );
    defparam sc0_tx .SIM_EMULATE = SIM_EMULATE;

    wire [1:0] frm_d;
    alt_ehipc3_fm_sl_delay2w2 d1_frm_tx (
      .clk(clk_tx),
      .din(frm),
      .dout(frm_d)
    ); 
    assign dout_tx = {from_scram, frm_d} ; // {scrambled data, frame header}  
  end  
  else if (OTN_MODE_EN == 2) begin
  
    wire tag_din_am;
    alt_ehipc3_fm_sl_delay6w1 d_am_tx (
      .clk  (clk_tx),
      .din  (din_am_tx),
      .dout (tag_din_am)
    );

    wire dout_valid_tag;
    alt_ehipc3_fm_sl_delay6w1 d_valid_tx (
      .clk  (clk_tx),
      .din  (din_valid_tx),
      .dout (dout_valid_tag)
    );

    wire din_ready_tag;
    alt_ehipc3_fm_sl_delay6w1 d_ready_tx (
      .clk  (clk_tx),
      .din  (dout_ready_tx),
      .dout (din_ready_tag)
    );
    
    ////////////////////
    // Encoder  
    wire [65:0] encoded_blk;
    alt_ehipc3_fm_sl_ethenc ee0_tx (
      .clk   (clk_tx),
      .sclr  (din_sclr_tx),
      .din_c (din_c_tx),
      .din_d (din_d_tx),
      .dout  (encoded_blk)
    );
    defparam ee0_tx .SIM_EMULATE = SIM_EMULATE; 
    
    ////////////////////
    // scramble
    wire [1:0] frm;
    wire [63:0] to_scram;
    assign {to_scram,frm} = encoded_blk;

    wire to_scram_valid;
    alt_ehipc3_fm_sl_delay4w1 d0_scram_valid_tx (
      .clk(clk_tx),
      .din(din_valid_tx),
      .dout(to_scram_valid)
    );

    wire [63:0] from_scram;
    alt_ehipc3_fm_sl_scram64 sc0_tx (
      .clk(clk_tx),
      .din_valid(to_scram_valid),
      .din(to_scram),
      .dout(from_scram),
      .dout_valid()
    );
    defparam sc0_tx .SIM_EMULATE = SIM_EMULATE;

    wire [1:0] frm_d;
    alt_ehipc3_fm_sl_delay2w2 d1_frm_tx (
      .clk(clk_tx),
      .din(frm),
      .dout(frm_d)
    );
    
    wire [65:0] tag_din;
    assign tag_din = {from_scram, frm_d} ; // {scrambled data, frame header}
    wire [65:0] tag_dout;

    alt_ehipc3_fm_sl_etag tag25_tx (
      .clk(clk_tx),
      .din(tag_din),
      .din_am(tag_din_am),
      .din_valid(1'b1),
      .dout(tag_dout),
      .dout_valid(),
      .srst(din_sclr_tx)
    );
     
    assign dout_tx = tag_dout;

    reg tag_din_am_r;
    reg dout_valid_tag_r;
    reg din_ready_tag_r;
    always @(posedge clk_tx) begin
         tag_din_am_r  <= tag_din_am;
         dout_valid_tag_r <= dout_valid_tag;
         din_ready_tag_r <= din_ready_tag;
    end
    assign dout_am_tx = tag_din_am_r;   
    assign dout_valid_tx = dout_valid_tag_r;
    assign din_ready_tx = din_ready_tag_r;
  end
  else begin
    alt_ehipc3_fm_sl_delay4w1 d_am_tx (
      .clk  (clk_tx),
      .din  (din_am_tx),
      .dout (dout_am_tx)
    );

    alt_ehipc3_fm_sl_delay4w1 d_valid_tx (
      .clk  (clk_tx),
      .din  (din_valid_tx),
      .dout (dout_valid_tx)
    );

    alt_ehipc3_fm_sl_delay4w1 d_ready_tx (
      .clk  (clk_tx),
      .din  (dout_ready_tx),
      .dout (din_ready_tx)
    );
    
    ////////////////////
    // Encoder
    alt_ehipc3_fm_sl_ethenc ee0_tx (
      .clk   (clk_tx),
      .sclr  (din_sclr_tx),
      .din_c (din_c_tx),
      .din_d (din_d_tx),
      .dout  (dout_tx)
    );
    defparam ee0_tx .SIM_EMULATE = SIM_EMULATE;
  end
endgenerate



///////////////////////////// RX SIDE ///////////////////////////////////////
generate 
  if (OTN_MODE_EN == 0) begin: CHK_RX_OTN
    ////////////////////
    // decode  
    alt_ehipc3_fm_sl_ethdec ed0_rx (
      .clk    (clk_rx),
      .din    (din_rx),
      .dout_c (dout_c_rx),
      .dout_d (dout_d_rx)
    );
    defparam ed0_rx .SIM_EMULATE = SIM_EMULATE;

    alt_ehipc3_fm_sl_delay3w1 d_valid_rx (
      .clk    (clk_rx),
      .din    (din_valid_rx),
      .dout   (dout_valid_rx)
    );
  
    alt_ehipc3_fm_sl_delay3w1 d_am_rx (
      .clk    (clk_rx),
      .din    (din_am_rx),
      .dout   (dout_am_rx)
    );
  end 
  else begin
    ////////////////////
    // descramble

    wire [1:0] frm_rx;
    wire [63:0] to_descram;
    assign {to_descram,frm_rx} = din_rx;

    wire [63:0] from_descram;
    wire from_descram_valid;
    alt_ehipc3_fm_sl_descram64 dsc0_rx (
      .clk(clk_rx),
      .din_valid(din_valid_rx),
      .din(to_descram),
      .dout(from_descram),
      .dout_valid(from_descram_valid)
    );
    defparam dsc0_rx .SIM_EMULATE = SIM_EMULATE;

    wire [1:0] frm_d_rx;
    alt_ehipc3_fm_sl_delay1w2 d1_frm_rx (
      .clk(clk_rx),
      .din(frm_rx),
      .dout(frm_d_rx)
    );
    defparam d1_frm_rx .SIM_EMULATE = SIM_EMULATE;
    
    ////////////////////
    // decode  
    wire [65:0] coded_blk = {from_descram,frm_d_rx};
    alt_ehipc3_fm_sl_ethdec ed0_rx (
      .clk    (clk_rx),
      .din    (coded_blk),
      .dout_c (dout_c_rx),
      .dout_d (dout_d_rx)
    );
    defparam ed0_rx .SIM_EMULATE = SIM_EMULATE;

    alt_ehipc3_fm_sl_delay3w1 d_valid_rx (
      .clk    (clk_rx),
      .din    (from_descram_valid),
      .dout   (dout_valid_rx)
    );
   
    alt_ehipc3_fm_sl_delay4w1 d_am_rx (
      .clk    (clk_rx),
      .din    (din_am_rx),
      .dout   (dout_am_rx)
    );
  
  end
endgenerate
  
endmodule


// ________________
//PRBS23 GENERATOR
// ________________
//------------------------------------------------------------
// Functional Description:
// This module is the Pseudo-Random Bit Sequence 23 Block
// where g(x) = x^23 + x^18 + x^0
//
// use lsb of m 1st first
// k can be > N, but part of the sequence will be skipped
//
//------------------------------------------------------------

module prbs23 ( clk, rst_n, load, enable, seed, d, m);

  parameter k = 23;       //step value = a^k
  parameter N = 23;
  input     clk;
  input     rst_n;
  input     load;
  input     enable;
  input     [N-1:0] seed;
  input     [N-1:0] d;
  output    [N-1:0] m;

  reg       [N-1:0] m;
  reg       [N-1:0] tmpa;
  reg       [N-1:0] tmpb;
  integer   i,j;

  always @ (d)
  begin
    tmpa = d;
    for (i=0; i<k; i=i+1) 
    begin
      for (j=0; j<(N-1); j=j+1) begin tmpb[j] = tmpa[j+1]; end
        tmpb[N-1] = tmpa[18] ^ tmpa[0];      //x^23 + x[18] + x[0]
        tmpa = tmpb;
    end
  end

  always @(posedge clk or negedge rst_n)
  begin
    if (!rst_n) m <= 0;
    else if (load) m <= seed;
    else if (enable) m <= tmpb;
  end

 endmodule
