# (C) 2001-2021 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


set ver 0.2s10c3
 
 set ADDR_KR4_BASECTRL    0xb0
 set ADDR_KR4_BASESTAT    0xb1
 set ADDR_KR4_FECCTRL0    0xb2
 set ADDR_KR4_FECCTRL1    0xb5
 set ADDR_KR4_FECCTRL2    0xb8
 set ADDR_KR4_FECCTRL3    0xbb
 set ADDR_KR4_ANCTRL      0xc0
 set ADDR_KR4_ANSTAT      0xc2
 set ADDR_KR4_ANOVRD      0xcc
 set ADDR_KR4_LTCTRL      0xd0
 set ADDR_KR4_LTCTL2      0xd1
 set ADDR_KR4_LTSTAT      0xd2
 set ADDR_KR4_LTBERT      0xd3
 set ADDR_KR4_LTSET0      0xd5
 set ADDR_KR4_LTSET1      0xe2
 set ADDR_KR4_LTSET2      0xe6
 set ADDR_KR4_LTSET3      0xea
 set ADDR_FEC_CORR0       0x200
 set ADDR_FEC_CORR1       0x208
 set ADDR_FEC_CORR2       0x210
 set ADDR_FEC_CORR3       0x218
 set ADDR_FEC_UCRR0       0x220
 set ADDR_FEC_UCRR1       0x228
 set ADDR_FEC_UCRR2       0x230
 set ADDR_FEC_UCRR3       0x238


proc Get_value_in_variable {uservar} {
    global $uservar
    set userval [set $uservar]
    return $userval
}

proc rst_pcs {} {
    global BASE_RXPHY 
    global ADDR_PHY_PMACFG 
    
    reg_write   $BASE_RXPHY $ADDR_PHY_PMACFG 1
    reg_write   $BASE_RXPHY $ADDR_PHY_PMACFG 0
}

proc kr_restart {} {
  global BASE_KR4
  global ADDR_KR4_BASECTRL
  
  set reg [reg_read   $BASE_KR4 $ADDR_KR4_BASECTRL]
  reg_write $BASE_KR4 $ADDR_KR4_BASECTRL [expr $reg | 0x7]
}

proc kr_test {iter} {
    for {set i 0} {$i < $iter} {incr i} {
        puts $i
        kr_restart
        after 1500
        set reg [reg_read 0xb1]
        puts $reg
        puts [reg_read 0xd2]
        if {($reg & 0xffff) != 0x2001 && ($reg & 0xffff) != 0x8001} {break}
    }
}

proc kr_pause {} {
  global BASE_KR4
  global ADDR_KR4_BASECTRL
  global ADDR_KR4_BASESTAT
  set reg [reg_read   $BASE_KR4 $ADDR_KR4_BASECTRL]
  reg_write $BASE_KR4 $ADDR_KR4_BASECTRL [expr $reg | 0x80000000]
  #bail if register write didn't work (probably no AN/LT in design, or in reset)
  if {!([reg_read   $BASE_KR4 $ADDR_KR4_BASECTRL] & 0x80000000)} {return}
  #otherwise, wait for acknowledge
  while {!([reg_read $BASE_KR4 $ADDR_KR4_BASESTAT] & 0x80000000)} {}
}

proc kr_unpause {} {
  global BASE_KR4
  global ADDR_KR4_BASECTRL
  global ADDR_KR4_BASESTAT
  set reg [reg_read   $BASE_KR4 $ADDR_KR4_BASECTRL]
  reg_write $BASE_KR4 $ADDR_KR4_BASECTRL [expr $reg & 0x7FFFFFFF]
  while {[reg_read $BASE_KR4 $ADDR_KR4_BASESTAT] & 0x80000000} {}
}

proc an_disable {} {
  global BASE_KR4
  global ADDR_KR4_ANCTRL
  
  set an_c0 [reg_read    $BASE_KR4 $ADDR_KR4_ANCTRL]
  set an_c0 [expr $an_c0 & 0xFFFFFFFE]
  reg_write   $BASE_KR4 $ADDR_KR4_ANCTRL $an_c0
}

proc an_enable {} {
  global BASE_KR4
  global ADDR_KR4_ANCTRL
  
  set an_c0 [reg_read    $BASE_KR4 $ADDR_KR4_ANCTRL]
  set an_c0 [expr $an_c0 | 0x1]
  reg_write   $BASE_KR4 $ADDR_KR4_ANCTRL $an_c0
}
proc ignore_nonce_en {} {
  global BASE_KR4
  global ADDR_KR4_ANCTRL
  set an_c0 [reg_read    $BASE_KR4 $ADDR_KR4_ANCTRL]
  set an_c0 [expr $an_c0 | 0x80]
  reg_write   $BASE_KR4 $ADDR_KR4_ANCTRL $an_c0
}
proc ignore_nonce_dis {} {
  global BASE_KR4
  global ADDR_KR4_ANCTRL
  set an_c0 [reg_read    $BASE_KR4 $ADDR_KR4_ANCTRL]
  set an_c0 [expr $an_c0 & 0xFFFFFF7F]
  reg_write   $BASE_KR4 $ADDR_KR4_ANCTRL $an_c0
}

proc kr_defaults {{nomwt 0} {man 0} {ac 0} {dc 0} {vga 0}} {
  global BASE_KR4
  global ADDR_KR4_LTCTRL
  global ADDR_KR4_ANCTRL
  global ADDR_KR4_BASECTRL

  set man [expr $man != 0]
  set nomwt [expr $nomwt != 0]
  set ac [expr ($ac & 0xf) >> 1]
  set dc [expr ($dc & 0x3f) >> 1]
  set vga [expr ($vga & 0x1f) >> 1]
  reg_write   $BASE_KR4 $ADDR_KR4_LTCTRL [expr 0x00000111 | ($man << 19) | ($ac << 20) | {$dc << 23} | {$vga << 28} | ($nomwt << 1)]
#turn off LT-fail-to-AN
  set reg [reg_read   $BASE_KR4 $ADDR_KR4_BASECTRL]
  reg_write $BASE_KR4 $ADDR_KR4_BASECTRL [expr $reg | 0x1000]
#disable nonce check in AN
  set reg [reg_read    $BASE_KR4 $ADDR_KR4_ANCTRL]
  reg_write   $BASE_KR4 $ADDR_KR4_ANCTRL [expr $reg | 0x80]
}


proc reconfig_read {chan addr {device 0}} {
    global BASE_S10RECO
    global RECO_CH
    global BASE_NDEVICE

    set chanoffs [expr ($BASE_NDEVICE * $device) + $BASE_S10RECO + ($RECO_CH * $chan)]
    return [reg_read $chanoffs $addr]
}

proc reconfig_write {chan addr val {device 0}} {
    global BASE_S10RECO
    global RECO_CH
    global BASE_NDEVICE

    set chanoffs [expr ($BASE_NDEVICE * $device) + $BASE_S10RECO + ($RECO_CH * $chan)]
    return [reg_write $chanoffs $addr $val]
}

proc reconfig_rmw {chan addr mask val {device 0}} {
    set value [reconfig_read $chan $addr $device]
    set value [expr $value & [expr 0xffffffff & ~$mask]] 
    set value [expr $value | $val] 
    reconfig_write $chan $addr $value $device
}

#E-tile TX taps
set TAP_PRE1 0x0
set TAP_PRE3 0x3
set TAP_ATTEN 0x4
set TAP_AMP 0x7
set TAP_POST1 0x8
set TAP_PRE2 0xC

proc pma_gettap {ch tap {device 0}} {
    return [pma_int $ch 0x15 0x00 [expr 0x0100 | ($tap << 12)] $device]
}

proc pma_settap {ch tap val {device 0}} {
    set val [pma_int $ch 0x15 0x00 [expr $val | ($tap << 12)] $device]
    return [expr $val == 0x15]
}


proc fec_err_corr {chan {device 0}} {
    global BASE_S10RECO
    global RECO_CH
    global ADDR_FEC_CORR0
    global ADDR_FEC_CORR1
    global ADDR_FEC_CORR2
    global ADDR_FEC_CORR3
    global BASE_NDEVICE

    set chanoffs [expr ($BASE_NDEVICE * $device) + $BASE_S10RECO + ($RECO_CH * $chan)]
    set val [reg_read $chanoffs $ADDR_FEC_CORR0]
    set val [expr $val + [reg_read $chanoffs $ADDR_FEC_CORR1] * 0x100]
    set val [expr $val + [reg_read $chanoffs $ADDR_FEC_CORR2] * 0x10000]
    set val [expr $val + [reg_read $chanoffs $ADDR_FEC_CORR3] * 0x1000000]

    return $val
}

proc fec_err_uncorr {chan {device 0}} {
    global BASE_S10RECO
    global RECO_CH
    global ADDR_FEC_UCRR0
    global ADDR_FEC_UCRR1
    global ADDR_FEC_UCRR2
    global ADDR_FEC_UCRR3
    global BASE_NDEVICE

    set chanoffs [expr ($BASE_NDEVICE * $device) + $BASE_S10RECO + ($RECO_CH * $chan)]
    set val [reg_read $chanoffs $ADDR_FEC_UCRR0]
    set val [expr $val + [reg_read $chanoffs $ADDR_FEC_UCRR1] * 0x100]
    set val [expr $val + [reg_read $chanoffs $ADDR_FEC_UCRR2] * 0x10000]
    set val [expr $val + [reg_read $chanoffs $ADDR_FEC_UCRR3] * 0x1000000]

    #puts "reg_read $chanoffs $ADDR_FEC_UCRR0"
    #puts "reg_read $chanoffs $ADDR_FEC_UCRR1"
    #puts "reg_read $chanoffs $ADDR_FEC_UCRR2"
    #puts "reg_read $chanoffs $ADDR_FEC_UCRR3"
    #puts [reg_read $chanoffs $ADDR_FEC_UCRR0]
    #puts [reg_read $chanoffs $ADDR_FEC_UCRR1]
    #puts [reg_read $chanoffs $ADDR_FEC_UCRR2]
    #puts [reg_read $chanoffs $ADDR_FEC_UCRR3]

    return $val
}

proc fec_err_inj_chan { chan rate pattern} {
  puts "Writing a rate of $rate and a pattern of $pattern to channel $chan"
    reg_write  0x10000 [expr 0x1e0 + $chan*4 + 0] $rate
    reg_write  0x10000 [expr 0x1e0 + $chan*4 + 1] $pattern
}


proc fec_err_inj { rate pattern } {
  fec_err_inj_chan 0 $rate $pattern
  fec_err_inj_chan 1 $rate $pattern
  fec_err_inj_chan 2 $rate $pattern
  fec_err_inj_chan 3 $rate $pattern
}


proc fec_err_rpt_helper {base_readout} {
  set res 0
  for {set i 0} {$i < 8} {incr i} {
  set res [expr $res << 8 ^ [reg_read 0x10000 [expr $base_readout - $i]]]
  }
  return $res
}

proc fec_err_rpt {} {
  puts "error injection reg " 
  puts -nonewline "rate:    ch0 ch1 ch2 ch3  " 
  puts -nonewline "[reg_read 0x10000 0x1e0] | "
  puts -nonewline "[reg_read 0x10000 0x1e4] | "
  puts -nonewline "[reg_read 0x10000 0x1e8] | "
  puts            "[reg_read 0x10000 0x1ec]   "

  puts -nonewline "pattern: ch0 ch1 ch2 ch3  " 
  puts -nonewline "[reg_read 0x10000 0x1e1] | "
  puts -nonewline "[reg_read 0x10000 0x1e5] | "
  puts -nonewline "[reg_read 0x10000 0x1e9] | "
  puts            "[reg_read 0x10000 0x1ed]   "

  puts "error injection status : ch0 ch1 ch2 ch3" 
  puts [reg_read 0x10000 0x1f0]
  puts [reg_read 0x10000 0x1f1]
  puts [reg_read 0x10000 0x1f4]
  puts [reg_read 0x10000 0x1f5]
  puts [reg_read 0x10000 0x1f8]
  puts [reg_read 0x10000 0x1f9]
  puts [reg_read 0x10000 0x1fc]
  puts [reg_read 0x10000 0x1fd]

  puts "x20f -- x220 : correctable" 
  puts -nonewline "ch0 -> " 
  puts [format 0x%x  [fec_err_rpt_helper 0x207]]
  puts -nonewline "ch1 -> " 
  puts [format 0x%x  [fec_err_rpt_helper 0x20f]]
  puts -nonewline "ch2 -> " 
  puts [format 0x%x  [fec_err_rpt_helper 0x217]]
  puts -nonewline "ch3 -> " 
  puts [format 0x%x  [fec_err_rpt_helper 0x21f]]

  
  puts "x23f -- x220 : uncorrectable" 

  puts -nonewline "ch0 -> "  
  puts [format 0x%x  [fec_err_rpt_helper 0x227]]
  puts -nonewline "ch1 -> " 
  puts [format 0x%x  [fec_err_rpt_helper 0x22f]]
  puts -nonewline "ch2 -> "  
  puts [format 0x%x  [fec_err_rpt_helper 0x237]]
  puts -nonewline "ch3 -> "  
  puts [format 0x%x  [fec_err_rpt_helper 0x23f]]
}

proc fec_errs {} {
    puts -nonewline "[fec_err_corr 0] [fec_err_uncorr 0] | "
    puts -nonewline "[fec_err_corr 1] [fec_err_uncorr 1] | "
    puts -nonewline "[fec_err_corr 2] [fec_err_uncorr 2] | "
    puts            "[fec_err_corr 3] [fec_err_uncorr 3]"
}

proc clear_fec_err {chan} {
    global BASE_KR4
    global ADDR_KR4_FECCTRL$chan
    
    
    reg_write $BASE_KR4 [set ADDR_KR4_FECCTRL$chan] 0x1000
    reg_write $BASE_KR4 [set ADDR_KR4_FECCTRL$chan] 0x0000
}

proc clear_all_fec_err {} {
    for {set idx 0} {$idx < 4} {incr idx} { 
      clear_fec_err $idx
    }
}

# Clear the registers read in proc fec_err_rpt
proc fec_err_counter_reset { } {
  reg_write 0x10000 0x108 0xf0 
  after 10
  reg_write 0x10000 0x108 0x00
  after 10
}

#insert FEC burst error, up to length 15
# FEC should correct anything length 11 or lower
proc ins_burst_err {chan len} {
    global BASE_S10RECO
    global RECO_CH
    global ADDR_FEC_ERRI
    global ADDR_FEC_TRNS_M
    global ADDR_FEC_BRST_M
    global ADDR_FEC_BLEN_O
    global ADDR_FEC_BLEN_M
    global BASE_KR4
    global ADDR_KR4_FECCTRL$chan

    set chanoffs [expr $BASE_S10RECO + ($RECO_CH * $chan)]
    set reg [reg_read $chanoffs $ADDR_FEC_ERRI]

    # clear transcode error, set burst error
    set reg [expr ~$ADDR_FEC_TRNS_M & $reg]
    set reg [expr $ADDR_FEC_BRST_M | $reg]

    # set burst error length
    set reg [expr ~$ADDR_FEC_BLEN_M & $reg]
    set reg [expr $reg | (($len << $ADDR_FEC_BLEN_O) & $ADDR_FEC_BLEN_M)]

    reg_write $chanoffs $ADDR_FEC_ERRI $reg

    # trigger error (self clearing)
    reg_write $BASE_KR4 [set ADDR_KR4_FECCTRL$chan] 0x800
}

proc fec_block_lock {chan} {
    global BASE_KR4
    global ADDR_KR4_BASESTAT
    
    
    set reg [reg_read $BASE_KR4 $ADDR_KR4_BASESTAT]
    return [expr ($reg & (0x100000 << $chan)) != 0]
}

proc force_fec {{on 1}} {
    global BASE_KR4
    global ADDR_KR4_BASECTRL
    
    
    set reg [reg_read $BASE_KR4 $ADDR_KR4_BASECTRL]
    if {$on} {
        set reg [expr $reg | 0x80]
    } else {
        set reg [expr $reg & (~0x80)]
    }
    
    reg_write $BASE_KR4 $ADDR_KR4_BASECTRL $reg
}

# To trigger ADP -->
# Reset the adp ckt --> rstn= 148[0]=0
# rmw_val [expr $offset + 0x148] 0x01 0x00 ; after 10
# ####Everything is set, now remove reset 148[0]=1 and trigger/toggle adp engine start 14C[0]
# rmw_val [expr $offset + 0x148] 0x01 0x01 ; after 5
# rmw_val [expr $offset + 0x14C] 0x01 0x00 ; after 5
# rmw_val [expr $offset + 0x14C] 0x01 0x01 5
proc trigger_adp {channel} {
  reconfig_rmw $channel 0x148 0x01 0x00
  reconfig_rmw $channel 0x148 0x01 0x00
  reconfig_rmw $channel 0x148 0x01 0x01

  reconfig_rmw $channel 0x148 0x01 0x01
  reconfig_rmw $channel 0x14c 0x01 0x00
  reconfig_rmw $channel 0x14c 0x01 0x00
  reconfig_rmw $channel 0x14c 0x01 0x01
  reconfig_rmw $channel 0x14c 0x01 0x01
}

proc trig_adp_all {} {
    trig_adp 0
    trig_adp 1
    trig_adp 2
    trig_adp 3
}

proc bypass_adp {channel} {
  #enable bypass ctle, vga, dfe
  reconfig_rmw $channel 0x149 0x80 0x80 ; #vga
  reconfig_rmw $channel 0x15c 0x80 0x80 ; #ctle-dc
  reconfig_rmw $channel 0x15f 0x08 0x08 ; #ctle-ac

  reconfig_rmw $channel 0x161 0x60 0x60 ; #dfe/dlev

}

proc bypass_adp_all {} {
    bypass_adp 0
    bypass_adp 1
    bypass_adp 2
    bypass_adp 3
}

#time in milliseconds
proc set_bertime {{t 0.425115}} {
    global BASE_KR4
    global ADDR_KR4_LTBERT
    set t [expr round($t*2.3523) & 0x3ff]
    reg_write $BASE_KR4 $ADDR_KR4_LTBERT [expr $t << 10]
    puts "ber_time_k_frames set to $t"
}

proc chan_stat_csv {chan {usefec 0}} {
    global BASE_KR4
    global ADDR_KR4_LTSTAT
    global ADDR_KR4_LTSET$chan
    global BASE_RXPHY
    global ADDR_PHY_TXPLLOCK
    global ADDR_PHY_FREQLOCK
    global ADDR_PHY_FRMERROR
    
    
    puts -nonewline "LANE$chan\t"
    
    set rdreg [reg_read $BASE_KR4 [set ADDR_KR4_LTSET$chan]]
    set vod [expr $rdreg & 0x0000003f]
    set post [expr ($rdreg & 0x00001f00)>>8]
    set pre [expr ($rdreg & 0x000f0000)>>16]
    
    set vod_rd [a10dprio_read $chan VOD]
    set vod_rd [expr $vod_rd & 0x1f]
    set post_rd [a10dprio_read $chan PST1]
    if {$post_rd & 0x40} {
        set post_rd [expr $post_rd & 0x3f]
    } else {
        set post_rd [expr -($post_rd & 0x3f)]
    }
    set pre_rd [a10dprio_read $chan PRE1]
    if {$pre_rd & 0x20} {
        set pre_rd [expr $pre_rd & 0x1f]
    } else {
        set pre_rd [expr -($pre_rd & 0x1f)]
    }
    
    #  Test bus = ADP 171[4:1]=1	
    reconfig_rmw $chan 0x171 0x1E 0x16
    reconfig_write $chan 0x17E 0
    
    if {[reconfig_read $chan 0x161] & 0x20} {
        set dlev "m"
    } else {
        reconfig_rmw $chan 0x149 0x3F 6
        reconfig_write $chan 0x17E 0
        set dlev [expr [ reconfig_read $chan 0x17E ] & 0x3F]
    }
    if {[reconfig_read $chan 0x149] & 0x80} {
        set vga "m [expr (([reconfig_read $chan 0x11f] & 0x1) << 4) | ([reconfig_read $chan 0x11a] & 0x7)]"
    } else {
        reconfig_rmw $chan 0x149 0x3F 5
        reconfig_write $chan 0x17E 0
        set vga [expr [ reconfig_read $chan 0x17E ] & 0x1F]
    }
    if {[reconfig_read $chan 0x15c] & 0x80} {
        set ctle_dc "m [expr ([reconfig_read $chan 0x11e] >> 7) | ([reconfig_read $chan 0x11c] & 0x1f)]"
    } else {
        reconfig_rmw $chan 0x149 0x3F 4
        reconfig_write $chan 0x17E 0
        set ctle_dc [expr [ reconfig_read $chan 0x17E ] & 0x3F]
    }
    if {[reconfig_read $chan 0x15f] & 0x08} {
        set ctle_ac "m [expr [reconfig_read $chan 0x11a] >> 4]"
    } else {
        reconfig_rmw $chan 0x149 0x3F 3
        reconfig_write $chan 0x17E 0
        set ctle_ac [expr [expr [ reconfig_read $chan 0x17E ] >> 3] & 0XF]
    }

    #  Test bus = ADP 171[4:1]=1	
    if {[reconfig_read $chan 0x161] & 0x40} {
        set tap1 "m"
        set tap2 "m"
        set tap3 "m"
        set tap4 "m"
        set tap5 "m"
        set tap6 "m"
        set tap7 "m"
    } else {
        reconfig_rmw $chan 0x171 0x1E 0x16
        reconfig_write $chan 0x17e 0
        reconfig_rmw $chan 0x149 0x3F 7
        reconfig_write $chan 0x17E 0
        set tap1 [expr [reconfig_read $chan   0x17E] & 0x7F]
        set tap1 [expr ($tap1 & 0x3F) * (($tap1 & 0x40) ? -1 : 1)]
        reconfig_rmw $chan 0x149 0x3F 8
        reconfig_write $chan 0x17E 0
        set tap2 [expr [reconfig_read $chan   0x17E] & 0x3F]
        set tap2 [expr ($tap2 & 0x1F) * (($tap2 & 0x20) ? -1 : 1)]
        reconfig_rmw $chan 0x149 0x3F 9
        reconfig_write $chan 0x17E 0
        set tap3 [expr [reconfig_read $chan   0x17E] & 0x3F]
        set tap3 [expr ($tap3 & 0x1F) * (($tap3 & 0x20) ? -1 : 1)]
        reconfig_rmw $chan 0x149 0x3F 10
        reconfig_write $chan 0x17E 0
        set tap4 [expr [reconfig_read $chan   0x17E] & 0x1F]
        set tap4 [expr ($tap4 & 0x0F) * (($tap4 & 0x10) ? -1 : 1)]
        reconfig_rmw $chan 0x149 0x3F 11
        reconfig_write $chan 0x17E 0
        set tap5 [expr [reconfig_read $chan   0x17E] & 0x1F]
        set tap5 [expr ($tap5 & 0x0F) * (($tap5 & 0x10) ? -1 : 1)]
        reconfig_rmw $chan 0x149 0x3F 12
        reconfig_write $chan 0x17E 0
        set tap6 [expr [reconfig_read $chan   0x17E] & 0x1F]
        set tap6 [expr ($tap6 & 0x0F) * (($tap6 & 0x10) ? -1 : 1)]
        reconfig_rmw $chan 0x149 0x3F 13
        reconfig_write $chan 0x17E 0
        set tap7 [expr [reconfig_read $chan   0x17E] & 0x1F]
        set tap7 [expr ($tap7 & 0x0F) * (($tap7 & 0x10) ? -1 : 1)]
    }
    
    
    
    
    set fec_corr [fec_err_corr $chan]
    set fec_uncorr [fec_err_uncorr $chan]
    
    puts -nonewline "$vod, $post, $pre,   \t$vod_rd, $post_rd, $pre_rd,   \t    $dlev,   $vga,    $ctle_ac,         $ctle_dc, "
    puts -nonewline "   $tap1,    $tap2,    $tap3,    $tap4,    $tap5,    $tap6,    $tap7,"
    if {$usefec} {puts -nonewline " \t$fec_corr, $fec_uncorr, "}

    #set rdreg [reg_read $BASE_RXPHY $ADDR_PHY_TXPLLOCK]
    # TX PLL Not Locked
    #if {!($rdreg & (1 << $chan))} {puts -nonewline "    X"}

    set rdreg [reg_read $BASE_RXPHY $ADDR_PHY_FREQLOCK]
    # RX CDR Not Locked
    if {!($rdreg & (1 << $chan))} {puts -nonewline "    x"}
    
    set rdreg [reg_read $BASE_RXPHY $ADDR_PHY_FRMERROR]
    # 40G frame error
    if {$rdreg & (1 << $chan)} {puts -nonewline "    e"}
    
    set rdreg [reg_read $BASE_KR4 $ADDR_KR4_LTSTAT]
    # not trained
    if {!($rdreg & (1 << ($chan * 8)))} {puts -nonewline "    N"}
    # frame lock
    if {$rdreg & (0x2 << ($chan * 8))} {puts -nonewline "    l"}
    # startup
    if {$rdreg & (0x4 << ($chan * 8))} {puts -nonewline "    s"}
    # training fail (timeout)
    if {$rdreg & (0x8 << ($chan * 8))} {puts -nonewline "    F"}

    # FEC not locked
    if {$usefec & ![fec_block_lock $chan]} {puts -nonewline "    L"}
       
}

proc run_test_meas {t} {
    global CLIENT_BASE
    global REG_TEMP_SENSE
    global BASE_KR4
    global ADDR_KR4_BASECTRL
    global ADDR_KR4_BASESTAT
    global ADDR_KR4_ANSTAT
    global ADDR_KR4_LTCTRL
    global ADDR_KR4_LTCTL2
    global ADDR_KR4_LTSTAT
    global BASE_RXPHY
    global ADDR_PHY_RFCLKHZ
    global ADDR_PHY_CLKMACOK
    global ADDR_PHY_FALIGNED
    global BASE_TXSTATS
    global BASE_RXSTATS
    global REG_ST_LO
    global REG_CRCERR_LO
    global REG_STAT_CFG
    global REG_STAT_STATUS
    global ver
    
    
    #set degf [expr [reg_read $CLIENT_BASE $REG_TEMP_SENSE]]
    set degf 72
    set refclk [ expr [ reg_read $BASE_RXPHY $ADDR_PHY_RFCLKHZ ]]
    
    puts "---------------------------------------"
    puts "KR4 TEST $ver, TIME $t, ????? F, refclk $refclk"
    
    stop_pkt_gen
    setphy_clear_frame_error
    # clear stats
    clear_all_stats
    clear_all_fec_err

    # todo add error counter/global status readout
    start_pkt_gen
    after $t
    
    stop_pkt_gen
    after 100
    
    puts "GLOBAL,reg 0xB0  ,reg 0xB1  ,reg 0xC2  ,reg 0xD0  ,reg 0xD1  ,reg 0xD2  ,\tTX Starts,RX Starts,RX FCS Err,\tFlags"
    
    set txcnt [expr [stats_read $BASE_TXSTATS $REG_ST_LO]]
    set rxcnt [expr [stats_read $BASE_RXSTATS $REG_ST_LO]]
    set err [expr [stats_read $BASE_RXSTATS $REG_CRCERR_LO]]
    reg_read $BASE_KR4 $ADDR_KR4_ANSTAT 
    puts -nonewline "GLOBAL,[reg_read $BASE_KR4 $ADDR_KR4_BASECTRL],[reg_read $BASE_KR4 $ADDR_KR4_BASESTAT],[reg_read $BASE_KR4 $ADDR_KR4_ANSTAT],[reg_read $BASE_KR4 $ADDR_KR4_LTCTRL],[reg_read $BASE_KR4 $ADDR_KR4_LTCTL2],[reg_read $BASE_KR4 $ADDR_KR4_LTSTAT],\t$txcnt,$rxcnt,$err,\t"
    
    set rdreg [reg_read $BASE_RXPHY $ADDR_PHY_FALIGNED]
    # 40G lanes not deskewed
    if {!($rdreg & 1)} {puts -nonewline "d"}
    # 40G hi BER
    if {$rdreg & 2} {puts -nonewline "H"}
    
    set rdreg [reg_read $BASE_RXPHY $ADDR_PHY_CLKMACOK]
    # Tx analog not online
    if {!($rdreg & 1)} {puts -nonewline "a"}
    # Tx mac pll not lock
    if {!($rdreg & 2)} {puts -nonewline "P"}
    # Rx mac pll not lock
    if {!($rdreg & 4)} {puts -nonewline "p"}
    
    set rdreg [reg_read $BASE_KR4 $ADDR_KR4_BASESTAT]
    # link down (not ready)
    if {!($rdreg & 1)} {puts -nonewline "D"}
    #autoneg timeout
    if {$rdreg & 2} {puts -nonewline "A"}
    #link train timeout
    if {$rdreg & 4} {puts -nonewline "T"}
    
  
    set rdreg [reg_read $BASE_KR4 $ADDR_KR4_ANSTAT]
    #AN not complete
    if {!($rdreg & 4)} {puts -nonewline "n"}
    #ANSM not idle
    if {!($rdreg & 0x10)} {puts -nonewline "I"}
    
    set fec_mode [expr ($rdreg & 0x100) != 0]

    set rdreg [reg_read $BASE_RXSTATS $REG_STAT_STATUS]
    # parity error detected somewhere in stats
    if {$rdreg & 1} {puts -nonewline "*"}
    
    puts ""
    puts -nonewline "LANE#\tvod, post, pre,\tvod_rd, post_rd, pre_rd, DLEV, VGA , CTLE_AC, CTLE_DC, DFE1, DFE2, DFE3, DFE4, DFE5, DFE6, DFE7, "
    if {$fec_mode} { puts -nonewline "\tFEC Corr, FEC Uncorr,"}
    puts " Flags"
    
    for {set idx 0} {$idx < 4} {incr idx} { 
      chan_stat_csv $idx $fec_mode
      puts ""
    }
    
}
