# (C) 2001-2021 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


# =====================================================
#     Top level register map: Base Addresses
# =====================================================
#    word address to byte address translations should
#    happen inside the basic.tcl defining write & read
# ______________________________________________________

set CLIENT_BASE  $BASE_TRFC
# =====================================================
#     Ethernet registers (MAC + PHY)
# =====================================================

set REG_TEMP_SENSE     0x0006
# =====================================================
#     test utilities
# =====================================================

set log 0

proc client_init {LOG} {
    global log
    set log $LOG
}

proc check_e_temp {} {
    global CLIENT_BASE
    global REG_TEMP_SENSE
    global rdata
    global log
    set rdata [reg_read $CLIENT_BASE $REG_TEMP_SENSE]
    return $rdata
}


proc stop_pkt_gen {} {
    global CLIENT_BASE

    global rdata
    global wdata
    global log

    reg_write $CLIENT_BASE 0x10 0x7
}

proc client_rom {} {
    global CLIENT_BASE
    global log
    global rdata
    global wdata
    reg_write $CLIENT_BASE 0x10 0x2
}

proc client_loop {} {
    global CLIENT_BASE
    reg_write $CLIENT_BASE 0x10 0xf
}

proc start_pkt_gen {} {
    global CLIENT_BASE
    global log
    global rdata
    global wdata
    reg_write $CLIENT_BASE 0x10 0x5
}

proc client_reset {} {
    global CLIENT_BASE

    reg_write $CLIENT_BASE 0x16 1
    after 100
    reg_write $CLIENT_BASE 0x16 0
}

proc start_random_pkt_gen { {gap 1} } {
    global CLIENT_BASE
    global log
    global rdata
    global wdata
    if {$gap == 1} {
        #start packet gen, random mode, no gap
        reg_write $CLIENT_BASE 0x10 0x85 
    } elseif {$gap == 0} {
        reg_write $CLIENT_BASE 0x10 0x5
    }
}

# The start and end sizes must be in hex
proc start_incr_pkt_gen { {start 0040} {end 2580} {number 100} {stop_mode 0} {gap 1}} {
    global CLIENT_BASE
    global log
    global rdata
    global wdata
    # set start and end packet sizes
    set value 0x$start$end
    reg_write $CLIENT_BASE 0x8 $value

    # set number of packets to send
    if {stop_mode == 1} {
        set number [format %X $number]
        reg_write $CLIENT_BASE 0x9 $number

        if {$gap == 1} {
            # start packet gen without gap
            reg_write $CLIENT_BASE 0x10 0xE5
        } else {
            # start packet gen with gap
            reg_write $CLIENT_BASE 0x10 0x65
        }
    }   elseif {stop_mode == 0} { 
        # manual stopping mode
        if {$gap == 1} {
            # start packet gen without gap
            reg_write $CLIENT_BASE 0x10 0xA5
        } else {
            # start packet gen with gap
            reg_write $CLIENT_BASE 0x10 0x25
        }
    }

}


# The start and end sizes must be in hex
proc start_fixed_pkt_gen { {size 64} {number 100} {stop_mode 0} {gap 1}} {
    global CLIENT_BASE
    global log
    global rdata
    global wdata
    # set start and end packet sizes
    set size [format %X $size]
    reg_write $CLIENT_BASE 0x8 $size

    # set number of packets to send
    if {stop_mode == 1} {
        set number [format %X $number]
        reg_write $CLIENT_BASE 0x9 $number

        if {$gap == 1} {
            # start packet gen without gap
            reg_write $CLIENT_BASE 0x10 0xD5
        } else {
            # start packet gen with gap
            reg_write $CLIENT_BASE 0x10 0x55
        }
    }   elseif {stop_mode == 0} {
        #set number [format %X $number]
        #reg_write 0x1009 $number

        if {$gap == 1} {
            # start packet gen without gap
            reg_write $CLIENT_BASE 0x10 0x95
        } else {
            # start packet gen with gap
            reg_write $CLIENT_BASE 0x10 0x15
        }
    }

}