// ============================================================================
// Copyright (c) 2021 by Terasic Inc.
// ============================================================================
//
// Permission:
//
// Terasic grants permission to use and modify this code for use
// in synthesis for all Terasic Development Boards and Altera Development
// Kits made by Terasic. Other use of this code, including the selling
// ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
// This VHDL/Verilog or C/C++ source code is intended as a design reference
// which illustrates how these types of functions can be implemented.
// It is the user's responsibility to verify their design for
// consistency and functionality through the use of formal
// verification methods. Terasic provides no warranty regarding the use
// or functionality of this code.
//
// ============================================================================
//
// Terasic Technologies Inc
//  9F., No.176, Sec.2, Gongdao 5th Rd, East Dist, Hsinchu City, 30070. Taiwan
// HsinChu County, Taiwan
// 302
//
// web: http://www.terasic.com/
// email: support@terasic.com
//
// ============================================================================
// Major Functions: This function is used for configuring si570 register value via 
// i2c_bus_controller .
//
//
// ============================================================================
// Design Description:
//
//
//
//
// ===========================================================================
// Revision History :
// ============================================================================
// Ver :| Author :| Mod. Date :| Changes Made:
// V1.0:|        :| 5/01/21   :| Initial Version
// ============================================================================

`define REG_NUM           418
`define DEVICE_SLAVE_ID 7'b1101111 //DE10_Agilex
//`define DEVICE_SLAVE_ID 7'b1101100 //EVB

// XA(48MHz) --> DSPLLx free run(no clean) --> OUT2/6(7) --> FPGA XCVR bank
`define XCVR_644M53125     4'h0  
`define XCVR_625M          4'h1  
`define XCVR_322M265625    4'h2  
`define XCVR_312M5         4'h3  
`define XCVR_307M2         4'h4  
`define XCVR_250M          4'h5  
`define XCVR_184M32        4'h6 
`define XCVR_156M25        4'h7  // (default)
`define XCVR_153M6         4'h8  
`define XCVR_125M          4'h9  
`define XCVR_100M          4'hA  

// XA/INx --> PLL clean --> OUT0/4 --> FPGA IO bank
`define IO_50M             4'h0  // XA 48MHz --> DSPLL free-run 50MHZ --> OUT (default)
`define IO_IN0             4'h1  // IN0 (from FPGA, assume 30.72MHz)--> DSPLLx clean --> OUT
`define IO_IN1             4'h2  // IN1 (from FPGA, assume 31.25MHz)--> DSPLLx clean --> OUT  
`define IO_IN2             4'h3  // IN2 (30.72MHz)--> DSPLLx clean --> OUT  
`define IO_IN3             4'h4  // IN3 (from UFL, assume 10MHz)--> DSPLLx clean --> OUT     


module  si5397a_i2c_reg_controller (

input                         iCLK,
input                         iRST_n,
input                         iENABLE,
input     [3:0]               XCVR0_REFCLK,   // XCVR_REF_XXX Si5397A  OUTPUT2
input     [3:0]               XCVR1_REFCLK,   // XCVR_REF_XXX Si5397A  OUTPUT6
input                         XCVR_REFCLK_RSV,// XCVR_REF_XXX Si5397A  OUTPUT7
input     [3:0]               IO0_CLK       , // IO , OUT0
input     [3:0]               IO1_CLK       , // IO , OUT4
input                         iI2C_CONTROLLER_STATE,
input                         iI2C_CONTROLLER_CONFIG_DONE,
output        [6:0]           oSLAVE_ADDR,
output        [7:0]       	   oBYTE_ADDR,
output        [7:0]           oBYTE_DATA ,
output                        oWR_CMD,
output                        oStart,
input                         iI2C_READ_DATA_RDY,
input         [7:0]           iI2C_READ_DATA,
output     reg                oONE_CLK_CONFIG_DONE,
output	  reg                oController_Ready,
output     reg   [10:0]       i2c_reg_state,
output     reg   [24:0] 	   i2c_ctrl_data,

output reg [31:0]              DELAY ,
output reg                     DONE ,
output                         access_next_i2c_reg_cmd,
output                         access_i2c_reg_start

);

//=============================================================================
// PARAMETER declarations
//=============================================================================
parameter write_cmd = 1'b1;
parameter read_cmd  = 1'b0;      
parameter NOT_LAST  = 1'b0;
//parameter LAST_REG = 1'b1;
parameter DELAY_REG = 1'b1;
//===========================================================================
// PORT declarations
//===========================================================================




//=============================================================================
// REG/WIRE declarations
//=============================================================================

//reg   [10:0]   i2c_reg_state;
wire  [6:0 ]   slave_addr   = `DEVICE_SLAVE_ID;

//reg   [24:0] 	i2c_ctrl_data;// end_reg_instruction ,  slave_addr(7bit) + byte_addr(8bit) + byte_data(8bit)+ wr_cmd (1bit) = 24bit

//wire        access_next_i2c_reg_cmd;
//wire        access_i2c_reg_start;

wire        i2c_controller_config_done_n;
wire        i2c_controller_config_done_p;

//wire        oONE_CLK_CONFIG_DONE;
//reg	        oController_Ready;

assign  oSLAVE_ADDR = i2c_ctrl_data[23:17];
assign  oBYTE_ADDR  = i2c_ctrl_data[16:9];
assign  oBYTE_DATA  = i2c_ctrl_data[8:1];
assign  oWR_CMD     = i2c_ctrl_data[0];
assign  oStart      = access_next_i2c_reg_cmd;
//----
//=============    wire   all  reg  =========================
wire	[7:0]	
REG_0018,
REG_0019,
REG_001A,
REG_002C,
REG_002E,
REG_002F,
REG_0030,
REG_0031,
REG_0032,
REG_0033,
REG_0034,
REG_0035,
REG_0036,
REG_0037,
REG_0038,
REG_0039,
REG_003A,
REG_003B,
REG_003C,
REG_003D,
REG_003E,
REG_003F,
REG_0040,
REG_0041,
REG_0042,
REG_0043,
REG_0044,
REG_0046,
REG_0047,
REG_0048,
REG_0049,
REG_004A,
REG_004B,
REG_004C,
REG_004D,
REG_004E,
REG_004F,
REG_0050,
REG_0051,
REG_0052,
REG_0053,
REG_0054,
REG_0055,
REG_0056,
REG_0057,
REG_0058,
REG_0059,
REG_005A,
REG_005B,
REG_005C,
REG_005D,
REG_005E,
REG_005F,
REG_0060,
REG_0061,
REG_0062,
REG_0063,
REG_0064,
REG_0065,
REG_0066,
REG_0067,
REG_0068,
REG_0069,
REG_0092,
REG_0094,
REG_0097,
REG_0099,
REG_009A,
REG_009C,
REG_009D,
REG_009F,
REG_00A1,
REG_00AE,
REG_00AF,
REG_00B0,
REG_00B1,
REG_00B3,
REG_00B4,
REG_00B5,
REG_00B6,
REG_00EE,
REG_00EF,
REG_00F0,
REG_00F1,
REG_00F2,
REG_00F3,
REG_00F4,
REG_00F5,
REG_0108,
REG_010B,
REG_0117,
REG_011A,
REG_0126,
REG_0129,
REG_0130,
REG_0133,
REG_013A,
REG_013D,
REG_0206,
REG_0208,
REG_0209,
REG_020A,
REG_020B,
REG_020C,
REG_020D,
REG_020E,
REG_020F,
REG_0210,
REG_0211,
REG_0212,
REG_0213,
REG_0214,
REG_0215,
REG_0216,
REG_0217,
REG_0218,
REG_0219,
REG_021A,
REG_021B,
REG_021C,
REG_021D,
REG_021E,
REG_021F,
REG_0220,
REG_0221,
REG_0222,
REG_0223,
REG_0224,
REG_0225,
REG_0226,
REG_0227,
REG_0228,
REG_0229,
REG_022A,
REG_022B,
REG_022C,
REG_022D,
REG_022E,
REG_022F,
REG_0230,
REG_0231,
REG_0232,
REG_0233,
REG_0234,
REG_0235,
REG_0236,
REG_0237,
REG_0238,
REG_0239,
REG_023A,
REG_023B,
REG_023C,
REG_023D,
REG_023E,
REG_023F,
REG_024A,
REG_024B,
REG_024C,
REG_0253,
REG_0254,
REG_0255,
REG_025C,
REG_025D,
REG_025E,
REG_0262,
REG_0263,
REG_0264,
REG_0268,
REG_0269,
REG_026A,
REG_0292,
REG_0293,
REG_0295,
REG_0296,
REG_0297,
REG_0299,
REG_02A0,
REG_02A1,
REG_02A2,
REG_02A3,
REG_02A4,
REG_02A5,
REG_02AC,
REG_02AD,
REG_02AE,
REG_02AF,
REG_02B0,
REG_02B1,
REG_0302,
REG_0303,
REG_0304,
REG_0305,
REG_0306,
REG_0307,
REG_0308,
REG_0309,
REG_030A,
REG_030B,
REG_030C,
REG_030D,
REG_030E,
REG_030F,
REG_0310,
REG_0311,
REG_0312,
REG_0313,
REG_0314,
REG_0315,
REG_0316,
REG_0317,
REG_0318,
REG_0319,
REG_031A,
REG_031B,
REG_031C,
REG_031D,
REG_031E,
REG_031F,
REG_0320,
REG_0321,
REG_0322,
REG_0323,
REG_0324,
REG_0325,
REG_0326,
REG_0327,
REG_0328,
REG_0329,
REG_032A,
REG_032B,
REG_032C,
REG_032D,
REG_0338,
REG_0420,
REG_0520,
REG_0608,
REG_0609,
REG_060A,
REG_060B,
REG_060C,
REG_060D,
REG_060E,
REG_060F,
REG_0610,
REG_0611,
REG_0612,
REG_0613,
REG_0615,
REG_0616,
REG_0617,
REG_0618,
REG_0619,
REG_061A,
REG_061B,
REG_061C,
REG_061D,
REG_061E,
REG_061F,
REG_0620,
REG_0621,
REG_0622,
REG_0623,
REG_0624,
REG_0625,
REG_0626,
REG_0627,
REG_0628,
REG_0629,
REG_062A,
REG_062B,
REG_062C,
REG_062E,
REG_062F,
REG_0632,
REG_0633,
REG_0634,
REG_0635,
REG_0636,
REG_063A,
REG_063B,
REG_063C,
REG_063D,
REG_063E,
REG_0688,
REG_0689,
REG_068A,
REG_068B,
REG_068C,
REG_068D,
REG_069B,
REG_069C,
REG_069D,
REG_069E,
REG_069F,
REG_06A0,
REG_06A1,
REG_06A2,
REG_06A4,
REG_06A5,
REG_06A6,
REG_06AC,
REG_06AD,
REG_06AE,
REG_06B1,
REG_06B2,
REG_0709,
REG_070A,
REG_070B,
REG_070C,
REG_070D,
REG_070E,
REG_070F,
REG_0710,
REG_0711,
REG_0712,
REG_0713,
REG_0714,
REG_0716,
REG_0717,
REG_0718,
REG_0719,
REG_071A,
REG_071B,
REG_071C,
REG_071D,
REG_071E,
REG_071F,
REG_0720,
REG_0721,
REG_0722,
REG_0723,
REG_0724,
REG_0725,
REG_0726,
REG_0727,
REG_0728,
REG_0729,
REG_072A,
REG_072B,
REG_072C,
REG_072D,
REG_072F,
REG_0730,
REG_0733,
REG_0734,
REG_0735,
REG_0736,
REG_0737,
REG_073B,
REG_073C,
REG_073D,
REG_073E,
REG_073F,
REG_0788,
REG_0789,
REG_078A,
REG_078B,
REG_078C,
REG_078D,
REG_079B,
REG_079C,
REG_079D,
REG_079E,
REG_079F,
REG_07A0,
REG_07A1,
REG_07A2,
REG_07A4,
REG_07A5,
REG_07A6,
REG_07AC,
REG_07AD,
REG_07AE,
REG_07B1,
REG_07B2,
REG_0822,
REG_0831,
REG_090E,
REG_0949,
REG_094A,
REG_0B44,
REG_0B47,
REG_0B48,
REG_0C03,
REG_0C09,
REG_0C0A,
REG_0C0B,
REG_0C0C;

wire [7:0] REG_ADDR_PAGE  =   8'h01;
wire [7:0] REG_ADDR_0018  =   8'h18;
wire [7:0] REG_ADDR_0019  =   8'h19;
wire [7:0] REG_ADDR_001A  =   8'h1A;
wire [7:0] REG_ADDR_002C  =   8'h2C;
wire [7:0] REG_ADDR_002E  =   8'h2E;
wire [7:0] REG_ADDR_002F  =   8'h2F;
wire [7:0] REG_ADDR_0030  =   8'h30;
wire [7:0] REG_ADDR_0031  =   8'h31;
wire [7:0] REG_ADDR_0032  =   8'h32;
wire [7:0] REG_ADDR_0033  =   8'h33;
wire [7:0] REG_ADDR_0034  =   8'h34;
wire [7:0] REG_ADDR_0035  =   8'h35;
wire [7:0] REG_ADDR_0036  =   8'h36;
wire [7:0] REG_ADDR_0037  =   8'h37;
wire [7:0] REG_ADDR_0038  =   8'h38;
wire [7:0] REG_ADDR_0039  =   8'h39;
wire [7:0] REG_ADDR_003A  =   8'h3A;
wire [7:0] REG_ADDR_003B  =   8'h3B;
wire [7:0] REG_ADDR_003C  =   8'h3C;
wire [7:0] REG_ADDR_003D  =   8'h3D;
wire [7:0] REG_ADDR_003E  =   8'h3E;
wire [7:0] REG_ADDR_003F  =   8'h3F;
wire [7:0] REG_ADDR_0040  =   8'h40;
wire [7:0] REG_ADDR_0041  =   8'h41;
wire [7:0] REG_ADDR_0042  =   8'h42;
wire [7:0] REG_ADDR_0043  =   8'h43;
wire [7:0] REG_ADDR_0044  =   8'h44;
wire [7:0] REG_ADDR_0046  =   8'h46;
wire [7:0] REG_ADDR_0047  =   8'h47;
wire [7:0] REG_ADDR_0048  =   8'h48;
wire [7:0] REG_ADDR_0049  =   8'h49;
wire [7:0] REG_ADDR_004A  =   8'h4A;
wire [7:0] REG_ADDR_004B  =   8'h4B;
wire [7:0] REG_ADDR_004C  =   8'h4C;
wire [7:0] REG_ADDR_004D  =   8'h4D;
wire [7:0] REG_ADDR_004E  =   8'h4E;
wire [7:0] REG_ADDR_004F  =   8'h4F;
wire [7:0] REG_ADDR_0050  =   8'h50;
wire [7:0] REG_ADDR_0051  =   8'h51;
wire [7:0] REG_ADDR_0052  =   8'h52;
wire [7:0] REG_ADDR_0053  =   8'h53;
wire [7:0] REG_ADDR_0054  =   8'h54;
wire [7:0] REG_ADDR_0055  =   8'h55;
wire [7:0] REG_ADDR_0056  =   8'h56;
wire [7:0] REG_ADDR_0057  =   8'h57;
wire [7:0] REG_ADDR_0058  =   8'h58;
wire [7:0] REG_ADDR_0059  =   8'h59;
wire [7:0] REG_ADDR_005A  =   8'h5A;
wire [7:0] REG_ADDR_005B  =   8'h5B;
wire [7:0] REG_ADDR_005C  =   8'h5C;
wire [7:0] REG_ADDR_005D  =   8'h5D;
wire [7:0] REG_ADDR_005E  =   8'h5E;
wire [7:0] REG_ADDR_005F  =   8'h5F;
wire [7:0] REG_ADDR_0060  =   8'h60;
wire [7:0] REG_ADDR_0061  =   8'h61;
wire [7:0] REG_ADDR_0062  =   8'h62;
wire [7:0] REG_ADDR_0063  =   8'h63;
wire [7:0] REG_ADDR_0064  =   8'h64;
wire [7:0] REG_ADDR_0065  =   8'h65;
wire [7:0] REG_ADDR_0066  =   8'h66;
wire [7:0] REG_ADDR_0067  =   8'h67;
wire [7:0] REG_ADDR_0068  =   8'h68;
wire [7:0] REG_ADDR_0069  =   8'h69;
wire [7:0] REG_ADDR_0092  =   8'h92;
wire [7:0] REG_ADDR_0094  =   8'h94;
wire [7:0] REG_ADDR_0097  =   8'h97;
wire [7:0] REG_ADDR_0099  =   8'h99;
wire [7:0] REG_ADDR_009A  =   8'h9A;
wire [7:0] REG_ADDR_009C  =   8'h9C;
wire [7:0] REG_ADDR_009D  =   8'h9D;
wire [7:0] REG_ADDR_009F  =   8'h9F;
wire [7:0] REG_ADDR_00A1  =   8'hA1;
wire [7:0] REG_ADDR_00AE  =   8'hAE;
wire [7:0] REG_ADDR_00AF  =   8'hAF;
wire [7:0] REG_ADDR_00B0  =   8'hB0;
wire [7:0] REG_ADDR_00B1  =   8'hB1;
wire [7:0] REG_ADDR_00B3  =   8'hB3;
wire [7:0] REG_ADDR_00B4  =   8'hB4;
wire [7:0] REG_ADDR_00B5  =   8'hB5;
wire [7:0] REG_ADDR_00B6  =   8'hB6;
wire [7:0] REG_ADDR_00EE  =   8'hEE;
wire [7:0] REG_ADDR_00EF  =   8'hEF;
wire [7:0] REG_ADDR_00F0  =   8'hF0;
wire [7:0] REG_ADDR_00F1  =   8'hF1;
wire [7:0] REG_ADDR_00F2  =   8'hF2;
wire [7:0] REG_ADDR_00F3  =   8'hF3;
wire [7:0] REG_ADDR_00F4  =   8'hF4;
wire [7:0] REG_ADDR_00F5  =   8'hF5;
wire [7:0] REG_ADDR_0108  =   8'h08;
wire [7:0] REG_ADDR_010B  =   8'h0B;
wire [7:0] REG_ADDR_0117  =   8'h17;
wire [7:0] REG_ADDR_011A  =   8'h1A;
wire [7:0] REG_ADDR_0126  =   8'h26;
wire [7:0] REG_ADDR_0129  =   8'h29;
wire [7:0] REG_ADDR_0130  =   8'h30;
wire [7:0] REG_ADDR_0133  =   8'h33;
wire [7:0] REG_ADDR_013A  =   8'h3A;
wire [7:0] REG_ADDR_013D  =   8'h3D;
wire [7:0] REG_ADDR_0206  =   8'h06;
wire [7:0] REG_ADDR_0208  =   8'h08;
wire [7:0] REG_ADDR_0209  =   8'h09;
wire [7:0] REG_ADDR_020A  =   8'h0A;
wire [7:0] REG_ADDR_020B  =   8'h0B;
wire [7:0] REG_ADDR_020C  =   8'h0C;
wire [7:0] REG_ADDR_020D  =   8'h0D;
wire [7:0] REG_ADDR_020E  =   8'h0E;
wire [7:0] REG_ADDR_020F  =   8'h0F;
wire [7:0] REG_ADDR_0210  =   8'h10;
wire [7:0] REG_ADDR_0211  =   8'h11;
wire [7:0] REG_ADDR_0212  =   8'h12;
wire [7:0] REG_ADDR_0213  =   8'h13;
wire [7:0] REG_ADDR_0214  =   8'h14;
wire [7:0] REG_ADDR_0215  =   8'h15;
wire [7:0] REG_ADDR_0216  =   8'h16;
wire [7:0] REG_ADDR_0217  =   8'h17;
wire [7:0] REG_ADDR_0218  =   8'h18;
wire [7:0] REG_ADDR_0219  =   8'h19;
wire [7:0] REG_ADDR_021A  =   8'h1A;
wire [7:0] REG_ADDR_021B  =   8'h1B;
wire [7:0] REG_ADDR_021C  =   8'h1C;
wire [7:0] REG_ADDR_021D  =   8'h1D;
wire [7:0] REG_ADDR_021E  =   8'h1E;
wire [7:0] REG_ADDR_021F  =   8'h1F;
wire [7:0] REG_ADDR_0220  =   8'h20;
wire [7:0] REG_ADDR_0221  =   8'h21;
wire [7:0] REG_ADDR_0222  =   8'h22;
wire [7:0] REG_ADDR_0223  =   8'h23;
wire [7:0] REG_ADDR_0224  =   8'h24;
wire [7:0] REG_ADDR_0225  =   8'h25;
wire [7:0] REG_ADDR_0226  =   8'h26;
wire [7:0] REG_ADDR_0227  =   8'h27;
wire [7:0] REG_ADDR_0228  =   8'h28;
wire [7:0] REG_ADDR_0229  =   8'h29;
wire [7:0] REG_ADDR_022A  =   8'h2A;
wire [7:0] REG_ADDR_022B  =   8'h2B;
wire [7:0] REG_ADDR_022C  =   8'h2C;
wire [7:0] REG_ADDR_022D  =   8'h2D;
wire [7:0] REG_ADDR_022E  =   8'h2E;
wire [7:0] REG_ADDR_022F  =   8'h2F;
wire [7:0] REG_ADDR_0230  =   8'h30;
wire [7:0] REG_ADDR_0231  =   8'h31;
wire [7:0] REG_ADDR_0232  =   8'h32;
wire [7:0] REG_ADDR_0233  =   8'h33;
wire [7:0] REG_ADDR_0234  =   8'h34;
wire [7:0] REG_ADDR_0235  =   8'h35;
wire [7:0] REG_ADDR_0236  =   8'h36;
wire [7:0] REG_ADDR_0237  =   8'h37;
wire [7:0] REG_ADDR_0238  =   8'h38;
wire [7:0] REG_ADDR_0239  =   8'h39;
wire [7:0] REG_ADDR_023A  =   8'h3A;
wire [7:0] REG_ADDR_023B  =   8'h3B;
wire [7:0] REG_ADDR_023C  =   8'h3C;
wire [7:0] REG_ADDR_023D  =   8'h3D;
wire [7:0] REG_ADDR_023E  =   8'h3E;
wire [7:0] REG_ADDR_023F  =   8'h3F;
wire [7:0] REG_ADDR_024A  =   8'h4A;
wire [7:0] REG_ADDR_024B  =   8'h4B;
wire [7:0] REG_ADDR_024C  =   8'h4C;
wire [7:0] REG_ADDR_0253  =   8'h53;
wire [7:0] REG_ADDR_0254  =   8'h54;
wire [7:0] REG_ADDR_0255  =   8'h55;
wire [7:0] REG_ADDR_025C  =   8'h5C;
wire [7:0] REG_ADDR_025D  =   8'h5D;
wire [7:0] REG_ADDR_025E  =   8'h5E;
wire [7:0] REG_ADDR_0262  =   8'h62;
wire [7:0] REG_ADDR_0263  =   8'h63;
wire [7:0] REG_ADDR_0264  =   8'h64;
wire [7:0] REG_ADDR_0268  =   8'h68;
wire [7:0] REG_ADDR_0269  =   8'h69;
wire [7:0] REG_ADDR_026A  =   8'h6A;
wire [7:0] REG_ADDR_0292  =   8'h92;
wire [7:0] REG_ADDR_0293  =   8'h93;
wire [7:0] REG_ADDR_0295  =   8'h95;
wire [7:0] REG_ADDR_0296  =   8'h96;
wire [7:0] REG_ADDR_0297  =   8'h97;
wire [7:0] REG_ADDR_0299  =   8'h99;
wire [7:0] REG_ADDR_02A0  =   8'hA0;
wire [7:0] REG_ADDR_02A1  =   8'hA1;
wire [7:0] REG_ADDR_02A2  =   8'hA2;
wire [7:0] REG_ADDR_02A3  =   8'hA3;
wire [7:0] REG_ADDR_02A4  =   8'hA4;
wire [7:0] REG_ADDR_02A5  =   8'hA5;
wire [7:0] REG_ADDR_02AC  =   8'hAC;
wire [7:0] REG_ADDR_02AD  =   8'hAD;
wire [7:0] REG_ADDR_02AE  =   8'hAE;
wire [7:0] REG_ADDR_02AF  =   8'hAF;
wire [7:0] REG_ADDR_02B0  =   8'hB0;
wire [7:0] REG_ADDR_02B1  =   8'hB1;
wire [7:0] REG_ADDR_0302  =   8'h02;
wire [7:0] REG_ADDR_0303  =   8'h03;
wire [7:0] REG_ADDR_0304  =   8'h04;
wire [7:0] REG_ADDR_0305  =   8'h05;
wire [7:0] REG_ADDR_0306  =   8'h06;
wire [7:0] REG_ADDR_0307  =   8'h07;
wire [7:0] REG_ADDR_0308  =   8'h08;
wire [7:0] REG_ADDR_0309  =   8'h09;
wire [7:0] REG_ADDR_030A  =   8'h0A;
wire [7:0] REG_ADDR_030B  =   8'h0B;
wire [7:0] REG_ADDR_030C  =   8'h0C;
wire [7:0] REG_ADDR_030D  =   8'h0D;
wire [7:0] REG_ADDR_030E  =   8'h0E;
wire [7:0] REG_ADDR_030F  =   8'h0F;
wire [7:0] REG_ADDR_0310  =   8'h10;
wire [7:0] REG_ADDR_0311  =   8'h11;
wire [7:0] REG_ADDR_0312  =   8'h12;
wire [7:0] REG_ADDR_0313  =   8'h13;
wire [7:0] REG_ADDR_0314  =   8'h14;
wire [7:0] REG_ADDR_0315  =   8'h15;
wire [7:0] REG_ADDR_0316  =   8'h16;
wire [7:0] REG_ADDR_0317  =   8'h17;
wire [7:0] REG_ADDR_0318  =   8'h18;
wire [7:0] REG_ADDR_0319  =   8'h19;
wire [7:0] REG_ADDR_031A  =   8'h1A;
wire [7:0] REG_ADDR_031B  =   8'h1B;
wire [7:0] REG_ADDR_031C  =   8'h1C;
wire [7:0] REG_ADDR_031D  =   8'h1D;
wire [7:0] REG_ADDR_031E  =   8'h1E;
wire [7:0] REG_ADDR_031F  =   8'h1F;
wire [7:0] REG_ADDR_0320  =   8'h20;
wire [7:0] REG_ADDR_0321  =   8'h21;
wire [7:0] REG_ADDR_0322  =   8'h22;
wire [7:0] REG_ADDR_0323  =   8'h23;
wire [7:0] REG_ADDR_0324  =   8'h24;
wire [7:0] REG_ADDR_0325  =   8'h25;
wire [7:0] REG_ADDR_0326  =   8'h26;
wire [7:0] REG_ADDR_0327  =   8'h27;
wire [7:0] REG_ADDR_0328  =   8'h28;
wire [7:0] REG_ADDR_0329  =   8'h29;
wire [7:0] REG_ADDR_032A  =   8'h2A;
wire [7:0] REG_ADDR_032B  =   8'h2B;
wire [7:0] REG_ADDR_032C  =   8'h2C;
wire [7:0] REG_ADDR_032D  =   8'h2D;
wire [7:0] REG_ADDR_0338  =   8'h38;
wire [7:0] REG_ADDR_0420  =   8'h20;
wire [7:0] REG_ADDR_0520  =   8'h20;
wire [7:0] REG_ADDR_0608  =   8'h08;
wire [7:0] REG_ADDR_0609  =   8'h09;
wire [7:0] REG_ADDR_060A  =   8'h0A;
wire [7:0] REG_ADDR_060B  =   8'h0B;
wire [7:0] REG_ADDR_060C  =   8'h0C;
wire [7:0] REG_ADDR_060D  =   8'h0D;
wire [7:0] REG_ADDR_060E  =   8'h0E;
wire [7:0] REG_ADDR_060F  =   8'h0F;
wire [7:0] REG_ADDR_0610  =   8'h10;
wire [7:0] REG_ADDR_0611  =   8'h11;
wire [7:0] REG_ADDR_0612  =   8'h12;
wire [7:0] REG_ADDR_0613  =   8'h13;
wire [7:0] REG_ADDR_0615  =   8'h15;
wire [7:0] REG_ADDR_0616  =   8'h16;
wire [7:0] REG_ADDR_0617  =   8'h17;
wire [7:0] REG_ADDR_0618  =   8'h18;
wire [7:0] REG_ADDR_0619  =   8'h19;
wire [7:0] REG_ADDR_061A  =   8'h1A;
wire [7:0] REG_ADDR_061B  =   8'h1B;
wire [7:0] REG_ADDR_061C  =   8'h1C;
wire [7:0] REG_ADDR_061D  =   8'h1D;
wire [7:0] REG_ADDR_061E  =   8'h1E;
wire [7:0] REG_ADDR_061F  =   8'h1F;
wire [7:0] REG_ADDR_0620  =   8'h20;
wire [7:0] REG_ADDR_0621  =   8'h21;
wire [7:0] REG_ADDR_0622  =   8'h22;
wire [7:0] REG_ADDR_0623  =   8'h23;
wire [7:0] REG_ADDR_0624  =   8'h24;
wire [7:0] REG_ADDR_0625  =   8'h25;
wire [7:0] REG_ADDR_0626  =   8'h26;
wire [7:0] REG_ADDR_0627  =   8'h27;
wire [7:0] REG_ADDR_0628  =   8'h28;
wire [7:0] REG_ADDR_0629  =   8'h29;
wire [7:0] REG_ADDR_062A  =   8'h2A;
wire [7:0] REG_ADDR_062B  =   8'h2B;
wire [7:0] REG_ADDR_062C  =   8'h2C;
wire [7:0] REG_ADDR_062E  =   8'h2E;
wire [7:0] REG_ADDR_062F  =   8'h2F;
wire [7:0] REG_ADDR_0632  =   8'h32;
wire [7:0] REG_ADDR_0633  =   8'h33;
wire [7:0] REG_ADDR_0634  =   8'h34;
wire [7:0] REG_ADDR_0635  =   8'h35;
wire [7:0] REG_ADDR_0636  =   8'h36;
wire [7:0] REG_ADDR_063A  =   8'h3A;
wire [7:0] REG_ADDR_063B  =   8'h3B;
wire [7:0] REG_ADDR_063C  =   8'h3C;
wire [7:0] REG_ADDR_063D  =   8'h3D;
wire [7:0] REG_ADDR_063E  =   8'h3E;
wire [7:0] REG_ADDR_0688  =   8'h88;
wire [7:0] REG_ADDR_0689  =   8'h89;
wire [7:0] REG_ADDR_068A  =   8'h8A;
wire [7:0] REG_ADDR_068B  =   8'h8B;
wire [7:0] REG_ADDR_068C  =   8'h8C;
wire [7:0] REG_ADDR_068D  =   8'h8D;
wire [7:0] REG_ADDR_069B  =   8'h9B;
wire [7:0] REG_ADDR_069C  =   8'h9C;
wire [7:0] REG_ADDR_069D  =   8'h9D;
wire [7:0] REG_ADDR_069E  =   8'h9E;
wire [7:0] REG_ADDR_069F  =   8'h9F;
wire [7:0] REG_ADDR_06A0  =   8'hA0;
wire [7:0] REG_ADDR_06A1  =   8'hA1;
wire [7:0] REG_ADDR_06A2  =   8'hA2;
wire [7:0] REG_ADDR_06A4  =   8'hA4;
wire [7:0] REG_ADDR_06A5  =   8'hA5;
wire [7:0] REG_ADDR_06A6  =   8'hA6;
wire [7:0] REG_ADDR_06AC  =   8'hAC;
wire [7:0] REG_ADDR_06AD  =   8'hAD;
wire [7:0] REG_ADDR_06AE  =   8'hAE;
wire [7:0] REG_ADDR_06B1  =   8'hB1;
wire [7:0] REG_ADDR_06B2  =   8'hB2;
wire [7:0] REG_ADDR_0709  =   8'h09;
wire [7:0] REG_ADDR_070A  =   8'h0A;
wire [7:0] REG_ADDR_070B  =   8'h0B;
wire [7:0] REG_ADDR_070C  =   8'h0C;
wire [7:0] REG_ADDR_070D  =   8'h0D;
wire [7:0] REG_ADDR_070E  =   8'h0E;
wire [7:0] REG_ADDR_070F  =   8'h0F;
wire [7:0] REG_ADDR_0710  =   8'h10;
wire [7:0] REG_ADDR_0711  =   8'h11;
wire [7:0] REG_ADDR_0712  =   8'h12;
wire [7:0] REG_ADDR_0713  =   8'h13;
wire [7:0] REG_ADDR_0714  =   8'h14;
wire [7:0] REG_ADDR_0716  =   8'h16;
wire [7:0] REG_ADDR_0717  =   8'h17;
wire [7:0] REG_ADDR_0718  =   8'h18;
wire [7:0] REG_ADDR_0719  =   8'h19;
wire [7:0] REG_ADDR_071A  =   8'h1A;
wire [7:0] REG_ADDR_071B  =   8'h1B;
wire [7:0] REG_ADDR_071C  =   8'h1C;
wire [7:0] REG_ADDR_071D  =   8'h1D;
wire [7:0] REG_ADDR_071E  =   8'h1E;
wire [7:0] REG_ADDR_071F  =   8'h1F;
wire [7:0] REG_ADDR_0720  =   8'h20;
wire [7:0] REG_ADDR_0721  =   8'h21;
wire [7:0] REG_ADDR_0722  =   8'h22;
wire [7:0] REG_ADDR_0723  =   8'h23;
wire [7:0] REG_ADDR_0724  =   8'h24;
wire [7:0] REG_ADDR_0725  =   8'h25;
wire [7:0] REG_ADDR_0726  =   8'h26;
wire [7:0] REG_ADDR_0727  =   8'h27;
wire [7:0] REG_ADDR_0728  =   8'h28;
wire [7:0] REG_ADDR_0729  =   8'h29;
wire [7:0] REG_ADDR_072A  =   8'h2A;
wire [7:0] REG_ADDR_072B  =   8'h2B;
wire [7:0] REG_ADDR_072C  =   8'h2C;
wire [7:0] REG_ADDR_072D  =   8'h2D;
wire [7:0] REG_ADDR_072F  =   8'h2F;
wire [7:0] REG_ADDR_0730  =   8'h30;
wire [7:0] REG_ADDR_0733  =   8'h33;
wire [7:0] REG_ADDR_0734  =   8'h34;
wire [7:0] REG_ADDR_0735  =   8'h35;
wire [7:0] REG_ADDR_0736  =   8'h36;
wire [7:0] REG_ADDR_0737  =   8'h37;
wire [7:0] REG_ADDR_073B  =   8'h3B;
wire [7:0] REG_ADDR_073C  =   8'h3C;
wire [7:0] REG_ADDR_073D  =   8'h3D;
wire [7:0] REG_ADDR_073E  =   8'h3E;
wire [7:0] REG_ADDR_073F  =   8'h3F;
wire [7:0] REG_ADDR_0788  =   8'h88;
wire [7:0] REG_ADDR_0789  =   8'h89;
wire [7:0] REG_ADDR_078A  =   8'h8A;
wire [7:0] REG_ADDR_078B  =   8'h8B;
wire [7:0] REG_ADDR_078C  =   8'h8C;
wire [7:0] REG_ADDR_078D  =   8'h8D;
wire [7:0] REG_ADDR_079B  =   8'h9B;
wire [7:0] REG_ADDR_079C  =   8'h9C;
wire [7:0] REG_ADDR_079D  =   8'h9D;
wire [7:0] REG_ADDR_079E  =   8'h9E;
wire [7:0] REG_ADDR_079F  =   8'h9F;
wire [7:0] REG_ADDR_07A0  =   8'hA0;
wire [7:0] REG_ADDR_07A1  =   8'hA1;
wire [7:0] REG_ADDR_07A2  =   8'hA2;
wire [7:0] REG_ADDR_07A4  =   8'hA4;
wire [7:0] REG_ADDR_07A5  =   8'hA5;
wire [7:0] REG_ADDR_07A6  =   8'hA6;
wire [7:0] REG_ADDR_07AC  =   8'hAC;
wire [7:0] REG_ADDR_07AD  =   8'hAD;
wire [7:0] REG_ADDR_07AE  =   8'hAE;
wire [7:0] REG_ADDR_07B1  =   8'hB1;
wire [7:0] REG_ADDR_07B2  =   8'hB2;
wire [7:0] REG_ADDR_0822  =   8'h22;
wire [7:0] REG_ADDR_0831  =   8'h31;
wire [7:0] REG_ADDR_090E  =   8'h0E;
wire [7:0] REG_ADDR_0949  =   8'h49;
wire [7:0] REG_ADDR_094A  =   8'h4A;
wire [7:0] REG_ADDR_0B44  =   8'h44;
wire [7:0] REG_ADDR_0B47  =   8'h47;
wire [7:0] REG_ADDR_0B48  =   8'h48;
wire [7:0] REG_ADDR_0C03  =   8'h03;
wire [7:0] REG_ADDR_0C09  =   8'h09;
wire [7:0] REG_ADDR_0C0A  =   8'h0A;
wire [7:0] REG_ADDR_0C0B  =   8'h0B;
wire [7:0] REG_ADDR_0C0C  =   8'h0C;



//
wire [7:0]   FIXREGSD10             =  ( ( IO0_CLK ==`IO_50M ) || ( IO1_CLK ==`IO_50M )) ?   1 :   0 ; 
wire [1:0]   CLK_SWITCH_MODE_PLLC   =  ( IO0_CLK ==`IO_50M )? 2:  0 ;
wire         HSW_EN_PLLC            =     1  ;
wire [3:0]   P0_FRACN_MODE   =            11;             
wire         P0_FRACN_EN     =            1 ; 
wire[3:0]    P1_FRACN_MODE   =            11  ;              
wire         P1_FRACN_EN     =            1   ;              
wire [3:0]   P2_FRACN_MODE   =            11  ;          
wire         P2_FRACN_EN     =            1   ;          
wire[3:0]    P3_FRACN_MODE   =            11  ;                     
wire         P3_FRACN_EN     =            0   ;

assign       REG_0636 = { 5'h0 , HSW_EN_PLLC, CLK_SWITCH_MODE_PLLC } ;
assign       REG_0822 = FIXREGSD10; 
assign REG_0231 = { 3'h0 , P0_FRACN_EN ,P0_FRACN_MODE[3:0]  } ;             
assign REG_0232 = { 3'h0 , P1_FRACN_EN ,P1_FRACN_MODE[3:0]  } ; 
assign REG_0233 = { 3'h0 , P2_FRACN_EN ,P2_FRACN_MODE[3:0]  } ; 
assign REG_0234 = { 3'h0 , P3_FRACN_EN ,P3_FRACN_MODE[3:0]  } ; 
//---no free run setting 

wire [3:0] LOS_INTR_MSK        = 0;
wire [3:0] OOF_INTR_MSK        = 0;

wire       LOL_INTR_MSK_PLLA   = 1 ;
wire       LOL_INTR_MSK_PLLB   = 1 ;
wire       LOL_INTR_MSK_PLLC   = 0 ;
wire       LOL_INTR_MSK_PLLD   = 0 ;

wire       HOLD_INTR_MSK_PLLA  = 1 ;
wire       HOLD_INTR_MSK_PLLB  = 1 ;
wire       HOLD_INTR_MSK_PLLC  = 0 ;
wire       HOLD_INTR_MSK_PLLD  = 0 ;

wire       CAL_INTR_MSK_PLLA   = 1 ;
wire       CAL_INTR_MSK_PLLB   = 1 ;
wire       CAL_INTR_MSK_PLLC   = 0 ;
wire       CAL_INTR_MSK_PLLD   = 0 ;

wire [3:0] LOS_EN              = 15  ;//3
wire       LOSXAXB_DIS         = 0  ;



wire [15:0]  LOS0_TRG_THR      =114; 
wire [15:0]  LOS1_TRG_THR      =114; 
wire [15:0]  LOS2_TRG_THR      =114; 
wire [15:0]  LOS3_TRG_THR      =114; 

wire [15:0]  LOS0_CLR_THR      = 2  ;
wire [15:0]  LOS1_CLR_THR      = 2  ;

wire [15:0]  LOS2_CLR_THR      =   2 ; 
wire [15:0]  LOS3_CLR_THR      =   2 ; 

wire [3:0] LOS_MIN_PERIOD_EN   = 15 ;
wire [3:0] OOF_EN              = 15 ;
wire [3:0] FAST_OOF_EN         = 15 ;
wire [2:0] OOF_REF_SEL         = 4 ;

wire [4:0]   OOF0_DIV_SEL      =11;
wire [4:0]   OOF1_DIV_SEL      =11;
wire [4:0]   OOF2_DIV_SEL      =11;
wire [4:0]   OOF3_DIV_SEL      =10;



wire [7:0]   OOF0_SET_THR      =50;
wire [7:0]   OOF1_SET_THR      =50;
wire [7:0]   OOF2_SET_THR      =50;
wire [7:0]   OOF3_SET_THR      =50;


wire [7:0]   OOF0_CLR_THR      = 50;
wire [7:0]   OOF1_CLR_THR      = 50;
wire [7:0]   OOF2_CLR_THR      = 50;
wire [7:0]   OOF3_CLR_THR      = 50;

wire [2:0]   OOF0_DETWIN_SEL     = 5 ;
wire [6:4]   OOF1_DETWIN_SEL   = 5  ;
wire [2:0]   OOF2_DETWIN_SEL   = 5  ;
wire [6:4]   OOF3_DETWIN_SEL   = 5  ;
wire [3:0]   OOF_ON_LOS        = 15 ;

wire [3:0] FAST_OOF0_SET_THR   = 3 ;
wire [3:0] FAST_OOF1_SET_THR   = 3 ;

wire [3:0] FAST_OOF2_SET_THR   = 3;
wire [3:0] FAST_OOF3_SET_THR   = 3;

wire [3:0] FAST_OOF0_CLR_THR   = 3 ;
wire [3:0] FAST_OOF1_CLR_THR   = 3 ;

wire [3:0]   FAST_OOF2_CLR_THR   =3 ;
wire [3:0]   FAST_OOF3_CLR_THR   =3 ;

wire [1:0] FAST_OOF0_DETWIN_SEL= 1;
wire [1:0] FAST_OOF1_DETWIN_SEL= 1;
wire [1:0] FAST_OOF2_DETWIN_SEL= 1;
wire [1:0] FAST_OOF3_DETWIN_SEL= 1;


wire [25:0]OOF0_RATIO_REF 	  =26'd21474836;
wire [25:0]OOF1_RATIO_REF	  =26'd21845333;

wire [25:0]OOF2_RATIO_REF    =26'd21474836;
wire [25:0]OOF3_RATIO_REF    =26'd13981013;

wire       LOL_FST_EN_PLLA           =0;
wire       LOL_FST_EN_PLLB           =0;
wire       LOL_FST_EN_PLLC           =( IO0_CLK ==`IO_50M )?0:1;//..
wire       LOL_FST_EN_PLLD           =( IO1_CLK ==`IO_50M )?0:1;//..
wire [3:0] LOL_FST_DETWIN_SEL_PLLC   =( IO0_CLK ==`IO_50M )?0:6  ;//..
wire [3:0] LOL_FST_DETWIN_SEL_PLLD   =( IO1_CLK ==`IO_50M )?0:6  ;//..
wire [3:0] LOL_FST_SET_THR_SEL_PLLC  =( IO0_CLK ==`IO_50M )?0:8   ;//..
wire [3:0] LOL_FST_SET_THR_SEL_PLLD  =( IO1_CLK ==`IO_50M )?0:8   ;//..
wire [3:0] LOL_FST_CLR_THR_SEL_PLLC  =( IO0_CLK ==`IO_50M )?0:6   ;//..
wire [3:0] LOL_FST_CLR_THR_SEL_PLLD  =( IO1_CLK ==`IO_50M )?0:6   ;//..

wire       LOL_SLOW_EN_PLLA          =0   ;
wire       LOL_SLOW_EN_PLLB          =0   ;
wire       LOL_SLOW_EN_PLLC          =( IO0_CLK ==`IO_50M )?0:1   ;//..
wire       LOL_SLOW_EN_PLLD          =( IO1_CLK ==`IO_50M )?0:1   ;//..
wire [3:0] LOL_SLW_DETWIN_SEL_PLLC   =( IO0_CLK ==`IO_50M )?0:3 ;//..
wire [3:0] LOL_SLW_DETWIN_SEL_PLLD   =( IO1_CLK ==`IO_50M )?0:3 ;//..

wire [1:0] LOL_SLW_VALWIN_SEL_PLLA   =0 ;
wire [1:0] LOL_SLW_VALWIN_SEL_PLLB   =0 ;

wire [3:0] LOL_SLW_SET_THR_PLLC      =( IO0_CLK ==`IO_50M )?0:4 ;//..
wire [3:0] LOL_SLW_SET_THR_PLLD      =( IO1_CLK ==`IO_50M )?0:4 ;//..
wire [3:0] LOL_SLW_CLR_THR_PLLC      =( IO0_CLK ==`IO_50M )?0:2 ;//..
wire [3:0] LOL_SLW_CLR_THR_PLLD      =( IO1_CLK ==`IO_50M )?0:2 ;//..


//-----parameter 1 
wire [28:0]LOL_CLR_DELAY_DIV256_PLLC_IN=(
(IO0_CLK == `IO_IN0 )?29'd36512:(
(IO0_CLK == `IO_IN1 )?29'd36503:(
(IO0_CLK == `IO_IN2 )?29'd36512:(
(IO0_CLK == `IO_IN3 )?29'd36510:29'd36510
))));
wire [28:0]LOL_CLR_DELAY_DIV256_PLLD_IN=(
(IO1_CLK == `IO_IN0 )?29'd36512:(
(IO1_CLK == `IO_IN1 )?29'd36503:(
(IO1_CLK == `IO_IN2 )?29'd36512:(
(IO1_CLK == `IO_IN3 )?29'd36510:29'd36510
))));






wire [28:0]LOL_CLR_DELAY_DIV256_PLLC =( IO0_CLK ==`IO_50M )?0:LOL_CLR_DELAY_DIV256_PLLC_IN ;//..
wire [28:0]LOL_CLR_DELAY_DIV256_PLLD =( IO1_CLK ==`IO_50M )?0:LOL_CLR_DELAY_DIV256_PLLD_IN ;//..

wire [28:0]FASTLOCK_EXTEND_PLLC      =( IO0_CLK ==`IO_50M )?0:29'd36066;//..
wire [28:0]FASTLOCK_EXTEND_PLLD      =( IO1_CLK ==`IO_50M )?0:29'd36066;//..


//This valid with external clock 
//0 = pre-scale value 1
//1 = pre-scale value 2
//2 = pre-scale value 4
//3 = pre-scale value 8
wire [1 :0]PXAXB                     =     0 ; // The divider value for the XAXB input

wire [47:0]P0_NUM                    =  768  ; //--300h
wire [31:0]P0_DEN                    =   25  ; 

wire [47:0]P1_NUM                    =  125  ; 
wire [31:0]P1_DEN                    =    4  ; 

wire [47:0]P2_NUM                    =  768  ; 
wire [31:0]P2_DEN                    =   25  ; 

wire [47:0]P3_NUM                    =   10  ; 
wire [31:0]P3_DEN                    =   1   ; 

wire [3:0]   OOF_STOP_ON_LOS         = 15;
wire [3:0]   OOF_CLEAR_ON_LOS        = 15;

//<------------------------------------------- 
wire [3:0] FASTLOCK_EXTEND_SCL_PLLC    =( IO0_CLK ==`IO_50M )?8:10  ;
wire [3:0] FASTLOCK_EXTEND_SCL_PLLD    =( IO1_CLK ==`IO_50M )?8:10  ;
wire       LOL_SLW_VALWIN_SELX_PLLA    = 0   ;
wire       LOL_SLW_VALWIN_SELX_PLLB    = 0   ;
wire       LOL_SLW_VALWIN_SELX_PLLC    =( IO0_CLK ==`IO_50M )?0:1   ;
wire       LOL_SLW_VALWIN_SELX_PLLD    =( IO1_CLK ==`IO_50M )?0:1   ;
wire       FASTLOCK_DLY_ONSW_EN_PLLA   = 0   ;
wire       FASTLOCK_DLY_ONSW_EN_PLLB   = 0   ;
wire       FASTLOCK_DLY_ONSW_EN_PLLC   =( IO0_CLK ==`IO_50M )?0:1   ;
wire       FASTLOCK_DLY_ONSW_EN_PLLD   =( IO1_CLK ==`IO_50M )?0:1   ;
wire       FASTLOCK_DLY_ONLOL_EN_PLLA  = 0 ;
wire       FASTLOCK_DLY_ONLOL_EN_PLLB  = 0 ;
wire       FASTLOCK_DLY_ONLOL_EN_PLLC  =( IO0_CLK ==`IO_50M )?0:1 ;
wire       FASTLOCK_DLY_ONLOL_EN_PLLD  =( IO1_CLK ==`IO_50M )?0:1 ;
wire [19:0]FASTLOCK_DLY_ONLOL_PLLC     =( IO0_CLK ==`IO_50M )?0:3604;
wire [19:0]FASTLOCK_DLY_ONLOL_PLLD     =( IO1_CLK ==`IO_50M )?0:3604;
wire [19:0]FASTLOCK_DLY_ONSW_PLLC      =( IO0_CLK ==`IO_50M )?0:4915;
wire [19:0]FASTLOCK_DLY_ONSW_PLLD      =( IO1_CLK ==`IO_50M )?0:4915 ;
//<-----------------------------------------------

//---DSP-C
wire [5:0] BW0_PLLC                    =( IO0_CLK ==`IO_50M )?0:15 ; 
wire [5:0] BW1_PLLC                    =( IO0_CLK ==`IO_50M )?0:30 ;
wire [5:0] BW2_PLLC                    =( IO0_CLK ==`IO_50M )?0:12 ;
wire [5:0] BW3_PLLC                    =( IO0_CLK ==`IO_50M )?0:11 ;
wire [5:0] BW4_PLLC                    =( IO0_CLK ==`IO_50M )?0:15 ;
wire [5:0] BW5_PLLC                    =( IO0_CLK ==`IO_50M )?0:63 ;
wire [5:0] FASTLOCK_BW0_PLLC           =( IO0_CLK ==`IO_50M )?0: 17;
wire [5:0] FASTLOCK_BW1_PLLC           =( IO0_CLK ==`IO_50M )?0: 36;
wire [5:0] FASTLOCK_BW2_PLLC           =( IO0_CLK ==`IO_50M )?0: 10 ;
wire [5:0] FASTLOCK_BW3_PLLC           =( IO0_CLK ==`IO_50M )?0: 9 ;
wire [5:0] FASTLOCK_BW4_PLLC           =( IO0_CLK ==`IO_50M )?0: 15 ;
wire [5:0] FASTLOCK_BW5_PLLC           =( IO0_CLK ==`IO_50M )?0: 63;


//-----parameter 2
wire [55:0]M_NUM_PLLC ;
assign     M_NUM_PLLC                  = (
 (IO0_CLK == `IO_50M )?0:(
 (IO0_CLK == `IO_IN0 )?56'd2473901162496:(
 (IO0_CLK == `IO_IN1 )?56'd1610612736000:(
 (IO0_CLK == `IO_IN2 )?56'd2473901162496:(
 (IO0_CLK == `IO_IN3 )?56'd1589137899520:56'd1589137899520
 )))));
wire [31:0]M_DEN_PLLC ;
assign     M_DEN_PLLC                  = (
(IO0_CLK == `IO_50M )?0:(
(IO0_CLK == `IO_IN0 )? 32'd3355443200:(
(IO0_CLK == `IO_IN1 )? 32'd2147483648:(
(IO0_CLK == `IO_IN2 )? 32'd3355443200:(
(IO0_CLK == `IO_IN3 )? 32'd2147483648:32'd2147483648
)))));




//
wire [3:0] M_FRAC_MODE_PLLC            = 11;
wire       M_FRAC_EN_PLLC              = (
  ( IO0_CLK ==`IO_50M )?0:(
  ( IO0_CLK ==`IO_IN0 )?1:(
  ( IO0_CLK ==`IO_IN1 )?0:(
  ( IO0_CLK ==`IO_IN2 )?1:(
  ( IO0_CLK ==`IO_IN3 )?0:0
  )))));
  
wire       M_FSTEP_MSK_PLLC            = 1 ;
wire       M_FSTEPW_DEN_PLLC           = 0 ;
wire [55:0]M_FSTEPW_PLLC               = 0;
wire       FASTLOCK_AUTO_EN_PLLC       = 1   ;
wire       FASTLOCK_MAN_PLLC           = 0   ;
wire [9:0]   HSW_PHMEAS_THR_PLLC       = 3 ;
wire       HOLD_RAMPPB_NOHIST_PLLC     = ( IO0_CLK ==`IO_50M )?0:1 ;
wire       HOLD_SETTLE_DET_EN_DSPLLC   = ( IO0_CLK ==`IO_50M )?0:1    ;
wire [15:0]OUT_MAX_LIMIT_LMT_DPSLLC    = (
  ( IO0_CLK ==`IO_50M )?0:( 
  ( IO0_CLK ==`IO_IN0 )?16'd20138:(
  ( IO0_CLK ==`IO_IN1 )?16'd19797:(
  ( IO0_CLK ==`IO_IN2 )?16'd20138:(
  ( IO0_CLK ==`IO_IN3 )?16'd20064 :16'd0
  )))));

//>>>.
wire [2:0] IN_SEL_PLLC                 =  ( IO0_CLK ==`IO_50M )?0 :IO0_CLK-1 ; //DSPC select input clock NUMBER (IN0~IN3) 
wire       HOLD_EN_PLLC                = 1   ;
wire       HOLD_RAMP_BYP_PLLC          = ( IO0_CLK ==`IO_50M )?1: 0   ;
wire       HOLDEXIT_BW_SEL1_PLLC       = 0   ;
wire [2:0] RAMP_STEP_INTERVAL_PLLC     = ( IO0_CLK ==`IO_50M )?0:4   ;
wire [4:0] HOLD_HIST_LEN_PLLC          = ( IO0_CLK ==`IO_50M )?0:25  ;
wire [4:0] HOLD_HIST_DELAY_PLLC        = ( IO0_CLK ==`IO_50M )?0:25  ;
wire [23:0]HOLD_15M_CYC_COUNT_PLLC     = ( IO0_CLK ==`IO_50M )?1024: 1667 ;
wire       FORCE_HOLD_PLLC             = ( IO0_CLK ==`IO_50M )?1: 0    ;
wire [1:0]   HSW_MODE_PLLC             = 1 ;   
wire [3:2]   HSW_PHMEAS_CTRL_PLLC      = 0 ;
wire [4:0] HSW_COARSE_PM_LEN_PLLC      = ( IO0_CLK ==`IO_50M )?10:4    ;
wire [4:0] HSW_COARSE_PM_DLY_PLLC      = ( IO0_CLK ==`IO_50M )?6:2    ;
wire [3:0] HSW_FINE_PM_LEN_PLLC        = ( IO0_CLK ==`IO_50M )?0:7    ;
wire [12:0]PFD_EN_DELAY_PLLC           = ( IO0_CLK ==`IO_50M )?12:25   ;
wire [19:0]HSW_MEAS_SETTLE_PLLC        = (
  ( IO0_CLK ==`IO_50M )?0:(
  ( IO0_CLK ==`IO_IN0 )?20'd12676:(
  ( IO0_CLK ==`IO_IN1 )?20'd12414:(
  ( IO0_CLK ==`IO_IN2 )?20'd12676:(
  ( IO0_CLK ==`IO_IN3 )?20'd12619:20'd12619 
  )))));
wire       INIT_LP_CLOSE_HO_PLLC       = 0    ; 
wire       HO_SKIP_PHASE_PLLC          = 0 ; 
wire       HOLD_PRESERVE_HIST_PLLC     = 1 ; 
wire       HOLD_FRZ_WITH_INTONLY_PLLC  = ( IO0_CLK ==`IO_50M )?0:1 ; 
wire       HOLDEXIT_BW_SEL0_PLLC       = ( IO0_CLK ==`IO_50M )?0:1 ; 
wire       HOLDEXIT_STD_BO_PLLC        = 0 ; 
wire       HOLDEXIT_ST_BO_PLLC         = 0 ; 

wire [5:0] HOLDEXIT_BW0_PLLC           = ( IO0_CLK ==`IO_50M )?0:15;
wire [5:0] HOLDEXIT_BW1_PLLC           = ( IO0_CLK ==`IO_50M )?0:32;
wire [5:0] HOLDEXIT_BW2_PLLC           = ( IO0_CLK ==`IO_50M )?0:12;
wire [5:0] HOLDEXIT_BW3_PLLC           = ( IO0_CLK ==`IO_50M )?0:11;
wire [5:0] HOLDEXIT_BW4_PLLC           = ( IO0_CLK ==`IO_50M )?0:15 ;
wire [5:0] HOLDEXIT_BW5_PLLC           = ( IO0_CLK ==`IO_50M )?0:63;
wire [7:0] HSW_LIMIT_PLLC              = ( IO0_CLK ==`IO_50M )?32:8;
wire       HSW_LIMIT_ACTION_PLLC       = 0 ;
wire [2:0] RAMP_STEP_SIZE_PLLC         = ( IO0_CLK ==`IO_50M )?0:3;
wire       RAMP_SWITCH_EN_PLLC         = 0 ;
wire       OUT_MAX_LIMIT_EN_PLLC       = ( IO0_CLK ==`IO_50M )?0:1;
wire [15:0]HOLD_SETTLE_TARGET_DSPLLC   = ( IO0_CLK ==`IO_50M )?0:51   ;

//---DSP-D
wire [3:0] M_FRAC_MODE_PLLD            = 11;
wire       M_FRAC_EN_PLLD              = (
 (IO1_CLK == `IO_50M )?1'b0: (
 (IO1_CLK == `IO_IN0 )?1'b1:(
 (IO1_CLK == `IO_IN1 )?1'b0:(
 (IO1_CLK == `IO_IN2 )?1'b1:(
 (IO1_CLK == `IO_IN3 )?1'b0 : 1'b0
 )))));



wire       M_FSTEP_MSK_PLLD            = 1 ;
wire       M_FSTEPW_DEN_PLLD           = 0 ;

assign REG_0722= { 3'h1 , M_FRAC_EN_PLLD , M_FRAC_MODE_PLLD  };// 5 R/W Reserved Must be set to 1 for DSPLL D
assign REG_0723 ={ 6'h0 , M_FSTEPW_DEN_PLLD, M_FSTEP_MSK_PLLD  };//

wire [55:0]M_FSTEPW_PLLD               = 0;
assign REG_0724= M_FSTEPW_PLLD[7:0  ];
assign REG_0725= M_FSTEPW_PLLD[15:8 ];
assign REG_0726= M_FSTEPW_PLLD[23:16];
assign REG_0727= M_FSTEPW_PLLD[31:24];
assign REG_0728= M_FSTEPW_PLLD[39:32];
assign REG_0729= M_FSTEPW_PLLD[47:40];
assign REG_072A= M_FSTEPW_PLLD[55:48];



wire       FASTLOCK_AUTO_EN_PLLD       = 1   ;
wire       FASTLOCK_MAN_PLLD           = 0   ;             
wire [9:0]    HSW_PHMEAS_THR_PLLD       = 3 ;

assign REG_072C={6'h0, FASTLOCK_MAN_PLLD,FASTLOCK_AUTO_EN_PLLD };//
assign REG_073C = HSW_PHMEAS_THR_PLLD [7:0]; 
assign REG_073D = { 6'h0 , HSW_PHMEAS_THR_PLLD [9:8] } ; 

         
///////------
wire [5:0] BW0_PLLD                    = ( IO1_CLK ==`IO_50M )?0:15  ;
wire [5:0] BW1_PLLD                    = ( IO1_CLK ==`IO_50M )?0:30  ;
wire [5:0] BW2_PLLD                    = ( IO1_CLK ==`IO_50M )?0:12  ;
wire [5:0] BW3_PLLD                    = ( IO1_CLK ==`IO_50M )?0:11  ;
wire [5:0] BW4_PLLD                    = ( IO1_CLK ==`IO_50M )?0:15   ;
wire [5:0] BW5_PLLD                    = ( IO1_CLK ==`IO_50M )?0:63  ;

wire [5:0] FASTLOCK_BW0_PLLD           = ( IO1_CLK ==`IO_50M )?0:17  ;
wire [5:0] FASTLOCK_BW1_PLLD           = ( IO1_CLK ==`IO_50M )?0:36 ;
wire [5:0] FASTLOCK_BW2_PLLD           = ( IO1_CLK ==`IO_50M )?0:10   ;
wire [5:0] FASTLOCK_BW3_PLLD           = ( IO1_CLK ==`IO_50M )?0:9   ;
wire [5:0] FASTLOCK_BW4_PLLD           = ( IO1_CLK ==`IO_50M )?0:15   ;
wire [5:0] FASTLOCK_BW5_PLLD           = ( IO1_CLK ==`IO_50M )?0:63  ;

//-----parameter 3
wire [55:0] M_NUM_PLLD               ;
assign     M_NUM_PLLD              = (
 (IO1_CLK == `IO_50M )?0: (
 (IO1_CLK == `IO_IN0 )?56'd2473901162496:(
 (IO1_CLK == `IO_IN1 )?56'd1610612736000:(
 (IO1_CLK == `IO_IN2 )?56'd2473901162496:(
 (IO1_CLK == `IO_IN3 )?56'd1589137899520 : 56'd1589137899520 
 )))));
wire [31:0] M_DEN_PLLD               ;
assign     M_DEN_PLLD                  = (
(IO1_CLK ==`IO_50M )?0:(
(IO1_CLK == `IO_IN0 )?32'd3355443200 :(
(IO1_CLK == `IO_IN1 )?32'd2147483648 :(
(IO1_CLK == `IO_IN2 )?32'd3355443200 :(
(IO1_CLK == `IO_IN3 )?32'd2147483648  :32'd2147483648 
)))));


wire [2:0]  IN_SEL_PLLD                = ( IO1_CLK ==`IO_50M )?0 :IO1_CLK-1 ; 
wire       HOLD_EN_PLLD                = 1           ;
wire       HOLD_RAMP_BYP_PLLD          = ( IO1_CLK ==`IO_50M )?1: 0   ; 
wire       HOLDEXIT_BW_SEL1_PLLD       = 0   ; 
wire [2:0] RAMP_STEP_INTERVAL_PLLD     = ( IO1_CLK ==`IO_50M )?0:4   ; 
wire [4:0] HOLD_HIST_LEN_PLLD          = ( IO1_CLK ==`IO_50M )?0:25  ; 
wire [4:0] HOLD_HIST_DELAY_PLLD        = ( IO1_CLK ==`IO_50M )?0:25  ; 
wire [23:0]HOLD_15M_CYC_COUNT_PLLD     = ( IO1_CLK ==`IO_50M )?1024 :1667 ; 
wire       FORCE_HOLD_PLLD             = ( IO1_CLK ==`IO_50M )?1:0    ;
wire [1:0]   CLK_SWITCH_MODE_PLLD      = ( IO1_CLK ==`IO_50M )?2:0 ;
wire         HSW_EN_PLLD               = 1; 
wire [1:0]   HSW_MODE_PLLD             = 1; 
wire [3:2]   HSW_PHMEAS_CTRL_PLLD      = 0; 
wire [4:0] HSW_COARSE_PM_LEN_PLLD      = ( IO1_CLK ==`IO_50M )?10: 4    ;
wire [4:0] HSW_COARSE_PM_DLY_PLLD      = ( IO1_CLK ==`IO_50M )?6:2    ;
wire [3:0] HSW_FINE_PM_LEN_PLLD        = ( IO1_CLK ==`IO_50M )?0:7    ;
wire [12:0]PFD_EN_DELAY_PLLD           = ( IO1_CLK ==`IO_50M )?12:25   ;
wire [19:0]HSW_MEAS_SETTLE_PLLD        = (
(IO1_CLK == `IO_50M )?20'd0:(
(IO1_CLK == `IO_IN0 )?20'd12676:(
(IO1_CLK == `IO_IN1 )?20'd12414:(
(IO1_CLK == `IO_IN2 )?20'd12676:(
(IO1_CLK == `IO_IN3 )?20'd12619:20'd12619
)))));




wire       INIT_LP_CLOSE_HO_PLLD       = 0    ;
wire       HO_SKIP_PHASE_PLLD          = 0    ;
wire       HOLD_PRESERVE_HIST_PLLD     = 1    ;
wire       HOLD_FRZ_WITH_INTONLY_PLLD  = ( IO1_CLK ==`IO_50M )?0:1 ;
wire       HOLDEXIT_BW_SEL0_PLLD       = ( IO1_CLK ==`IO_50M )?0:1 ;
wire       HOLDEXIT_STD_BO_PLLD        = 0; 
wire       HOLDEXIT_ST_BO_PLLD         = 0; 
wire       HOLD_RAMPBP_NOHIST_PLLD     = ( IO1_CLK ==`IO_50M )?0:1  ; 
wire [5:0] HOLDEXIT_BW0_PLLD           = ( IO1_CLK ==`IO_50M )?0:15 ; 
wire [5:0] HOLDEXIT_BW1_PLLD           = ( IO1_CLK ==`IO_50M )?0:32 ; 
wire [5:0] HOLDEXIT_BW2_PLLD           = ( IO1_CLK ==`IO_50M )?0:12 ; 
wire [5:0] HOLDEXIT_BW3_PLLD           = ( IO1_CLK ==`IO_50M )?0:11 ; 
wire [5:0] HOLDEXIT_BW4_PLLD           = ( IO1_CLK ==`IO_50M )?0:15  ; 
wire [5:0] HOLDEXIT_BW5_PLLD           = ( IO1_CLK ==`IO_50M )?0:63 ; 
wire [7:0] HSW_LIMIT_PLLD              = ( IO1_CLK ==`IO_50M )?32:8;
wire       HSW_LIMIT_ACTION_PLLD       = 0;
wire [2:0] RAMP_STEP_SIZE_PLLD         = ( IO1_CLK ==`IO_50M )?0:3;
wire       RAMP_SWITCH_EN_PLLD         = 0;
wire       OUT_MAX_LIMIT_EN_PLLD       = ( IO1_CLK ==`IO_50M )?0:1;
wire       HOLD_SETTLE_DEN_EN_DSPLLD   = ( IO1_CLK ==`IO_50M )?0:1    ;
wire [15:0]OUT_MAX_LIMIT_LMT_PLLD      = (
(IO1_CLK == `IO_50M )?16'd0:(
(IO1_CLK == `IO_IN0 )?16'd20138 :(
(IO1_CLK == `IO_IN1 )?16'd19797 :(
(IO1_CLK == `IO_IN2 )?16'd20138 :(
(IO1_CLK == `IO_IN3 )?16'd20064 :16'd20064
)))));

wire [15:0]HOLD_SETTLE_TARGET_DSPLLD   = ( IO1_CLK ==`IO_50M )?0:50   ;


wire [7:0] FIXREGSD15                  = ((IO0_CLK ==`IO_50M) | (IO1_CLK ==`IO_50M)) ?1:0;
wire [3:0] INX_TO_PFD_EN               = 15;
wire [3:0] PDIV_FRACN_CLK_DIS          = 8;
wire       FRACN_CLK_DIS_PLLA          = 0 ;
wire       FRACN_CLK_DIS_PLLB          = 0 ;
wire       FRACN_CLK_DIS_PLLC          = ( 
 (IO0_CLK == `IO_50M )?1'b0:(
 (IO0_CLK == `IO_IN0 )?1'b0:(
 (IO0_CLK == `IO_IN1 )?1'b1:(
 (IO0_CLK == `IO_IN2 )?1'b0:(
 (IO0_CLK == `IO_IN3 )?1'b1:1'b1
 )))));

wire       FRACN_CLK_DIS_PLLD          = (
 (IO1_CLK == `IO_50M )?1'b0:(
 (IO1_CLK == `IO_IN0 )?1'b0:(
 (IO1_CLK == `IO_IN1 )?1'b1:(
 (IO1_CLK == `IO_IN2 )?1'b0:(
 (IO1_CLK == `IO_IN3 )?1'b1:1'b1
 )))));

wire [4:0] OOF_CLK_DIS                 = 0 ;
wire [4:0] OOF_DIV_CLK_DIS             = 0 ;
wire [3:0] IN_CLK_VAL_PWR_UP_DIS       = ((IO0_CLK ==`IO_50M) | (IO1_CLK ==`IO_50M)) ?0: 15  ;
wire       IN_CLK_VAL_EN_PLLC          = (IO0_CLK ==`IO_50M)?0:  1  ;
wire [7:0] IN_CLK_VAL_TIME_PLLC        = (IO0_CLK ==`IO_50M)?0:  1  ;
wire       IN_CLK_VAL_EN_PLLD          = (IO1_CLK ==`IO_50M)?0:  1  ;
wire [7:0] IN_CLK_VAL_TIME_PLLD        = (IO1_CLK ==`IO_50M)?0:  1  ;

//-------------------REG 
assign REG_002E = LOS0_TRG_THR[7 :0];
assign REG_002F = LOS0_TRG_THR[15:8];
assign REG_0030 = LOS1_TRG_THR[7 :0];
assign REG_0031 = LOS1_TRG_THR[15:8];
assign REG_0032 = LOS2_TRG_THR[7 :0];
assign REG_0033 = LOS2_TRG_THR[15:8];
assign REG_0034 = LOS3_TRG_THR[7 :0];
assign REG_0035 = LOS3_TRG_THR[15:8];

assign REG_003A = LOS2_CLR_THR[7 :0];
assign REG_003B = LOS2_CLR_THR[15:8];
assign REG_003C = LOS3_CLR_THR[7 :0];
assign REG_003D = LOS3_CLR_THR[15:8];


assign REG_0043 = OOF2_DIV_SEL;
assign REG_0044 = OOF3_DIV_SEL;
assign REG_0046 = OOF0_SET_THR;
assign REG_0047 = OOF1_SET_THR;
assign REG_0048 = OOF2_SET_THR;
assign REG_0049 = OOF3_SET_THR;
assign REG_004A = OOF0_CLR_THR;
assign REG_004B = OOF1_CLR_THR;
assign REG_004C = OOF2_CLR_THR;
assign REG_004D = OOF3_CLR_THR;

assign REG_004F  ={ 1'h0 ,OOF3_DETWIN_SEL ,1'h0, OOF2_DETWIN_SEL};//
assign REG_0050  = OOF_ON_LOS ; 
assign REG_0053 = {4'h0, FAST_OOF2_SET_THR};
assign REG_0054 = {4'h0, FAST_OOF3_SET_THR};
assign REG_0057  = { 4'h0 ,FAST_OOF2_CLR_THR }; 
assign REG_0058  = { 4'h0 ,FAST_OOF3_CLR_THR }; 
assign REG_0062 = OOF2_RATIO_REF [7:0  ];
assign REG_0063 = OOF2_RATIO_REF [15:8 ];
assign REG_0064 = OOF2_RATIO_REF [23:16];
assign REG_0065 = OOF2_RATIO_REF [25:24];
assign REG_0066 = OOF3_RATIO_REF [7:0  ];
assign REG_0067 = OOF3_RATIO_REF [15:8 ];
assign REG_0068 = OOF3_RATIO_REF [23:16];
assign REG_0069 = OOF3_RATIO_REF [25:24];

assign REG_0292  = { 4'h0 ,OOF_STOP_ON_LOS  };//
assign REG_0293  = { 4'h0 ,OOF_CLEAR_ON_LOS };//

assign REG_063A = { 4'h0 , HSW_PHMEAS_CTRL_PLLC  , HSW_MODE_PLLC } ; 
assign REG_063B = HSW_PHMEAS_THR_PLLC [7:0]; 
assign REG_063C = { 6'h0 , HSW_PHMEAS_THR_PLLC [9:8] } ; 


assign REG_0737 = { 5'h0 , HSW_EN_PLLD ,CLK_SWITCH_MODE_PLLD };//
assign REG_073B = {4'h0 , HSW_PHMEAS_CTRL_PLLD ,HSW_MODE_PLLD };
assign REG_0018 ={OOF_INTR_MSK,LOS_INTR_MSK} ;
assign REG_0019 ={ HOLD_INTR_MSK_PLLD,HOLD_INTR_MSK_PLLC ,HOLD_INTR_MSK_PLLB ,HOLD_INTR_MSK_PLLA, LOL_INTR_MSK_PLLD,LOL_INTR_MSK_PLLC, LOL_INTR_MSK_PLLB,LOL_INTR_MSK_PLLA };//0,0x0
assign REG_001A = { CAL_INTR_MSK_PLLD ,CAL_INTR_MSK_PLLC, CAL_INTR_MSK_PLLB,CAL_INTR_MSK_PLLA ,4'h0 } ;// 
assign REG_002C ={ 3'h0, LOSXAXB_DIS, LOS_EN };//

assign REG_0036 = LOS0_CLR_THR[7 :0];
assign REG_0037 = LOS0_CLR_THR[15:8];
assign REG_0038 = LOS1_CLR_THR[7 :0];
assign REG_0039 = LOS1_CLR_THR[15:8];
assign REG_003E = { LOS_MIN_PERIOD_EN,4'h0};// 
assign REG_003F={ FAST_OOF_EN, OOF_EN };// 
assign REG_0040 ={ 5'h0 , OOF_REF_SEL };// 
assign REG_0041 ={ 3'h0, OOF0_DIV_SEL};//
assign REG_0042 ={ 3'h0, OOF1_DIV_SEL};//

assign REG_004E ={ 1'h0 ,OOF1_DETWIN_SEL ,1'h0, OOF0_DETWIN_SEL};//
assign REG_0051 ={ 4'h0, FAST_OOF0_SET_THR};//
assign REG_0052 ={ 4'h0, FAST_OOF1_SET_THR};//
assign REG_0055 ={ 4'h0, FAST_OOF0_CLR_THR};//
assign REG_0056 ={ 4'h0, FAST_OOF1_CLR_THR};//
assign REG_0059 ={ FAST_OOF3_DETWIN_SEL,FAST_OOF2_DETWIN_SEL,FAST_OOF1_DETWIN_SEL,FAST_OOF0_DETWIN_SEL };// 
assign REG_005A= OOF0_RATIO_REF[7:0  ];
assign REG_005B= OOF0_RATIO_REF[15:8 ];
assign REG_005C= OOF0_RATIO_REF[23:16];
assign REG_005D= {6'h0, OOF0_RATIO_REF[25:24]} ;
assign REG_005E= OOF1_RATIO_REF[7:0  ];
assign REG_005F= OOF1_RATIO_REF[15:8 ];
assign REG_0060= OOF1_RATIO_REF[23:16];
assign REG_0061= {6'h0,OOF1_RATIO_REF[25:24]} ;
assign REG_0092={ 4'h0 , LOL_FST_EN_PLLD,LOL_FST_EN_PLLC,LOL_FST_EN_PLLB,LOL_FST_EN_PLLA };//
assign REG_0094 = { LOL_FST_DETWIN_SEL_PLLD,LOL_FST_DETWIN_SEL_PLLC };// 
assign REG_0097={ LOL_FST_SET_THR_SEL_PLLD, LOL_FST_SET_THR_SEL_PLLC };//
assign REG_0099= { LOL_FST_CLR_THR_SEL_PLLD,LOL_FST_CLR_THR_SEL_PLLC };//
assign REG_009A={ 4'h0 , LOL_SLOW_EN_PLLD,LOL_SLOW_EN_PLLC,LOL_SLOW_EN_PLLB,LOL_SLOW_EN_PLLA };//
assign REG_009C ={LOL_SLW_DETWIN_SEL_PLLD,LOL_SLW_DETWIN_SEL_PLLC};//
assign REG_009D={4'h0 , LOL_SLW_VALWIN_SEL_PLLB,LOL_SLW_VALWIN_SEL_PLLA };//
assign REG_009F ={ LOL_SLW_SET_THR_PLLD,LOL_SLW_SET_THR_PLLC };//
assign REG_00A1 ={ LOL_SLW_CLR_THR_PLLD,LOL_SLW_CLR_THR_PLLC };//
assign REG_00AE= LOL_CLR_DELAY_DIV256_PLLC[7:0  ];
assign REG_00AF= LOL_CLR_DELAY_DIV256_PLLC[15:8 ];
assign REG_00B0= LOL_CLR_DELAY_DIV256_PLLC[23:16];
assign REG_00B1= {3'h0, LOL_CLR_DELAY_DIV256_PLLC[28:24] };
assign REG_00B3= LOL_CLR_DELAY_DIV256_PLLD[7:0  ];
assign REG_00B4= LOL_CLR_DELAY_DIV256_PLLD[15:8 ];
assign REG_00B5= LOL_CLR_DELAY_DIV256_PLLD[23:16];
assign REG_00B6= LOL_CLR_DELAY_DIV256_PLLD[28:24];
assign REG_00EE= FASTLOCK_EXTEND_PLLC[7:0  ];
assign REG_00EF= FASTLOCK_EXTEND_PLLC[15:8 ];
assign REG_00F0= FASTLOCK_EXTEND_PLLC[23:16];
assign REG_00F1= FASTLOCK_EXTEND_PLLC[28:24];
assign REG_00F2= FASTLOCK_EXTEND_PLLD[7:0  ];
assign REG_00F3= FASTLOCK_EXTEND_PLLD[15:8 ];
assign REG_00F4= FASTLOCK_EXTEND_PLLD[23:16];
assign REG_00F5= {3'h0, FASTLOCK_EXTEND_PLLD[28:24]};
assign REG_0206= {6'h0, PXAXB[1:0]  } ;
assign REG_0208= P0_NUM[7:0  ];
assign REG_0209= P0_NUM[15:8 ];
assign REG_020A= P0_NUM[23:16];
assign REG_020B= P0_NUM[31:24];
assign REG_020C= P0_NUM[39:32];
assign REG_020D= P0_NUM[47:40];
assign REG_020E= P0_DEN[7:0  ];
assign REG_020F= P0_DEN[15:8 ];
assign REG_0210= P0_DEN[23:16];
assign REG_0211= P0_DEN[31:24];
assign REG_0212= P1_NUM[7:0  ];
assign REG_0213= P1_NUM[15:8 ];
assign REG_0214= P1_NUM[23:16];
assign REG_0215= P1_NUM[31:24];
assign REG_0216= P1_NUM[39:32];
assign REG_0217= P1_NUM[47:40];
assign REG_0218= P1_DEN[7:0  ];
assign REG_0219= P1_DEN[15:8 ];
assign REG_021A= P1_DEN[23:16];
assign REG_021B= P1_DEN[31:24];
assign REG_021C= P2_NUM[7:0  ];
assign REG_021D= P2_NUM[15:8 ];
assign REG_021E= P2_NUM[23:16];
assign REG_021F= P2_NUM[31:24];
assign REG_0220= P2_NUM[39:32];
assign REG_0221= P2_NUM[47:40];
assign REG_0222= P2_DEN[7:0  ];
assign REG_0223= P2_DEN[15:8 ];
assign REG_0224= P2_DEN[23:16];
assign REG_0225= P2_DEN[31:24];
assign REG_0226= P3_NUM[7:0  ];
assign REG_0227= P3_NUM[15:8 ];
assign REG_0228= P3_NUM[23:16];
assign REG_0229= P3_NUM[31:24];
assign REG_022A= P3_NUM[39:32];
assign REG_022B= P3_NUM[47:40];
assign REG_022C= P3_DEN[7:0  ];
assign REG_022D= P3_DEN[15:8 ];
assign REG_022E= P3_DEN[23:16];
assign REG_022F= P3_DEN[31:24];


assign REG_0295={ FASTLOCK_EXTEND_SCL_PLLD,FASTLOCK_EXTEND_SCL_PLLC };//
assign REG_0296 ={ 4'h0 , LOL_SLW_VALWIN_SELX_PLLD,LOL_SLW_VALWIN_SELX_PLLC,LOL_SLW_VALWIN_SELX_PLLB,LOL_SLW_VALWIN_SELX_PLLA };//
assign REG_0297={ 4'h0 ,FASTLOCK_DLY_ONSW_EN_PLLD ,FASTLOCK_DLY_ONSW_EN_PLLC ,FASTLOCK_DLY_ONSW_EN_PLLB ,FASTLOCK_DLY_ONSW_EN_PLLA  };//
assign REG_0299={ 4'h0 ,FASTLOCK_DLY_ONLOL_EN_PLLD,FASTLOCK_DLY_ONLOL_EN_PLLC,FASTLOCK_DLY_ONLOL_EN_PLLB,FASTLOCK_DLY_ONLOL_EN_PLLA };//
assign REG_02A0 = FASTLOCK_DLY_ONLOL_PLLC[7:0   ];
assign REG_02A1 = FASTLOCK_DLY_ONLOL_PLLC[15:8  ];
assign REG_02A2 = {4'h0, FASTLOCK_DLY_ONLOL_PLLC[19:16]} ;
assign REG_02A3 = FASTLOCK_DLY_ONLOL_PLLD[7:0  ];
assign REG_02A4 = FASTLOCK_DLY_ONLOL_PLLD[15:8 ];
assign REG_02A5 = {4'h0 , FASTLOCK_DLY_ONLOL_PLLD[19:16]} ;
assign REG_02AC = FASTLOCK_DLY_ONSW_PLLC[7:0  ];
assign REG_02AD = FASTLOCK_DLY_ONSW_PLLC[15:8 ];
assign REG_02AE = {4'h0, FASTLOCK_DLY_ONSW_PLLC[19:16] } ;
assign REG_02AF = FASTLOCK_DLY_ONSW_PLLD[7:0  ];
assign REG_02B0 = FASTLOCK_DLY_ONSW_PLLD[15:8 ];
assign REG_02B1 = {4'h0 , FASTLOCK_DLY_ONSW_PLLD[19:16] } ;
assign REG_0608={2'h0, BW0_PLLC};
assign REG_0609={2'h0, BW1_PLLC};
assign REG_060A={2'h0, BW2_PLLC};
assign REG_060B={2'h0, BW3_PLLC};
assign REG_060C={2'h0, BW4_PLLC};
assign REG_060D={2'h0, BW5_PLLC};
assign REG_060E={2'h0, FASTLOCK_BW0_PLLC};//
assign REG_060F={2'h0, FASTLOCK_BW1_PLLC};//
assign REG_0610={2'h0, FASTLOCK_BW2_PLLC};//
assign REG_0611={2'h0, FASTLOCK_BW3_PLLC};//
assign REG_0612={2'h0, FASTLOCK_BW4_PLLC};//
assign REG_0613={2'h0, FASTLOCK_BW5_PLLC};//
assign REG_0615= M_NUM_PLLC[7:0  ];//
assign REG_0616= M_NUM_PLLC[15:8 ];//
assign REG_0617= M_NUM_PLLC[23:16];//
assign REG_0618= M_NUM_PLLC[31:24];//
assign REG_0619= M_NUM_PLLC[39:32];//
assign REG_061A= M_NUM_PLLC[47:40];//
assign REG_061B= M_NUM_PLLC[55:48];//
assign REG_061C= M_DEN_PLLC[7:0  ];//
assign REG_061D= M_DEN_PLLC[15:8 ];//
assign REG_061E= M_DEN_PLLC[23:16];//
assign REG_061F= M_DEN_PLLC[31:24];//
assign REG_0621= { 3'h1 , M_FRAC_EN_PLLC ,M_FRAC_MODE_PLLC };// bit 5 Must be set to 1 for DSPLL C
assign REG_0622 ={ 6'h0 , M_FSTEPW_DEN_PLLC, M_FSTEP_MSK_PLLC };//
assign REG_0623= M_FSTEPW_PLLC[7:0  ];
assign REG_0624= M_FSTEPW_PLLC[15:8 ];
assign REG_0625= M_FSTEPW_PLLC[23:16];
assign REG_0626= M_FSTEPW_PLLC[31:24];
assign REG_0627= M_FSTEPW_PLLC[39:32];
assign REG_0628= M_FSTEPW_PLLC[47:40];
assign REG_0629= M_FSTEPW_PLLC[55:48];
assign REG_062A = { 5'h0 , IN_SEL_PLLC};//
assign REG_062B={6'h0, FASTLOCK_MAN_PLLC,FASTLOCK_AUTO_EN_PLLC };//
//assign REG_062C ={ RAMP_STEP_INTERVAL_PLLC,HOLDEXIT_BW_SEL1_PLLC,HOLD_RAMP_BYP_PLLC, 2'h0 , HOLD_EN_PLLC };//
assign REG_062C =( IO0_CLK ==`IO_50M )?8'h0f : 8'h87;
assign REG_062E ={ 3'h0, HOLD_HIST_LEN_PLLC };//
assign REG_062F ={ 3'h0, HOLD_HIST_DELAY_PLLC};//
assign REG_0632= HOLD_15M_CYC_COUNT_PLLC[7:0  ];
assign REG_0633= HOLD_15M_CYC_COUNT_PLLC[15:8 ];
assign REG_0634= HOLD_15M_CYC_COUNT_PLLC[23:16];
assign REG_0635= {7'h0 , FORCE_HOLD_PLLC };//
assign REG_063D={ 3'h0 , HSW_COARSE_PM_LEN_PLLC};//
assign REG_063E={ 3'h0 , HSW_COARSE_PM_DLY_PLLC};//
assign REG_0688={4'h0, HSW_FINE_PM_LEN_PLLC};//
assign REG_0689=PFD_EN_DELAY_PLLC[7:0];
assign REG_068A={3'h0, PFD_EN_DELAY_PLLC[12:8] } ;
assign REG_068B=HSW_MEAS_SETTLE_PLLC[7:0  ];//
assign REG_068C=HSW_MEAS_SETTLE_PLLC[15:8 ];//
assign REG_068D={4'h0, HSW_MEAS_SETTLE_PLLC[19:16]} ;//
assign REG_069B={ HOLDEXIT_STD_BO_PLLC,
                  HOLDEXIT_BW_SEL0_PLLC,
						HOLD_FRZ_WITH_INTONLY_PLLC,
						HOLD_PRESERVE_HIST_PLLC,
						1'h1, 
						HO_SKIP_PHASE_PLLC ,
						INIT_LP_CLOSE_HO_PLLC, 
						1'h0 }; 
assign REG_069C={ HOLD_RAMPPB_NOHIST_PLLC,HOLDEXIT_ST_BO_PLLC, 2'h0,4'hc };// 
assign REG_069D={ 2'h0 , HOLDEXIT_BW0_PLLC};//
assign REG_069E={ 2'h0 , HOLDEXIT_BW1_PLLC};//
assign REG_069F={ 2'h0 , HOLDEXIT_BW2_PLLC};//
assign REG_06A0={ 2'h0 , HOLDEXIT_BW3_PLLC};//
assign REG_06A1={ 2'h0 , HOLDEXIT_BW4_PLLC};//
assign REG_06A2={ 2'h0 , HOLDEXIT_BW5_PLLC};//
assign REG_06A4= HSW_LIMIT_PLLC ; 

assign REG_06A5 ={ 7'h0, HSW_LIMIT_ACTION_PLLC};// 
assign REG_06A6={ 4'h0 , RAMP_SWITCH_EN_PLLC,RAMP_STEP_SIZE_PLLC };// 
assign REG_06AC={ 4'h0 , HOLD_SETTLE_DET_EN_DSPLLC,2'h0, OUT_MAX_LIMIT_EN_PLLC };//
assign REG_06AD = OUT_MAX_LIMIT_LMT_DPSLLC[7 :0];//same OUT_MAX_LIMIT_LMT_PLL
assign REG_06AE = OUT_MAX_LIMIT_LMT_DPSLLC[15:8];
assign REG_06B1 = HOLD_SETTLE_TARGET_DSPLLC[7 :0];
assign REG_06B2 = HOLD_SETTLE_TARGET_DSPLLC[15:8];
assign REG_0709={2'h0, BW0_PLLD          };//
assign REG_070A={2'h0, BW1_PLLD          };//
assign REG_070B={2'h0, BW2_PLLD          };//
assign REG_070C={2'h0, BW3_PLLD          };//
assign REG_070D={2'h0, BW4_PLLD          };//
assign REG_070E={2'h0, BW5_PLLD          };//
assign REG_070F={2'h0, FASTLOCK_BW0_PLLD};//3
assign REG_0710={2'h0, FASTLOCK_BW1_PLLD};//8
assign REG_0711={2'h0, FASTLOCK_BW2_PLLD};//
assign REG_0712={2'h0, FASTLOCK_BW3_PLLD};//
assign REG_0713={2'h0, FASTLOCK_BW4_PLLD};//
assign REG_0714={2'h0, FASTLOCK_BW5_PLLD};//
assign REG_0716=M_NUM_PLLD[7:0  ];//
assign REG_0717=M_NUM_PLLD[15:8 ];//
assign REG_0718=M_NUM_PLLD[23:16];//
assign REG_0719=M_NUM_PLLD[31:24];//
assign REG_071A=M_NUM_PLLD[39:32];//
assign REG_071B=M_NUM_PLLD[47:40];//
assign REG_071C=M_NUM_PLLD[55:48];//
assign REG_071D=M_DEN_PLLD[7:0  ];
assign REG_071E=M_DEN_PLLD[15:8 ];
assign REG_071F=M_DEN_PLLD[23:16];
assign REG_0720=M_DEN_PLLD[31:24];
assign REG_072B={ 5'h0, IN_SEL_PLLD };
//assign REG_072D={ RAMP_STEP_INTERVAL_PLLD,HOLDEXIT_BW_SEL1_PLLD,HOLD_RAMP_BYP_PLLD,2'h0 , HOLD_EN_PLLD };//
assign REG_072D =( IO1_CLK ==`IO_50M )?8'h0f : 8'h87;
assign REG_072F={ 3'h0, HOLD_HIST_LEN_PLLD  };//
assign REG_0730={ 3'h0, HOLD_HIST_DELAY_PLLD};//
assign REG_0733=HOLD_15M_CYC_COUNT_PLLD[7:0  ] ;
assign REG_0734=HOLD_15M_CYC_COUNT_PLLD[15:8 ] ;
assign REG_0735=HOLD_15M_CYC_COUNT_PLLD[23:16] ;
assign REG_0736={ 7'h0 , FORCE_HOLD_PLLD};// 
assign REG_073E={ 3'h0 , HSW_COARSE_PM_LEN_PLLD};//
assign REG_073F={ 3'h0 , HSW_COARSE_PM_DLY_PLLD};//
assign REG_0788={ 4'h0 , HSW_FINE_PM_LEN_PLLD };// 
assign REG_0789 = PFD_EN_DELAY_PLLD[7:0];
assign REG_078A = {3'h0, PFD_EN_DELAY_PLLD[12:8]};
assign REG_078B = HSW_MEAS_SETTLE_PLLD[7:0  ];
assign REG_078C = HSW_MEAS_SETTLE_PLLD[15:8 ];
assign REG_078D = {4'h0, HSW_MEAS_SETTLE_PLLD[19:16]};
assign REG_079B={ HOLDEXIT_STD_BO_PLLD ,
                  HOLDEXIT_BW_SEL0_PLLD ,
						HOLD_FRZ_WITH_INTONLY_PLLD ,
                  HOLD_PRESERVE_HIST_PLLD
                  ,1'b1,
						HO_SKIP_PHASE_PLLD,
						INIT_LP_CLOSE_HO_PLLD,
						1'b0 };//
assign REG_079C ={ HOLD_RAMPBP_NOHIST_PLLD,HOLDEXIT_ST_BO_PLLD ,2'h0,4'hc };// 
assign REG_079D={2'h0 , HOLDEXIT_BW0_PLLD};//
assign REG_079E={2'h0 , HOLDEXIT_BW1_PLLD};//
assign REG_079F={2'h0 , HOLDEXIT_BW2_PLLD};//
assign REG_07A0={2'h0 , HOLDEXIT_BW3_PLLD};//
assign REG_07A1={2'h0 , HOLDEXIT_BW4_PLLD};//
assign REG_07A2={2'h0 , HOLDEXIT_BW5_PLLD};//
assign REG_07A4= HSW_LIMIT_PLLD ; 
assign REG_07A5 ={ 7'h0 , HSW_LIMIT_ACTION_PLLD };// 
assign REG_07A6 ={ 4'h0 , RAMP_SWITCH_EN_PLLD,RAMP_STEP_SIZE_PLLD };//
assign REG_07AC={4'h0 , HOLD_SETTLE_DEN_EN_DSPLLD,2'h0 , OUT_MAX_LIMIT_EN_PLLD };//
assign REG_07AD = OUT_MAX_LIMIT_LMT_PLLD[7:0 ];
assign REG_07AE = OUT_MAX_LIMIT_LMT_PLLD[15:8];
assign REG_07B1=  HOLD_SETTLE_TARGET_DSPLLD[7:0 ];
assign REG_07B2=  HOLD_SETTLE_TARGET_DSPLLD[15:8];
assign REG_0831 = FIXREGSD15 ; 
assign REG_094A ={ 4'h0 , INX_TO_PFD_EN };// 
assign REG_0949={ IN_PULSED_CMOS_EN, IN_EN };// in0~3 0: Standard Input Format//1: Pulsed CMOS Input Format for IN3 See for//more information //0: Disable and Powerdown Input Buffer //and free run  //1: enable  Input Buffer//for in0~3
assign REG_0B44  ={
 FRACN_CLK_DIS_PLLD, 
 FRACN_CLK_DIS_PLLC, 
 FRACN_CLK_DIS_PLLB, 
 FRACN_CLK_DIS_PLLA, 
 PDIV_FRACN_CLK_DIS };//
assign REG_0B47={3'h0 , OOF_CLK_DIS    };//
assign REG_0B48={3'h0 , OOF_DIV_CLK_DIS};//
assign REG_0C03={4'h0 , IN_CLK_VAL_PWR_UP_DIS };// 
assign REG_0C09  ={ 7'h0, IN_CLK_VAL_EN_PLLC };// 
assign REG_0C0A= IN_CLK_VAL_TIME_PLLC ; 
assign REG_0C0B  ={7'h0 , IN_CLK_VAL_EN_PLLD };// 
assign REG_0C0C= IN_CLK_VAL_TIME_PLLD ; 
//==============================================


//-----------for free run -------------------
wire 			OUT0_PDN                = 0;
wire 			OUT0_OE                 = 1;
wire 			OUT0_RDIV_FORCE2        = ( IO0_CLK ==`IO_50M )? 1 : 0; //OUT0_RDIV_FORCE2 = 1 and then the R0_REG value is irrelevant.
wire [2:0]  OUT0_MUX_SEL            = 2 ;//<-----OUT0 SEL 
wire [5:4]  OUT0_VDD_SEL            = 1 ;
wire        OUT0_VDD_SEL_EN         = 1 ;
wire [1:0]  OUT0_INV                = 0 ;
wire        OUT2_PDN                = 0 ;
wire        OUT2_OE                 = 1 ;
wire        OUT2_RDIV_FORCE2        = 1 ;
wire [2:0]  OUT2_MUX_SEL            = 0 ;//<-----OUT2 SEL 
wire [1:0]  OUT2_VDD_SEL            = 1 ;
wire        OUT2_VDD_SEL_EN         = 1 ;
wire [1:0]  OUT2_INV                = 0 ;
wire        OUT4_PDN                = 0 ;
wire        OUT4_OE                 = 1 ;
wire        OUT4_RDIV_FORCE2        = ( IO1_CLK ==`IO_50M )? 1 : 0; //OUT4_RDIV_FORCE2 = 1 and then the R0_REG value is irrelevant.1 ;
wire [2:0]  OUT4_MUX_SEL            = 3 ;//<-----OUT4 SEL 
wire [1:0]  OUT4_VDD_SEL            = 1 ;
wire        OUT4_VDD_SEL_EN         = 1 ; 
wire [1:0]  OUT4_INV                = 0 ; 
wire        OUT6_PDN                = 0 ; 
wire        OUT6_OE                 = 1 ; 
wire        OUT6_RDIV_FORCE2        = 1 ; 
wire [2:0]  OUT6_MUX_SEL            = 1 ;//<-----OUT6 SEL  
wire [1:0]  OUT6_VDD_SEL            = 1 ; 
wire        OUT6_VDD_SEL_EN         = 1 ; 
wire [1:0]  OUT6_INV                = 0 ;
wire        OUT7_PDN                = 0 ;
wire        OUT7_OE                 = XCVR_REFCLK_RSV ;// 1 ;
wire        OUT7_RDIV_FORCE2        = 1 ;
wire [2:0]  OUT7_MUX_SEL            = 1 ;//<-----OUT7 SEL 
wire [1:0]  OUT7_VDD_SEL            = 1 ;
wire        OUT7_VDD_SEL_EN         = 1 ;
wire [1:0]  OUT7_INV                = 0 ;
 
wire [43:0] M_NUM            = 44'd633507676160;
wire [31:0] M_DEN            = 32'd2147483648  ;

wire        MXAXB_UPDATE     = 1 ;  //Set to 1 to update the MXAXB_NUM and//MXAXB_DEN values. A SOFT_RST may also be//used to update these values.
wire [23:0] R2_REG           = 0 ; 

//-----parameter 4
wire [23:0] R0_REG           ; 
assign R0_REG =( 
(IO0_CLK == `IO_IN0 )?11:(
(IO0_CLK == `IO_IN1 )?11:(
(IO0_CLK == `IO_IN2 )?11:(
(IO0_CLK == `IO_IN3 )?36:36
))));
wire [23:0] R4_REG          ; 
assign R4_REG =( 
(IO1_CLK == `IO_IN0 )?11:(
(IO1_CLK == `IO_IN1 )?11:(
(IO1_CLK == `IO_IN2 )?11:(
(IO1_CLK == `IO_IN3 )?36:36
))));


wire [23:0] R6_REG           = 0 ; 
wire [23:0] R7_REG           = 0 ; 
wire [43:0] N0_NUM           ;
wire [31:0] N0_DEN           = 32'd2147483648 ;
wire        N0_UPDATE        = 1           ;  
wire [43:0] N1_NUM           ;  
wire [31:0] N1_DEN           = 32'd2147483648  ;  
wire        N1_UPDATE        = 1           ;  
wire [43:0] N2_NUM_            ; 

//-----parameter 5
////--OUT0
wire [43:0] N2_NUM_IN=(
(IO0_CLK == `IO_IN0 )?44'd61865984000:(
(IO0_CLK == `IO_IN1 )?44'd63350767616:(
(IO0_CLK == `IO_IN2 )?44'd61865984000:(
(IO0_CLK == `IO_IN3 )?44'd47513075712:44'd47513075712
))));
wire [31:0] N2_DEN_IN=(
(IO0_CLK == `IO_IN0 )?32'd3221225472:(
(IO0_CLK == `IO_IN1 )?32'd3355443200:(
(IO0_CLK == `IO_IN2 )?32'd3221225472:(
(IO0_CLK == `IO_IN3 )?32'd2483027968:32'd2483027968
))));
wire [43:0] N2_NUM           = (IO0_CLK ==`IO_50M)?N2_NUM_       :N2_NUM_IN;
wire [31:0] N2_DEN           = (IO0_CLK ==`IO_50M)?32'd2147483648:N2_DEN_IN;
wire        N2_UPDATE        = 1           ;  
//--OUT4
wire [43:0] N3_NUM_IN=( 
(IO1_CLK == `IO_IN0 )?44'd61865984000:(
(IO1_CLK == `IO_IN1 )?44'd63350767616:(
(IO1_CLK == `IO_IN2 )?44'd61865984000:(
(IO1_CLK == `IO_IN3 )?44'd47513075712: 44'd47513075712
))));
wire [31:0] N3_DEN_IN= (
(IO1_CLK == `IO_IN0 )?32'd3221225472:(
(IO1_CLK == `IO_IN1 )?32'd3355443200:(
(IO1_CLK == `IO_IN2 )?32'd3221225472:(
(IO1_CLK == `IO_IN3 )?32'd2483027968: 32'd2483027968
))));
wire [43:0] N3_NUM_ ;
wire [43:0] N3_NUM           = (IO1_CLK ==`IO_50M)?N3_NUM_       : N3_NUM_IN;  
wire [31:0] N3_DEN           = (IO1_CLK ==`IO_50M)?32'd2147483648: N3_DEN_IN;

 
 
 
 
wire        N3_UPDATE        = 1          ;
wire        N_UPDATE         = 1          ;//Writing a 1 to this bit will update all DSPLL internal divider values. When this bit is written, all other bits in this//register must be written as zeros.

wire        M_UPDATE_PLLA    = 1; //  Must write a 1 to this bit to cause the individual M divider//changes to take effect. Note that a corresponding//SOFT_RST_PLLx or device SOFT_RST will also update//the M divider values.
wire        M_UPDATE_PLLB    = 1;
wire        M_UPDATE_PLLC    = 1;
wire        M_UPDATE_PLLD    = 1;
wire [3:0] IN_PULSED_CMOS_EN = 0; // 0: Standard Input Format//1: Pulsed CMOS Input Format for IN3 See for//more information
wire       XAXB_EXTCLK_EN    = 1;//de10_agiex  //Selects between the XTAL or external reference cloc//on the XA/XB pins. Default is 0, XTAL. Set to 1 to use//an external reference oscillator.
//wire       XAXB_EXTCLK_EN    = 0; //evb //Selects between the XTAL or external reference cloc//on the XA/XB pins. Default is 0, XTAL. Set to 1 to use//an external reference oscillator.
wire [3:0] IN_EN             = 15;// ( (IO0_CLK ==`IO_50M) &  (IO1_CLK ==`IO_50M)) ?0:  15   ; //0: Disable and Powerdown Input Buffer //and free run  //1: enable  Input Buffer//for in0~3

wire        P0_UPDATE        = 1 ; //0// - No update for P-divider value
wire        P1_UPDATE        = 1 ; //1// - Update P-divider value
wire        P2_UPDATE        = 1 ;
wire        P3_UPDATE        = 1 ;

//----
assign REG_0108  ={ 5'h0 , OUT0_RDIV_FORCE2, OUT0_OE, OUT0_PDN } ;
assign REG_010B  ={ OUT0_INV , OUT0_VDD_SEL,OUT0_VDD_SEL_EN,OUT0_MUX_SEL }; 
assign REG_0117  ={ 5'h0 , OUT2_RDIV_FORCE2 , OUT2_OE , OUT2_PDN };// 
assign REG_011A  ={ OUT2_INV , OUT2_VDD_SEL , OUT2_VDD_SEL_EN ,OUT2_MUX_SEL };//
assign REG_0126  ={ 5'h0 , OUT4_RDIV_FORCE2 ,OUT4_OE,OUT4_PDN };//
assign REG_0129  ={ OUT4_INV,OUT4_VDD_SEL,OUT4_VDD_SEL_EN,OUT4_MUX_SEL } ;
assign REG_0130  ={ 5'h0 , OUT6_RDIV_FORCE2,OUT6_OE ,OUT6_PDN };// 
assign REG_0133  ={ OUT6_INV,OUT6_VDD_SEL,OUT6_VDD_SEL_EN,OUT6_MUX_SEL };//
assign REG_013A  ={ 5'h0 , OUT7_RDIV_FORCE2,OUT7_OE ,OUT7_PDN };//
assign REG_013D  = { OUT7_INV,OUT7_VDD_SEL,OUT7_VDD_SEL_EN ,OUT7_MUX_SEL };//
assign REG_0230  ={4'h0 , P3_UPDATE ,P2_UPDATE , P1_UPDATE ,P0_UPDATE };
assign REG_0235= M_NUM[7:0  ] ;//0x09380000000
assign REG_0236= M_NUM[15:8 ] ;
assign REG_0237= M_NUM[23:16] ;
assign REG_0238= M_NUM[31:24] ;
assign REG_0239= M_NUM[39:32] ;
assign REG_023A= M_NUM[43:40] ;
assign REG_023B= M_DEN[7:0  ] ; //0x80000000
assign REG_023C= M_DEN[15:8 ];
assign REG_023D= M_DEN[23:16];
assign REG_023E= M_DEN[31:24];
assign REG_023F= {7'h0 ,MXAXB_UPDATE };//  //Set to 1 to update the MXAXB_NUM and//MXAXB_DEN values. A SOFT_RST may also be//used to update these values.
assign REG_024A= R0_REG[7:0  ];
assign REG_024B= R0_REG[15:8 ];
assign REG_024C= R0_REG[23:16];
assign REG_0253= R2_REG[7:0  ];
assign REG_0254= R2_REG[15:8 ];
assign REG_0255= R2_REG[23:16];
assign REG_025C= R4_REG[7:0  ];
assign REG_025D= R4_REG[15:8 ];
assign REG_025E= R4_REG[23:16];
assign REG_0262= R6_REG[7:0  ];
assign REG_0263= R6_REG[15:8 ];
assign REG_0264= R6_REG[23:16];
assign REG_0268= R7_REG[7:0  ];
assign REG_0269= R7_REG[15:8 ];
assign REG_026A= R7_REG[23:16];
assign REG_0302= N0_NUM[7:0  ];
assign REG_0303= N0_NUM[15:8 ];
assign REG_0304= N0_NUM[23:16];
assign REG_0305= N0_NUM[31:24];
assign REG_0306= N0_NUM[39:32];
assign REG_0307= N0_NUM[43:40];
assign REG_0308= N0_DEN[7:0  ];
assign REG_0309= N0_DEN[15:8 ];
assign REG_030A= N0_DEN[23:16];
assign REG_030B= N0_DEN[31:24];
assign REG_030C= {7'h0 , N0_UPDATE }; 
assign REG_030D= N1_NUM[7:0  ];
assign REG_030E= N1_NUM[15:8 ];
assign REG_030F= N1_NUM[23:16];
assign REG_0310= N1_NUM[31:24];
assign REG_0311= N1_NUM[39:32];
assign REG_0312= N1_NUM[43:40];
assign REG_0313= N1_DEN[7:0  ];
assign REG_0314= N1_DEN[15:8 ];
assign REG_0315= N1_DEN[23:16];
assign REG_0316= N1_DEN[31:24];
assign REG_0317= {7'h0 , N1_UPDATE };// 
assign REG_0318= N2_NUM[7:0  ];
assign REG_0319= N2_NUM[15:8 ];
assign REG_031A= N2_NUM[23:16];
assign REG_031B= N2_NUM[31:24];
assign REG_031C= N2_NUM[39:32];
assign REG_031D= N2_NUM[43:40];
assign REG_031E= N2_DEN[7:0  ];
assign REG_031F= N2_DEN[15:8 ];
assign REG_0320= N2_DEN[23:16];
assign REG_0321= N2_DEN[31:24];
assign REG_0322= {7'h0 ,N2_UPDATE};//
assign REG_0323= N3_NUM[7:0  ];
assign REG_0324= N3_NUM[15:8 ];
assign REG_0325= N3_NUM[23:16];
assign REG_0326= N3_NUM[31:24];
assign REG_0327= N3_NUM[39:32];
assign REG_0328= N3_NUM[43:40];
assign REG_0329= N3_DEN[7:0  ];
assign REG_032A= N3_DEN[15:8 ];
assign REG_032B= N3_DEN[23:16];
assign REG_032C= N3_DEN[31:24];
assign REG_032D= {7'h0 ,N3_UPDATE};//
assign REG_0338= {7'h0 ,N_UPDATE };////Writing a 1 to this bit will update all DSPLL internal divider values. When this bit is written, all other bits in this//register must be written as zeros.
assign REG_0420= {7'h0, M_UPDATE_PLLA};// //  Must write a 1 to this bit to cause the individual M divider//changes to take effect. Note that a corresponding//SOFT_RST_PLLx or device SOFT_RST will also update//the M divider values.
assign REG_0520= {7'h0, M_UPDATE_PLLB};//
assign REG_0620= {7'h0, M_UPDATE_PLLC};//
assign REG_0721= {7'h0, M_UPDATE_PLLD};//
assign REG_090E= {7'h1, XAXB_EXTCLK_EN };//  //Selects between the XTAL or external reference cloc//on the XA/XB pins. Default is 0, XTAL. Set to 1 to use//an external reference oscillator.



//---DSPA
si5397a_freq_prameter_selector dspa(
.iPLL_OUT_FREQ_SEL(XCVR0_REFCLK),
.NX_NUM           (N0_NUM)
);
//---DSPB
si5397a_freq_prameter_selector dspb(
.iPLL_OUT_FREQ_SEL(XCVR1_REFCLK),
.NX_NUM           (N1_NUM)
);

//---DSPc
si5397a_freq_prameter_selector dspc(
.iPLL_OUT_FREQ_SEL( (IO0_CLK ==`IO_50M)? 8'hb : 8'hc), //b freerun .c input clock 
.NX_NUM           (N2_NUM_)//50M
//.NX_DEN           (N2_DEN)
);

//---DSPD
si5397a_freq_prameter_selector dspd(
.iPLL_OUT_FREQ_SEL( (IO1_CLK ==`IO_50M)? 8'hb : 8'hc), //b freerun .c input clock 
.NX_NUM           (N3_NUM_) //50M
//.NX_DEN           (N3_DEN)

);

//=====================================
//  State control
//=====================================
			
always@(posedge iCLK or negedge iRST_n)
	begin
		if (!iRST_n) i2c_reg_state <= 0;
		else
			begin
				if (access_i2c_reg_start) i2c_reg_state <= 1'b1; 
				else if ( i2c_controller_config_done_n ) i2c_reg_state <= i2c_reg_state + 1; 				
				else if (i2c_reg_state == (`REG_NUM+1))
					i2c_reg_state <= 0;	
			end
	end
	
	
///DELAY 
wire i2c_controller_config_done_n_r	 ; 

always@(posedge iCLK or negedge iRST_n)
	begin
		if (!iRST_n)
			begin
					   DELAY <= 0;
						DONE  <= 0 ;
			end
		else
			begin
				if (access_i2c_reg_start) begin 
					   DELAY <= 0; //
						DONE  <= 0 ; 
				end 
				//else if (( i2c_controller_config_done_n_r ) &  (i2c_ctrl_data[24] == DELAY_REG ) & (!DONE))   begin
				else if (( i2c_controller_config_done_n ) &  (i2c_ctrl_data[24] == DELAY_REG ) & (!DONE))   begin
							       DELAY <= 15_000_000; // delay 300mS
									 DONE  <=1 ; 
									 
	            end
				else if (DELAY!=0)  DELAY <= DELAY-1; 
				
			end
end 			

//=====================================
//  i2c bus address & data control 
//=====================================	

always@(*)
  begin
    i2c_ctrl_data = 0;
    case (i2c_reg_state)
000:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	8'd0	,	8'd0	,	read_cmd	}	;
001:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	REG_ADDR_PAGE	,	8'h0B	,	write_cmd	}	;	//change_page 0
002:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	8'h24	,	8'hC0	,	write_cmd	}	;	         //{ 0x0B24, 0xC0 },/* Start configuration preamble
003:i2c_ctrl_data	= { DELAY_REG ,slave_addr	,	8'h25	,	8'h00	,	write_cmd	}	;	         // 0x0B25, 0x00 },  End configuration preamble  Delay 300 msec Delay is worst case time for device to complete any calibration that is running due to device state change previous to this script
004:i2c_ctrl_data	= { NOT_LAST ,slave_addr,	REG_ADDR_PAGE	,	8'h00	,	write_cmd	}	;	//change_page 0
005:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_002E,REG_002E,write_cmd};//
006:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_002F,REG_002F,write_cmd};//
007:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0030,REG_0030,write_cmd};//
008:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0031,REG_0031,write_cmd};//
009:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0032,REG_0032,write_cmd};//
010:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0033,REG_0033,write_cmd};//
011:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0034,REG_0034,write_cmd};//
012:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0035,REG_0035,write_cmd};//
013:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_003A,REG_003A,write_cmd};//
014:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_003B,REG_003B,write_cmd};//
015:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_003C,REG_003C,write_cmd};//
016:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_003D,REG_003D,write_cmd};//
017:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0041,REG_0041,write_cmd};//
018:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0042,REG_0042,write_cmd};//
019:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0043,REG_0043,write_cmd};//
020:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0044,REG_0044,write_cmd};//
021:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0046,REG_0046,write_cmd};//
022:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0047,REG_0047,write_cmd};//
023:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0048,REG_0048,write_cmd};//
024:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0049,REG_0049,write_cmd};//
025:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_004A,REG_004A,write_cmd};//
026:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_004B,REG_004B,write_cmd};//
027:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_004C,REG_004C,write_cmd};//
028:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_004D,REG_004D,write_cmd};//
029:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_004F,REG_004F,write_cmd};//
030:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0050,REG_0050,write_cmd};//
031:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0053,REG_0053,write_cmd};
032:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0054,REG_0054,write_cmd};
033:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0057,REG_0057,write_cmd};//
034:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0058,REG_0058,write_cmd};//
035:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0062,REG_0062,write_cmd};//
036:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0063,REG_0063,write_cmd};//
037:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0064,REG_0064,write_cmd};//
038:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0065,REG_0065,write_cmd};//
039:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0066,REG_0066,write_cmd};//
040:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0067,REG_0067,write_cmd};//
041:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0068,REG_0068,write_cmd};//
042:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0069,REG_0069,write_cmd};//
043:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0018,REG_0018,write_cmd};
044:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0019,REG_0019,write_cmd};
045:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_001A,REG_001A,write_cmd};
046:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_002C,REG_002C,write_cmd};
047:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0036,REG_0036,write_cmd};
048:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0037,REG_0037,write_cmd};
049:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0038,REG_0038,write_cmd};
050:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0039,REG_0039,write_cmd};
051:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_003E,REG_003E,write_cmd};
052:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_003F,REG_003F,write_cmd};
053:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0040,REG_0040,write_cmd};
054:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_004E,REG_004E,write_cmd};
055:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0051,REG_0051,write_cmd};
056:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0052,REG_0052,write_cmd};
057:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0055,REG_0055,write_cmd};
058:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0056,REG_0056,write_cmd};
059:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0059,REG_0059,write_cmd};
060:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_005A,REG_005A,write_cmd};
061:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_005B,REG_005B,write_cmd};
062:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_005C,REG_005C,write_cmd};
063:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_005D,REG_005D,write_cmd};
064:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_005E,REG_005E,write_cmd};
065:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_005F,REG_005F,write_cmd};
066:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0060,REG_0060,write_cmd};
067:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0061,REG_0061,write_cmd};
068:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0092,REG_0092,write_cmd};
069:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0094,REG_0094,write_cmd};
070:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0097,REG_0097,write_cmd};
071:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_0099,REG_0099,write_cmd};
072:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_009A,REG_009A,write_cmd};
073:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_009C,REG_009C,write_cmd};
074:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_009D,REG_009D,write_cmd};
075:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_009F,REG_009F,write_cmd};
076:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00A1,REG_00A1,write_cmd};
077:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00AE,REG_00AE,write_cmd};
078:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00AF,REG_00AF,write_cmd};
079:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00B0,REG_00B0,write_cmd};
080:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00B1,REG_00B1,write_cmd};
081:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00B3,REG_00B3,write_cmd};
082:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00B4,REG_00B4,write_cmd};
083:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00B5,REG_00B5,write_cmd};
084:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00B6,REG_00B6,write_cmd};
085:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00EE,REG_00EE,write_cmd};
086:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00EF,REG_00EF,write_cmd};
087:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00F0,REG_00F0,write_cmd};
088:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00F1,REG_00F1,write_cmd};
089:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00F2,REG_00F2,write_cmd};
090:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00F3,REG_00F3,write_cmd};
091:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00F4,REG_00F4,write_cmd};
092:i2c_ctrl_data = { NOT_LAST ,slave_addr , REG_ADDR_00F5,REG_00F5,write_cmd};
093:i2c_ctrl_data	= { NOT_LAST ,	slave_addr	,	REG_ADDR_PAGE	,	8'h01	,	write_cmd	}	;	//change_page 1
094:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0108,REG_0108,write_cmd};
095:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_010B,REG_010B,write_cmd};
096:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0117,REG_0117,write_cmd};
097:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_011A,REG_011A,write_cmd};
098:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0126,REG_0126,write_cmd};
099:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0129,REG_0129,write_cmd};
100:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0130,REG_0130,write_cmd};
101:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0133,REG_0133,write_cmd};
102:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_013A,REG_013A,write_cmd};
103:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_013D,REG_013D,write_cmd};
104:i2c_ctrl_data	= { NOT_LAST ,	slave_addr	,	REG_ADDR_PAGE	,	8'h02	,	write_cmd	}	;	//change_page 2
105:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0292,REG_0292,write_cmd };//
106:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0293,REG_0293,write_cmd };//
107:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0206,REG_0206,write_cmd};
108:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0208,REG_0208,write_cmd};
109:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0209,REG_0209,write_cmd};
110:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_020A,REG_020A,write_cmd};
111:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_020B,REG_020B,write_cmd};
112:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_020C,REG_020C,write_cmd};
113:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_020D,REG_020D,write_cmd};
114:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_020E,REG_020E,write_cmd};
115:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_020F,REG_020F,write_cmd};
116:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0210,REG_0210,write_cmd};
117:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0211,REG_0211,write_cmd};
118:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0212,REG_0212,write_cmd};
119:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0213,REG_0213,write_cmd};
120:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0214,REG_0214,write_cmd};
121:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0215,REG_0215,write_cmd};
122:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0216,REG_0216,write_cmd};
123:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0217,REG_0217,write_cmd};
124:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0218,REG_0218,write_cmd};
125:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0219,REG_0219,write_cmd};
126:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_021A,REG_021A,write_cmd};
127:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_021B,REG_021B,write_cmd};
128:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_021C,REG_021C,write_cmd};
129:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_021D,REG_021D,write_cmd};
130:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_021E,REG_021E,write_cmd};
131:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_021F,REG_021F,write_cmd};
132:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0220,REG_0220,write_cmd};
133:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0221,REG_0221,write_cmd};
134:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0222,REG_0222,write_cmd};
135:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0223,REG_0223,write_cmd};
136:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0224,REG_0224,write_cmd};
137:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0225,REG_0225,write_cmd};
138:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0226,REG_0226,write_cmd};
139:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0227,REG_0227,write_cmd};
140:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0228,REG_0228,write_cmd};
141:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0229,REG_0229,write_cmd};
142:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_022A,REG_022A,write_cmd};
143:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_022B,REG_022B,write_cmd};
144:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_022C,REG_022C,write_cmd};
145:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_022D,REG_022D,write_cmd};
146:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_022E,REG_022E,write_cmd};
147:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_022F,REG_022F,write_cmd};
148:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0230,REG_0230,write_cmd};
149:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0231,REG_0231,write_cmd};
150:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0232,REG_0232,write_cmd};
151:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0233,REG_0233,write_cmd};
152:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0234,REG_0234,write_cmd};
153:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0235,REG_0235,write_cmd};
154:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0236,REG_0236,write_cmd};
155:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0237,REG_0237,write_cmd};
156:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0238,REG_0238,write_cmd};
157:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0239,REG_0239,write_cmd};
158:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_023A,REG_023A,write_cmd};
159:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_023B,REG_023B,write_cmd};
160:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_023C,REG_023C,write_cmd};
161:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_023D,REG_023D,write_cmd};
162:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_023E,REG_023E,write_cmd};
163:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_023F,REG_023F,write_cmd};
164:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_024A,REG_024A,write_cmd};
165:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_024B,REG_024B,write_cmd};
166:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_024C,REG_024C,write_cmd};
167:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0253,REG_0253,write_cmd};
168:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0254,REG_0254,write_cmd};
169:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0255,REG_0255,write_cmd};
170:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_025C,REG_025C,write_cmd};
171:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_025D,REG_025D,write_cmd};
172:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_025E,REG_025E,write_cmd};
173:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0262,REG_0262,write_cmd};
174:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0263,REG_0263,write_cmd};
175:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0264,REG_0264,write_cmd};
176:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0268,REG_0268,write_cmd};
177:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0269,REG_0269,write_cmd};
178:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_026A,REG_026A,write_cmd};
179:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0295,REG_0295,write_cmd};
180:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0296,REG_0296,write_cmd};
181:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0297,REG_0297,write_cmd};
182:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0299,REG_0299,write_cmd};
183:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02A0,REG_02A0,write_cmd};
184:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02A1,REG_02A1,write_cmd};
185:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02A2,REG_02A2,write_cmd};
186:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02A3,REG_02A3,write_cmd};
187:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02A4,REG_02A4,write_cmd};
188:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02A5,REG_02A5,write_cmd};
189:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02AC,REG_02AC,write_cmd};
190:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02AD,REG_02AD,write_cmd};
191:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02AE,REG_02AE,write_cmd};
192:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02AF,REG_02AF,write_cmd};
193:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02B0,REG_02B0,write_cmd};
194:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_02B1,REG_02B1,write_cmd};
195:i2c_ctrl_data	= { NOT_LAST ,	slave_addr	,	REG_ADDR_PAGE	,	8'h03	,	write_cmd	}	;	//change_page 3
196:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0302,REG_0302,write_cmd};
197:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0303,REG_0303,write_cmd};
198:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0304,REG_0304,write_cmd};
199:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0305,REG_0305,write_cmd};
200:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0306,REG_0306,write_cmd};
201:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0307,REG_0307,write_cmd};
202:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0308,REG_0308,write_cmd};
203:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0309,REG_0309,write_cmd};
204:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_030A,REG_030A,write_cmd};
205:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_030B,REG_030B,write_cmd};
206:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_030C,REG_030C,write_cmd};
207:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_030D,REG_030D,write_cmd};
208:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_030E,REG_030E,write_cmd};
209:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_030F,REG_030F,write_cmd};
210:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0310,REG_0310,write_cmd};
211:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0311,REG_0311,write_cmd};
212:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0312,REG_0312,write_cmd};
213:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0313,REG_0313,write_cmd};
214:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0314,REG_0314,write_cmd};
215:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0315,REG_0315,write_cmd};
216:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0316,REG_0316,write_cmd};
217:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0317,REG_0317,write_cmd};
218:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0318,REG_0318,write_cmd};
219:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0319,REG_0319,write_cmd};
220:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_031A,REG_031A,write_cmd};
221:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_031B,REG_031B,write_cmd};
222:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_031C,REG_031C,write_cmd};
223:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_031D,REG_031D,write_cmd};
224:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_031E,REG_031E,write_cmd};
225:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_031F,REG_031F,write_cmd};
226:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0320,REG_0320,write_cmd};
227:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0321,REG_0321,write_cmd};
228:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0322,REG_0322,write_cmd};
229:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0323,REG_0323,write_cmd};
230:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0324,REG_0324,write_cmd};
231:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0325,REG_0325,write_cmd};
232:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0326,REG_0326,write_cmd};
233:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0327,REG_0327,write_cmd};
234:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0328,REG_0328,write_cmd};
235:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0329,REG_0329,write_cmd};
236:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_032A,REG_032A,write_cmd};
237:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_032B,REG_032B,write_cmd};
238:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_032C,REG_032C,write_cmd};
239:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_032D,REG_032D,write_cmd};
240:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0338,REG_0338,write_cmd};
241:i2c_ctrl_data	= { NOT_LAST ,	slave_addr	,	REG_ADDR_PAGE	,	8'h04	,	write_cmd	}	;	//change_page 4
242:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0420,REG_0420,write_cmd};
243:i2c_ctrl_data	= { NOT_LAST ,	slave_addr	,	REG_ADDR_PAGE	,	8'h05	,	write_cmd	}	;	//change_page 5
244:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0520,REG_0520,write_cmd};
245:i2c_ctrl_data	= { NOT_LAST ,	slave_addr	,	REG_ADDR_PAGE	,	8'h06	,	write_cmd	}	;	//change_page 6
246:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_063A,REG_063A,write_cmd };//
247:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_063B,REG_063B,write_cmd };//
248:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_063C,REG_063C,write_cmd };//
249:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0608,REG_0608,write_cmd};
250:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0609,REG_0609,write_cmd};
251:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_060A,REG_060A,write_cmd};
252:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_060B,REG_060B,write_cmd};
253:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_060C,REG_060C,write_cmd};
254:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_060D,REG_060D,write_cmd};
255:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_060E,REG_060E,write_cmd};
256:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_060F,REG_060F,write_cmd};
257:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0610,REG_0610,write_cmd};
258:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0611,REG_0611,write_cmd};
259:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0612,REG_0612,write_cmd};
260:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0613,REG_0613,write_cmd};
261:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0615,REG_0615,write_cmd};
262:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0616,REG_0616,write_cmd};
263:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0617,REG_0617,write_cmd};
264:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0618,REG_0618,write_cmd};
265:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0619,REG_0619,write_cmd};
266:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_061A,REG_061A,write_cmd};
267:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_061B,REG_061B,write_cmd};
268:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_061C,REG_061C,write_cmd};
269:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_061D,REG_061D,write_cmd};
270:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_061E,REG_061E,write_cmd};
271:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_061F,REG_061F,write_cmd};
272:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0620,REG_0620,write_cmd};
273:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0621,REG_0621,write_cmd};
274:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0622,REG_0622,write_cmd};
275:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0623,REG_0623,write_cmd};
276:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0624,REG_0624,write_cmd};
277:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0625,REG_0625,write_cmd};
278:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0626,REG_0626,write_cmd};
279:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0627,REG_0627,write_cmd};
280:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0628,REG_0628,write_cmd};
281:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0629,REG_0629,write_cmd};
282:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_062A,REG_062A,write_cmd};
283:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_062B,REG_062B,write_cmd};
284:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_062C,REG_062C,write_cmd};
285:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_062E,REG_062E,write_cmd};
286:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_062F,REG_062F,write_cmd};
287:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0632,REG_0632,write_cmd};
288:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0633,REG_0633,write_cmd};
289:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0634,REG_0634,write_cmd};
290:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0635,REG_0635,write_cmd};
291:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0636,REG_0636,write_cmd};
292:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_063A,REG_063A,write_cmd};
293:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_063B,REG_063B,write_cmd};
294:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_063C,REG_063C,write_cmd};
295:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_063D,REG_063D,write_cmd};
296:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_063E,REG_063E,write_cmd};
297:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0688,REG_0688,write_cmd};
298:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0689,REG_0689,write_cmd};
299:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_068A,REG_068A,write_cmd};
300:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_068B,REG_068B,write_cmd};
301:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_068C,REG_068C,write_cmd};
302:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_068D,REG_068D,write_cmd};
303:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_069B,REG_069B,write_cmd};
304:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_069C,REG_069C,write_cmd};
305:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_069D,REG_069D,write_cmd};
306:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_069E,REG_069E,write_cmd};
307:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_069F,REG_069F,write_cmd};
308:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06A0,REG_06A0,write_cmd};
309:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06A1,REG_06A1,write_cmd};
310:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06A2,REG_06A2,write_cmd};
311:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06A4,REG_06A4,write_cmd};
312:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06A5,REG_06A5,write_cmd};
313:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06A6,REG_06A6,write_cmd};
314:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06AC,REG_06AC,write_cmd};
315:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06AD,REG_06AD,write_cmd};
316:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06AE,REG_06AE,write_cmd};
317:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06B1,REG_06B1,write_cmd};
318:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_06B2,REG_06B2,write_cmd};
319:i2c_ctrl_data	= { NOT_LAST ,slave_addr, REG_ADDR_PAGE,	8'h07	,	write_cmd	}	;	//change_page 7
320:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0737,REG_0737,write_cmd};//
321:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_073B,REG_073B,write_cmd};
322:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0709,REG_0709,write_cmd};
323:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_070A,REG_070A,write_cmd};
324:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_070B,REG_070B,write_cmd};
325:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_070C,REG_070C,write_cmd};
326:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_070D,REG_070D,write_cmd};
327:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_070E,REG_070E,write_cmd};
328:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_070F,REG_070F,write_cmd};
329:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0710,REG_0710,write_cmd};
330:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0711,REG_0711,write_cmd};
331:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0712,REG_0712,write_cmd};
332:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0713,REG_0713,write_cmd};
333:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0714,REG_0714,write_cmd};
334:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0716,REG_0716,write_cmd};
335:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0717,REG_0717,write_cmd};
336:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0718,REG_0718,write_cmd};
337:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0719,REG_0719,write_cmd};
338:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_071A,REG_071A,write_cmd};
339:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_071B,REG_071B,write_cmd};
340:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_071C,REG_071C,write_cmd};
341:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_071D,REG_071D,write_cmd};
342:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_071E,REG_071E,write_cmd};
343:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_071F,REG_071F,write_cmd};
344:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0720,REG_0720,write_cmd};
345:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0721,REG_0721,write_cmd};
346:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0722,REG_0722,write_cmd } ;
347:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0723,REG_0723,write_cmd } ;
348:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0724,REG_0724,write_cmd } ;
349:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0725,REG_0725,write_cmd } ;
350:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0726,REG_0726,write_cmd } ;
351:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0727,REG_0727,write_cmd } ;
352:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0728,REG_0728,write_cmd } ;
353:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0729,REG_0729,write_cmd } ;
354:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_072A,REG_072A,write_cmd } ; 
355:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_072B,REG_072B,write_cmd};
356:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_072C,REG_072C,write_cmd};
357:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_072D,REG_072D,write_cmd};
358:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_072F,REG_072F,write_cmd};
359:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0730,REG_0730,write_cmd};
360:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0733,REG_0733,write_cmd};
361:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0734,REG_0734,write_cmd};
362:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0735,REG_0735,write_cmd};
363:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0736,REG_0736,write_cmd};
364:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_073B,REG_073B,write_cmd};
365:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_073C,REG_073C,write_cmd};
366:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_073D,REG_073D,write_cmd};
367:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_073C,REG_073C,write_cmd};
368:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_073D,REG_073D,write_cmd};
369:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_073E,REG_073E,write_cmd};
370:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_073F,REG_073F,write_cmd};
371:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0788,REG_0788,write_cmd};
372:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0789,REG_0789,write_cmd};
373:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_078A,REG_078A,write_cmd};
374:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_078B,REG_078B,write_cmd};
375:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_078C,REG_078C,write_cmd};
376:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_078D,REG_078D,write_cmd};
377:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_079B,REG_079B,write_cmd};
378:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_079C,REG_079C,write_cmd};
379:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_079D,REG_079D,write_cmd};
380:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_079E,REG_079E,write_cmd};
381:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_079F,REG_079F,write_cmd};
382:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07A0,REG_07A0,write_cmd};
383:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07A1,REG_07A1,write_cmd};
384:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07A2,REG_07A2,write_cmd};
385:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07A4,REG_07A4,write_cmd};
386:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07A5,REG_07A5,write_cmd};
387:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07A6,REG_07A6,write_cmd};
388:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07AC,REG_07AC,write_cmd};
389:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07AD,REG_07AD,write_cmd};
390:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07AE,REG_07AE,write_cmd};
391:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07B1,REG_07B1,write_cmd};
392:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_07B2,REG_07B2,write_cmd};
393:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	REG_ADDR_PAGE	,	8'h08	,	write_cmd	}	;	//change_page 8
394:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0822,REG_0822,write_cmd};
395:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0831,REG_0831,write_cmd};
396:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	REG_ADDR_PAGE	,	8'h09	,	write_cmd	}	;	//change_page 9
397:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_090E,REG_090E,write_cmd};
398:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0949,REG_0949,write_cmd};
399:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_094A,REG_094A,write_cmd};
400:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	REG_ADDR_PAGE	,	8'h0b	,	write_cmd	}	;	//change_page b
401:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0B44,REG_0B44,write_cmd};
402:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0B47,REG_0B47,write_cmd};
403:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0B48,REG_0B48,write_cmd};
404:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	REG_ADDR_PAGE	,	8'h0c	,	write_cmd	}	;	//change_page C
405:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0C03,REG_0C03,write_cmd};
406:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0C09,REG_0C09,write_cmd};
407:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0C0A,REG_0C0A,write_cmd};
408:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0C0B,REG_0C0B,write_cmd};
409:i2c_ctrl_data = { NOT_LAST ,slave_addr, REG_ADDR_0C0C,REG_0C0C,write_cmd};   
410:i2c_ctrl_data	= { NOT_LAST ,slave_addr, REG_ADDR_PAGE	,	8'h06,write_cmd}	;	   //page6 Start configurationpostamble 
411:i2c_ctrl_data	= { NOT_LAST ,slave_addr,     8'h14	      ,	8'h01,write_cmd}	;	
412:i2c_ctrl_data	= { NOT_LAST ,slave_addr, REG_ADDR_PAGE   ,	8'h07	,	write_cmd	}	;	//page7
413:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	8'h15	      ,	8'h01,	write_cmd	}	;	
414:i2c_ctrl_data	= { NOT_LAST ,slave_addr, REG_ADDR_PAGE	,	8'h00	,	write_cmd	}	;	//page0
415:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	8'h1c	      ,	8'h01,	write_cmd	}	;	
416:i2c_ctrl_data	= { NOT_LAST ,slave_addr, REG_ADDR_PAGE	,	8'h0B	,	write_cmd	}	;	//pageb
417:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	8'h24	      ,	8'hC3	,	write_cmd	}	;	
418:i2c_ctrl_data	= { NOT_LAST ,slave_addr	,	8'h25	      ,	8'h02	,	write_cmd	}	;	 //End configuration postamble 


//	/* Start configuration postamble */
//	{ 0x0614, 0x01 },//0x0614 0 S BW_UPDATE_PLLC
//	{ 0x0715, 0x01 },//0x0715 0 S BW_UPDATE_PLLD
//	{ 0x001C, 0x01 },
//	{ 0x0B24, 0xC3 },
//	{ 0x0B25, 0x02 },
//	/* End configuration postamble */

                                                     
    endcase	
  end 




edge_detector u1(

.iCLK      (iCLK),
.iRST_n        (iRST_n),
.iIn          (iI2C_CONTROLLER_CONFIG_DONE),
.oFallING_EDGE(i2c_controller_config_done_n),
.oRISING_EDGE (i2c_controller_config_done_p),
.oFallING_EDGE_r(i2c_controller_config_done_n_r)
);

// johnny modify  oONE_CLK_CONFIG_DONE on 2017.12.15
always@(posedge iCLK or negedge iRST_n)
	begin
		if (!iRST_n)
			begin
				oController_Ready    <= 1'b1;
				oONE_CLK_CONFIG_DONE <= 1'b0;
			end
		else if (i2c_reg_state == `REG_NUM+1)	
			begin
				oController_Ready    <= 1'b1;
				oONE_CLK_CONFIG_DONE <= 1'b1;
			end
		else if (i2c_reg_state >0)
			begin
				oController_Ready    <= 1'b0;
				oONE_CLK_CONFIG_DONE <= 1'b0;
			end
	end

//-----SI5397
assign access_next_i2c_reg_cmd =  ((iI2C_CONTROLLER_STATE == 1'b0 ) && ( DELAY==0) &&
                                  ((i2c_reg_state <= `REG_NUM) && (i2c_reg_state > 0))
                                  ) ? 1'b1 : 1'b0;
assign access_i2c_reg_start     = ( (iENABLE == 1'b1) && (iI2C_CONTROLLER_STATE == 1'b0)) ? 1'b1 : 1'b0;
//-------
						
							

endmodule 
//===






























